package lbl.akenti.gen.controller;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.Color;

/**
  * Allows the user to specify the cache time for the 
  * <i>policy certificate</i> being generated.
  *
  * @see LBLTextFieldFactory
  */
public class LBLCacheFrame 
  extends LBLFrame implements DocumentListener {

  private JTextField cacheField;

  /** 
    * Builds this frame. 
    *
    * @param title of this frame
    * @param instructions displayed in the top portion of this frame  
    */
  public LBLCacheFrame(String title, String instructions) {
    super(title, instructions);

    JLabel label = new JLabel("Cache Time");
    JPanel contentPane = (JPanel) getContentPane();

    label.setForeground(Color.red);
    cacheField = LBLTextFieldFactory.createNumberField(this);

    contentPane.add(
      wrapComponents(
        label,
        cacheField
      ),
      "Center"
    );
   
    enableBlueButton("Next");
  }

  /**
    */
  public void changedUpdate(DocumentEvent e)  {
    // do nothing
  }
 
  /**
    * Called when text is inserted in the text field.
    * Enables the <i>next button</i>. 
    *
    * @see #doEnable()
    */
  public void insertUpdate(DocumentEvent e)  {
    doEnable();
  }

  /**
    * Called when text is removed from the text field.
    * May disable the <i>next button</i> off. 
    *
    * @see #doEnable()
    */
  public void removeUpdate(DocumentEvent e) {
    doEnable();
  }

  /**
    * Turns the <i>next button</i> on or off. 
    */
  private void doEnable() {
  }

  /**
    * Returns the cache time.
    */
  public long getCacheTime() {
    return Long.valueOf(getCacheTimeAsString()).longValue();
  }

  /**
    * Sets the cache time.
    */
  public void setCacheTime(long cacheTime) {
    cacheField.setText(String.valueOf(cacheTime));
  }

  /**
    * Returns the cache time as a string.
    */
  public String getCacheTimeAsString() {
    String temp = cacheField.getText();
    
    return temp.equals("") ? "-1" : temp;
  }

  /**
    */
  public void clear() {
    cacheField.setText("");
  }
}
