package lbl.akenti.gen.controller;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JList;
import javax.swing.DefaultListModel;
import javax.swing.JScrollPane;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import lbl.akenti.certs.elements.Directory;

/**
  * This option pane allows a user to view and edit directories.
  *
  * @see javax.swing.JPanel
  */
public class LBLDirPane 
  extends JOptionPane
  implements ListDataListener, ListSelectionListener, ActionListener {
  private final static String DIR_PANEL_TITLE = "Directory Builder";

  private String[] protocols;

  private JList dirList;
  private ListSelectionListener listSelectionListener;
  private ListDataListener listDataListener;

  private LBLBuildDirPane optionPane;

  private JButton remButton = new JButton("Remove"); 

  /** 
    * Constructs this widget. 
    */
  public LBLDirPane() {
    this(null);
  }
  
  /**
    * Constructs this widget. 
    */
  public LBLDirPane(String dirsLabel) {
    this(dirsLabel, null);
  }

  /** 
    * Constructor 
    */
  public LBLDirPane(String dirsLabel, String []protocols) {
    setLayout(new BorderLayout());

    if (protocols != null) {
      this.protocols = protocols;
    } 

    DefaultListModel model = new DefaultListModel();

    model.addListDataListener(this);
    dirList = new JList(model);
    dirList.setSelectionMode(2);
    dirList.addListSelectionListener(this);
	
    JPanel temp = new JPanel(new BorderLayout());
    JLabel label;

    if (dirsLabel != null) {
      label = new JLabel(dirsLabel);
    } else {
      label = new JLabel("Directories");
    }

    label.setForeground(Color.red);
    temp.add(label, "North");
    temp.add(new JScrollPane(dirList), "Center");

    JButton addButton = new JButton("Add");

    addButton.setActionCommand("add");
    addButton.addActionListener(this);
    LBLButtonPanel.enableBlueButton(addButton);

    remButton.setActionCommand("remove");
    remButton.addActionListener(this);
    LBLButtonPanel.disableRedButton(remButton);

    add(
      WrapUtil.wrapComponents(
        temp, 
        WrapUtil.wrapVertical(addButton, remButton)
      ), 
      "Center"
    );
  }
  
  /**
    */
  public void setListSelectionListener(ListSelectionListener listSelectionListener) {
    this.listSelectionListener = listSelectionListener;
  }

  /**
    */
  public void setListDataListener(ListDataListener listDataListener) {
    this.listDataListener = listDataListener;
  }

  /**
    */
  private LBLBuildDirPane getDirOptionPane() {
    if (optionPane == null) {
      optionPane = new LBLBuildDirPane(
        null,
        protocols
      );
    }

    return optionPane;
  }

  /**
    */
  public void actionPerformed(ActionEvent event) {
    if (event.getActionCommand().equals("add")) {
      LBLBuildDirPane optionPane = getDirOptionPane();

      optionPane.clear();
      optionPane.createDialog(this, DIR_PANEL_TITLE).setVisible(true);

      Object val = optionPane.getValue();

      optionPane.setValue(null);

      if (val != null && val != GenConsts.AK_CANCEL_VALUE) {
        addDir(new Directory((String) val));
      }
    } else { // REMOVE 
      int []indices = dirList.getSelectedIndices();
      DefaultListModel model = (DefaultListModel) dirList.getModel();

      for (int i = indices.length - 1; i > -1; --i) {
        model.removeElementAt(indices[i]);
      }
    }
  }

  /**
    */
  public void valueChanged(ListSelectionEvent e) {
    doEnable();

    if (listSelectionListener != null) {
      listSelectionListener.valueChanged(e);
    }
  }
    
  /**
    */
  public void contentsChanged(ListDataEvent e) {
    if (listDataListener != null) {
      listDataListener.contentsChanged(e);
    }
  }

  /**
    */
  public void intervalAdded(ListDataEvent e) {
    if (listDataListener != null) {
      listDataListener.intervalAdded(e);
    }
  }

  /**
    */
  public void intervalRemoved(ListDataEvent e)  {
    doEnable();

    if (listDataListener != null) {
      listDataListener.intervalRemoved(e);
    }
  }

  private void doEnable() {
    if (dirList.getSelectedIndex() != -1) {
      LBLButtonPanel.enableRedButton(remButton);
    } else {
      LBLButtonPanel.disableRedButton(remButton);
    }
  }

  /**
    */
  public void clear() {
    DefaultListModel model = (DefaultListModel) dirList.getModel();

    model.clear();
  }
   
  /**
    */
  public int numOfDirs() {
    DefaultListModel model = (DefaultListModel) dirList.getModel();

    return model.getSize();
  }

  /**
    */
  public int setDirs(Directory[] dirs) {
    DefaultListModel model = (DefaultListModel) dirList.getModel();

    model.clear();
    return addDirs(dirs);
  }

  /**
    */
  public int addDirs(Directory[] dirs) {
    DefaultListModel model = (DefaultListModel) dirList.getModel();
    int added = 0;
    int size = model.getSize();

    dirList.clearSelection();

    for (int i = 0; i < dirs.length; ++i) {
      int index = model.indexOf(dirs[i]);

      if (index != -1) {
        dirList.addSelectionInterval(index, index);
      } else {
        model.addElement(dirs[i]);
        dirList.addSelectionInterval(size, size++);
        ++added;
      }
    }

    return added;
  }

  public boolean addDir(Directory dir) {
    return addDirs(new Directory[] { dir }) != 0;
  }

  /**
    */
  public Directory[] getDirs() {
    DefaultListModel model = (DefaultListModel) dirList.getModel();
    Directory[] dirs = new Directory[model.getSize()];
    
    model.copyInto(dirs);
    return dirs;
  }
}
