package lbl.akenti.gen.controller;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.DefaultListModel;
import javax.swing.JTextField;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.util.ArrayList;
import java.net.URL;
import java.net.MalformedURLException;
import java.security.cert.X509Certificate;
import javax.naming.NamingException;

import lbl.akenti.util.LDAPSearchUtil;
import lbl.akenti.gen.common.GenCa;

/**
  * Allows to select one or more entities and returns
  * the selected entities x509 certificates.
  */
public class LBLLdapPanel 
  extends JOptionPane
  implements ActionListener {

  private JTextField baseTextField = new JTextField();
  private JTextField cnTextField = new JTextField(25);
  private JComboBox orgUnit = new JComboBox();
  private JPanel searchPanel;

  private URL url;
  
  public LBLLdapPanel(String instructions) {
    setLayout(new BorderLayout(5, 5));

    JLabel labels[] = { 
      new JLabel("SearchBase"),
      new JLabel("Organizational Unit OU"),
      new JLabel("CommonName CN")
    };

    for(int i = 0; i < labels.length; i++) {
      labels[i].setForeground(Color.red);
    }

    baseTextField.setEditable(false);

    JButton searchButton = new JButton("Search");

    searchButton.setBackground(Color.blue);
    searchButton.setForeground(Color.white);
    searchButton.setActionCommand("search");
    searchButton.addActionListener(this);
	
    JComponent fields[] = {
       baseTextField, 
       orgUnit, 
       WrapUtil.wrapComponents(cnTextField, searchButton)
    };

    searchPanel = WrapUtil.wrapComponents(labels,fields);
    add(searchPanel, "Center");
    
    JButton cancelButton = new JButton("Cancel");
    JPanel cancelPanel = new JPanel();

    cancelPanel.add(cancelButton);
    cancelButton.setActionCommand("cancel");
    cancelButton.addActionListener(this);
    add(cancelPanel, "South");
  }    
    
  /**
    */
  public void setURL(URL url) {
    String base = url.getFile();
    baseTextField.setText(url.getFile().substring(1));
    orgUnit.removeAllItems();
    cnTextField.setText("*");
    this.url = url;
  }

  /**
    */
  public URL getURL() {
    return url;
  }

  /**
    */
  public boolean loadOrgUnits() {
    String base = baseTextField.getText(); 

    if (base.indexOf("OU") != -1) {
      return true;
    }

    String[] results;

    try {
      results = LDAPSearchUtil.loadOrgUnits(url);
    } catch(NamingException ex) {
      LBLDialogFactory.showInfoDialog(
        this,
        "Ldap error",
        "Unable to load org units ... " + ex
      );

      return false;
    }
            
    for (int i = 0; i < results.length; ++i) {
      orgUnit.addItem(results[i]);
    }

    return true;
  }

  private static String PRINCIPAL_MESSAGE =
    "<html><font size=+1 color=blue>Looking for X509 certificates...</font>\n\n" +
    "<li><font color=blue>Select one more principals" +
    "<li><font color=blue>Click <i>ok</i> to load x509 certificates"; 

  /**
    *
    */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("search")){
      try {
        StringBuffer sb = new StringBuffer();
        String org = (String) orgUnit.getSelectedItem();

        if (org != null && org.length() != 0) {
          sb.append("OU=");
          sb.append(org);
          sb.append(',');
        }

        sb.append(baseTextField.getText());

        String base = sb.toString();
        String temp = cnTextField.getText();

        if (temp.length() == 0) {
          temp = "*";
        }

        String filter = "CN=" + temp;
     
        URL tempURL = new URL("ldap", url.getHost(), url.getPort(), base);
        String[] userList = LDAPSearchUtil.loadCns(tempURL, filter);

        if (userList.length == 0) {
          LBLDialogFactory.createInfoDialog(
            this,
            "Searching Failed",
            "<html><font color=blue>No user found<BR>" + "BASE:" + base + "<BR>FILTER:" + filter
          ).setVisible(true);
            
          return;
        }

        Object[] aps = LBLDialogFactory.showListDialog(
          this, 
          "Principal Selection Dialog", 
          PRINCIPAL_MESSAGE,
          userList
        );

        if (aps == null) {
          setValue(GenConsts.AK_CANCEL_VALUE);
          return;
        }

        ArrayList certList = new ArrayList();

        for (int i = 0; i < aps.length; ++i) {
          // TODO:
          // hack around the ldap tool runnind doe's ca
          // filter = "cn=" + aps[i];
          filter = "cn=*" + aps[i];
             
          X509Certificate[] certs = LDAPSearchUtil.loadX509Certs(tempURL, filter);

          if (certs.length != 0) {
            certList.add(certs[0]);
          } else {
            LBLDialogFactory.showInfoDialog( 
              this,
              "Ldap Search Error",
              "<html><font color=blue>No X509 cert found for<BR>" + aps[i] 
            );
          }
        }

        if (certList.size() != 0) {
          setValue(certList.toArray(new X509Certificate[0]));
        }
      } catch(MalformedURLException ex) {
        throw new RuntimeException(ex.getMessage());
      } catch(NamingException ex) {
        LBLDialogFactory.showInfoDialog( 
          this,
          "Ldap Search Error",
          "<html><font color=blue>Searching failed ...<BR>" + ex.getMessage()
        );
      }
    } else { // cmd = "cancel"
      setValue(GenConsts.AK_CANCEL_VALUE);
    } 
  }
}

