package lbl.akenti.gen.controller;

import javax.swing.JTextArea;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import java.awt.Color;
import java.awt.Dimension;

import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.gen.common.HtmlCertWriter;

/**
  * A sub-class of LBLFrame, used to allow the issuer to 
  * review the information that will be in the certficate.
  * This frame is used by both the Attribute and Use-condition 
  *
  * Certificate generators.
  */
public class LBLReviewFrame extends LBLFrame { 
  private JLabel certificateLabel;
  private AkentiCertificate cert;

  /** constructor
    * Create an frame with empty text area in the middle to 
    * display the certficate in.
    *
    * @param title title of the frame
    * @param instructions header/instruction to be displayed at top of frame
    */
  public LBLReviewFrame(String TITLE, String INSTRUCTIONS) {
    super(TITLE, INSTRUCTIONS);
    certificateLabel = new JLabel("", JLabel.LEFT);
    certificateLabel.setBackground(Color.white);
    certificateLabel.setOpaque(true);
    certificateLabel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

    JScrollPane sPane =  new JScrollPane(certificateLabel);

    sPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS); 
    sPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS); 
    sPane.setPreferredSize(new Dimension(400, 350));
    getContentPane().add(sPane, "Center");
  }

  /**
    */
  public void setCertificate(AkentiCertificate cert) {
    this.cert = cert;
    certificateLabel.setText(new HtmlCertWriter().toHtml(cert));
  }

  /**
    */
  public AkentiCertificate getCertificate() {
    return cert;
  }

  /** 
    * Method must be defined but it's never used since 
    * the text in this frame is reset everytime it is visited
    * by hitting the next button.
    */
  public void clear () {
    cert = null;
    certificateLabel.setText("");
  }
}
