package lbl.akenti.gen.controller.attr;


/**
  * A class to contain the constants used by the attrGen class
  */
public interface AtConsts {

    // Indices for accessing the controllers frames 
    public static final int RESOURCE_INDEX  = 0;
    public static final int ATTRIBUTE_INDEX = 1;
    public static final int ISSUER_INDEX    = 2;
    public static final int USER_INDEX      = 3;
    public static final int TIME_INDEX      = 4;
    public static final int REVIEW_INDEX    = 5;   
    public static final int SIGN_INDEX      = 6;

    static final String[] TITLES =  {
	"Resource Info",
	"Attribute Value Pairs",
	"Attribute Issuers",
	"User Info",
	"Certificate Lifetime",
	"Certificate Content",
        "Information for Signing And Saving the Certificate"
    };
 
    // Resource Frame
    public static String RESOURCE_HEADER =
      "<html><font size=+1 color=blue>"+TITLES[RESOURCE_INDEX] +
      "<li><font color=blue>Enter the name of the resource.<i>" +
      "<li><font color=blue>Enter the base url."
    ; // end of header

    // Attribute Frame	
    public static String ATTRIBUTE_HEADER =
      "<html><font size=+1 color=blue>" + TITLES[ATTRIBUTE_INDEX] +
      "<li><font color=blue>Enter an attribute/value pair." + 
      "<li><font color=blue>You can use the hints if appropriate.";
    ; // end of header

    // Issuer Frame
    public static String ISSUER_HEADER =
      "<html><font size=+1 color=blue>" + TITLES[ISSUER_INDEX] +
      "<li><font color=blue>Select Attribute Issuer (yourself) from this" +
      " list, provided<br> the hints are up to date."
    ; // end of header

    // User Frame
    public static String USER_HEADER =
      "<html><font size=+1 color=blue>Akenti Principal Panel" +
      "<li><font color=blue>Enter distinguished name of the  subject to whom <br>" +
      "this attribute certificate will apply." + 
      "<li><font color=blue>Enter distinguished</i> name of the principal's CA." +
      "<li><font color=blue>You can click <i>search</i> to search for x509 certs."
    ; // end of header

    // Time Frame
    public static String TIME_HEADER =
      "<html><font size=+1 color=blue>" + TITLES[TIME_INDEX] +"\n\n" +
      "<li><font color=blue>Choose how long the certificate will be valid for."
    ; // end of header

    // Review Frame
    public static String REVIEW_HEADER =
      "<html><font size=+1 color=blue>Review Your Attribute Certificate" +
      "<li><font color=blue>Click <i>next</i> when ready.";
    ; // end of header

    // Sign And Save Frame
    public static String SIGN_HEADER =
      "<html><font size=+1 color=blue>"+TITLES[SIGN_INDEX] +
      "<li><font color=blue>Enter your <font color=red>private key</font> file path " +
      "and <font color=red>passphase</font> if <br> did not already provide." +
      "<li><font color=blue>Enter the full path of the file where" +
      " the newly <br>generated Attribute Certificate will be stored."
    ; // end of header

    // CONVENIENT ARRAY OF HEADERS
    public static String[] HEADERS = {
      RESOURCE_HEADER,
      ATTRIBUTE_HEADER,
      ISSUER_HEADER,
      USER_HEADER,
      TIME_HEADER,
      REVIEW_HEADER,
      SIGN_HEADER
    };

    /*
     * Help messages. Messages are indexed in the following order:
     * RsrcMess, PolMess, IssMess, UsMess, TimeMess, RevMess
     */

    static final String [] helpMess = {
        // Resource Frame Help message
        "The resouce can be hierarchical or flat. " +
        "The relative resource name is appended to the base url " +
        "which the generator uses to find information about this resource. " +
        "\n\nA base url, different from the default one, " +
        "can be chosen using the Create button, or be typed in. " +
        "If you are not sure of the correct relative resource name " +
        "clicking on the Browse button will give you a file browse window  " +
        "that server controls. The Policy Definition Server returns the contents of the " +
        "authorization file (which lists the stakeholders for the resource) " +
        "and the resource attributes file which contains the attributes, " +
        "values, and actions that are defined for the resource.\n\n" +
        "The host and port constitute the address of the Policy Definition Server." +
        "\nAll default values are read from the AkentiGen.conf file.",

 	// Value Frame help Message
	"Attributes and values that are meaningful to a resource are " +
	"defined in the resource attributes file for that resource.",

	// IssuerFrame help message 
	"Only these users are recognized attribute issuers for this resource "+
	" and value. If your name does not appear here, contact one of the " +
	"stakeholders or the administrator for the resource tree.",

	// UserFrame help message 
        "If you don't know the exact name of the principal's distinguished " +
        "and CA's distinguished name, click the search button.",

	// TimeFrame help message 
	"The certificate valdity period with begin with the current time " + 
        "and end after curent time plus the duration chosen here.",

	// ReviewFrame help message 
	"At this point you can use the Back button to go back to any " +
	"previous window where you can re-enter any information that "+
	"you may want to correct.", 

	// Sign and Save frame help message
        "Your keyfile has to be a pkcs (.p12) file. The Certificate file " +
        "should be located in a directory that is accessible by the akenti server."};			 
}
