package lbl.akenti.gen.controller.pol;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.border.Border;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.net.URL;
import java.net.MalformedURLException;

import lbl.akenti.gen.controller.GenConsts;
import lbl.akenti.gen.controller.WrapUtil;
import lbl.akenti.gen.controller.LBLDialogFactory;
import lbl.akenti.gen.controller.LBLButtonPanel;
import lbl.akenti.gen.controller.LBLTextFieldFactory;
import lbl.akenti.gen.controller.LBLDirPane;
import lbl.akenti.gen.controller.LBLDirField;
import lbl.akenti.gen.controller.LBLLabelFactory;

import java.security.cert.X509Certificate;

import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;

/**
  */
public 
class CAInfoPanel 
  extends JOptionPane
  implements ActionListener, ListDataListener {

  private static final Color B_COLOR = new Color (255, 255, 204);

  // Resource Frame
  private static String CA_INFO_PANEL_HEADER  =
    "<html><font size=+1 color=blue>CA Info Editor</font>\n\n" +
    "<li><font color=blue>Must provide an X509 chain for the CA" +
    "<li><font color=blue>Should provide at least one ID directory\n" +
    "<li><font color=blue>Can provide CRL directories\n"
    ; // end of header

  private static String HELP_MESSAGE = 
    "Provide trusted certificate authority information including an X509 " +
    "chain of the trusted CAs, at least one ID directory and optional CRL " +
    "directories.  An x509 chain consists of one or more x509 certificates. " +
    "An ID directory is the directory in which the CA " +
    "publishes the certificates that it issues.  A CRL directory is the " +
    "directory in which the CA publishes the certificate revocation list.";

  private CACertsList certsList;
  private LBLDirPane idDirPane;
  private LBLDirPane crlDirPane;

  private JButton helpButton = new JButton("Help");
  private JButton okButton = new JButton("OK");
  private JButton cancelButton = new JButton("CANCEL");;

  /** 
    * Builds this frame.
    *
    */
  public CAInfoPanel() {
    this(null);
  }

  /** 
    * Builds this frame.
    *
    */
  public CAInfoPanel(String instructions) {
    this(instructions, null);
  }

  /** 
    * Builds this frame.
    *
    */
  public CAInfoPanel(String instructions, String []prots) {
    setLayout(new BorderLayout(5, 5));

    if (instructions == null) {
      instructions = CA_INFO_PANEL_HEADER;
    }

    add(LBLLabelFactory.createHtmlLabel(instructions) , "North");

    certsList = new CACertsList();
    certsList.setListDataListener(this);

    idDirPane = new LBLDirPane("X509 Identity Directories", prots);
    idDirPane.setListDataListener(this);

    crlDirPane = new LBLDirPane("X509 CRL Directories", prots);
    crlDirPane.setListDataListener(this);

    add(
      WrapUtil.wrapVertical(
        new JComponent[] { certsList, idDirPane, crlDirPane }
      ),
      "Center"
    );

    JPanel buttonPanel = new JPanel(); 

    helpButton.setActionCommand("help");
    helpButton.addActionListener(this);
    buttonPanel.add(helpButton);
      

    okButton.setActionCommand("ok");
    okButton.addActionListener(this);
    okButton.setActionCommand("ok");
    buttonPanel.add(okButton);
      
    cancelButton.addActionListener(this);
    cancelButton.setActionCommand("cancel");
    buttonPanel.add(cancelButton);
    add(buttonPanel, "South");
    doOkEnable(false);
  }

  /**
    */
  public void setOKLabel(String label) {
    okButton.setLabel(label);
  }

  /**
    *
    */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("help")) {
      showDialog("Info", HELP_MESSAGE); 
    } else if (cmd.equals("ok")) {
      setValue(getCAInfo());
    } else if (cmd.equals("cancel")) {
      setValue(GenConsts.AK_CANCEL_VALUE);
    } else {
      throw new RuntimeException(cmd + " not supported");
    }
  }
 
  protected void showDialog(String title, String info) {
    LBLDialogFactory.createInfoDialog(
      this,
      title,
      (info == null? "" : info)
    ).setVisible(true);
  }
 
  /**
    */
  public void contentsChanged(ListDataEvent e) {
    doEnable();
  }

  /**
    */
  public void intervalAdded(ListDataEvent e) {
    doEnable();
  }

  /**
    */
  public void intervalRemoved(ListDataEvent e)  {
    doEnable();
  }

  /**
    */
  public void doEnable() {
    doOkEnable(certsList.numOfCerts() != 0);
  }

  /**
    */
  private void doOkEnable(boolean on) {
    if (on) {
      enableRedButton(okButton);
    } else {
      disableRedButton(okButton);
    }
  }

  /**
    */
  public void clear() {
    certsList.clear();
    idDirPane.clear();
    crlDirPane.clear();
  }

  /**
    */
  private void enableBlueButton(JButton button) {
    LBLButtonPanel.enableBlueButton(button);
  }

  /**
    */
  private void disableBlueButton(JButton button) {
    LBLButtonPanel.disableBlueButton(button);
  }

  /**
    */
  private void enableRedButton(JButton button) {
    LBLButtonPanel.enableRedButton(button);
  }

  /**
    */
  private void disableRedButton(JButton button) {
    LBLButtonPanel.disableRedButton(button);
  }

  /**
    */
  public void setCAInfo(CertificateAuthorityInfo caInfo) {
    certsList.setCerts(caInfo.getChain());
    setIDDirs(caInfo.getIdentityDirectories());
    setCRLDirs(caInfo.getCRLDirectories());
    doOkEnable(false);
  }

  /**
    *
    */
  public CertificateAuthorityInfo getCAInfo() {
     return new CertificateAuthorityInfo(
      getCACerts(),
      getIDDirs(),
      getCRLDirs()
    );
  }

  /**
    */
  public X509Certificate[] getCACerts() {
    return certsList.getCerts();
  }

  /**
    */
  public int numOfIDDirs() {
    return idDirPane.numOfDirs();
  }

  /**
    */
  public Directory[] getIDDirs() {
    return idDirPane.getDirs();
  }

  /**
    */
  public boolean addIDDir(Directory dir) {
    return idDirPane.addDir(dir);
  }

  /**
    */
  public void setIDDirs(Directory[] dirs) {
    idDirPane.setDirs(dirs);
  }

  /**
    */
  public int addIDDirs(Directory[] dirs) {
    return idDirPane.addDirs(dirs);
  }

  /**
    */
  public int numOfCRLDirs() {
    return crlDirPane.numOfDirs();
  }

  /**
    */
  public Directory[] getCRLDirs() {
    return crlDirPane.getDirs();
  }

  /**
    */
  public void setCRLDirs(Directory[] dirs) {
    crlDirPane.setDirs(dirs);
  }

  /**
    */
  public boolean addCRLDir(Directory dir) {
    return crlDirPane.addDir(dir);
  }

  /**
    */
  public int addCRLDirs(Directory[] dirs) {
    return crlDirPane.addDirs(dirs);
  }
}
