package lbl.akenti.messaging;

import lbl.akenti.jaxb.AkentiRequestType;
import javax.xml.bind.JAXBException;

/**
  * Encapsulates the various requests that are handed to the 
  * <i>Akenti Server</i>.                                        
  *
  * Each request contains a client-generated unique id that can be useful 
  * when clients are sending or keeping track of multiple requests.
  * 
  * @author Abdelilah Essiari.
  */

public abstract
class AkentiRequest {
  private int id;

 /**
   * Constructs a request.
   * @param id the id (which will match the id of the response)
   */
  protected  AkentiRequest(int id) {
    if (id < 0) {
      throw new IllegalArgumentException ();
    }

    this.id = id;
  }

 /**
   * Returns the client-generated unique id.
   * @return the id
   */
  public int getID() {
    return id;
  }

 /**
   * Constructs this <code>AkentiRequest</code> from an <code>AkentiRequestType</code>.
   * @param impl the AkentiRequestType
   */
  AkentiRequest(AkentiRequestType impl) {
    id = impl.getID();
  }
  
 /**
   * TODO:
   *
   * JAXB STUFF
   * HERE WE ARE BREAKING THE RULES TO MAKE
   * JAXB/SOAP/C++ PARSING HAPPY.
   * TO BE REVISITED.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the AkentiRequestType whose info matches that of this object
   */
  public final AkentiRequestType toJAXBImpl(boolean type) throws JAXBException {
    AkentiRequestType impl = doToJAXBImpl(type);

    impl.setID(getID());
    return impl;
  }

 /**
   * JAXB STUFF
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the AkentiRequestType whose info matches that of this object
   */
  abstract protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException;
}
