package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.CheckPolicyRequestType;

/**
 * Holds the inputs to the check section of the <code> Akenti
 * Authorization Engine</code>.  This differs from CheckPolicy
 * in that it returns certificates even if they are malformed
 * or if errors are encountered.
 * @author Tommaso Sciortino
 */
final public
class CheckPolicyRequest extends AkentiRequest {
  private String resource;

 /**
   * Constructs a <code>CheckPolicyRequest</code>.
   * @param uid the id
   * @param resource the resource
   */
  public CheckPolicyRequest(int uid, String resource) {
    super(uid);

    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.resource = resource;
  }

 /**
   * Returns the name of the resource.
   * @return the resource
   */
  public String getResource() {
    return resource;
  }

 /**
   * Constructs a <code>CheckPolicyRequest</code> from a <code>CheckPolicyRequestType</code>.
   * @param impl the CheckPolicyRequestType
   */
  CheckPolicyRequest(CheckPolicyRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * Produces a CheckPolicyRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the CheckPolicyRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    CheckPolicyRequestType impl = type? Util.getObjectFactory().createCheckPolicyRequestType()
                                          : Util.getObjectFactory().createCheckPolicyRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
