package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.DeleteResourceRequestType;

/**
 * A request to delete a (non-root) resource.
 */
final public
class DeleteResourceRequest extends AkentiRequest {
  private String resource;

 /**
   * Used to request the deletion of a (non-root) resource.
   * @param uid the id
   * @param resource the resource to be deleted
   */
  public DeleteResourceRequest(int uid, String resource) {
    super(uid);

    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.resource = resource;
  }

 /**
   * Returns the resource to be deleted.
   * @return the resource
   */
  public String getResource() {
    return resource;
  }

 /**
    * USED FOR DOING XML STUFF 
    */
  DeleteResourceRequest(DeleteResourceRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * Produces a DeleteResourceRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the DeleteResourceRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    DeleteResourceRequestType impl = type? Util.getObjectFactory().createDeleteResourceRequestType()
                                           : Util.getObjectFactory().createDeleteResourceRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
