package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.GetResourceInfoRequestType;

/**
 * A request to return the resource attributes (hints) file.
 * @author Tommaso Sciortino
 */
final public
class GetResourceInfoRequest extends AkentiRequest {
  private String resource;

 /**
   * Constructs a <code>GetResourceInfoRequest</code>.
   * @param uid the id
   * @param resource the resource
   */
  public GetResourceInfoRequest(int uid, String resource) {
    super(uid);
    this.resource = resource;
  }

 /**
   * Returns the resource whose hint file we wish to find.
   * @return the resource
   */
  public String getResource() {
    return resource;
  }

 /**
    */
  GetResourceInfoRequest(GetResourceInfoRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * Produces a GetResourceInfoRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the GetResourceInfoRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    GetResourceInfoRequestType impl = type? Util.getObjectFactory().createGetResourceInfoRequestType()
                                          : Util.getObjectFactory().createGetResourceInfoRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
