package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.ResourceBrowseRequestType;

/**
 * Holds a request to find the sub-resource to the
 * resource given by getResource()
 * @author Tommaso Sciortino
 */
final public
class ResourceBrowseRequest extends AkentiRequest {
  private String resource;

 /**
   * Constructs a <code>resourceBrowseRequest</code>.
   * @param uid ID number
   * @param resource Resource to browse
   */
  public ResourceBrowseRequest(int uid, String resource) {
    super(uid);
    this.resource = resource;
  }

 /**
   * Returns the resource to browse.
   * @return Resource
   */
  public String getResource() {
    return resource;
  }

 /**
   * USED FOR DOING XML STUFF
   * @param impl Request implementation
   */
  ResourceBrowseRequest(ResourceBrowseRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * USED FOR DOING XML STUFF
   * @return Request
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    ResourceBrowseRequestType impl = type? Util.getObjectFactory().createResourceBrowseRequestType()
                                           : Util.getObjectFactory().createResourceBrowseRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
