package lbl.akenti.messaging;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.ResourceBrowseResponseType;

/**
 * A response to a browse request type.
 * Holds a list of sub-resources.
 * @author Tommaso Sciortino
 */
final public
class ResourceBrowseResponse extends AkentiResponse {
  
  private List resources;
  
 /**
   * Constructs this <code>ResourceBrowseResponse</code>.
   * @param uid the id
   * @param mesg the AkentiMessage
   */  
  public ResourceBrowseResponse(int uid, AkentiMessage mesg) {
    this(uid,mesg,new String[0]);
  }
  
 /**
   * Constructs this <code>ResourceBrowseResponse</code> with an array of
   * resources.
   * @param uid the id
   * @param mesg the AkentiMessage
   * @param resourcesList an array of resources
   */
  public ResourceBrowseResponse(int uid, AkentiMessage mesg, String[] resourcesList) {
    super(uid, mesg);
    
    resources = new ArrayList(resourcesList.length);
    
    for (int i = 0; i < resourcesList.length; ++i) {
      resources.add(resourcesList[i]);
    }
    
    resources = Collections.unmodifiableList(resources);
  }
  
 /**
   * Returns an array of the sub-resources of the requested resource.
   * @return the sub-resources
   */
  public String[] getResources() {
    return (String[]) resources.toArray(new String[resources.size()]);
  }
  
 /**
   * JAXB STUFF
   * TODO: Efficiency
   */
  ResourceBrowseResponse(ResourceBrowseResponseType impl){
    super(impl);
    resources = Collections.unmodifiableList(impl.getResource());
  }
  
 /**
   * Produces a ResourceBrowseResponseType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the ResourceBrowseResponseType
   */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    ResourceBrowseResponseType impl = Util.getObjectFactory().createResourceBrowseResponseGenerated();
    
    impl.getResource().addAll(resources);
    return impl;
  }
}
