package lbl.akenti.messaging;

import java.io.IOException;
import java.text.ParseException;
import javax.xml.bind.JAXBException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.UploadPolicyCertificateRequestType;

/**
 * A request for uploading a certificate.
 * @author Tommaso Sciortino
 */
final public
class UploadPolicyCertificateRequest extends AkentiRequest {
  private PolicyCertificate cert;

 /**
   * Creates an UploadCertificateRequest.
   * @param uid the id
   * @param cert the Policy Certificate to be uploaded
   */
  public UploadPolicyCertificateRequest(int uid, PolicyCertificate cert) {
    super(uid);

    if (cert == null) {
      throw new IllegalArgumentException();
    }

    this.cert = cert;
  }

 /**
   * Returns the certificate we wish to upload.
   * @return the PolicyCertificate
   */
  public PolicyCertificate getCertificate() {
    return cert;
  }

 /**
    * USED FOR DOING XML STUFF 
    */
  UploadPolicyCertificateRequest(UploadPolicyCertificateRequestType impl)
    throws IOException, ParseException {
    super(impl);
    AkentiCertificate temp = OpaqueCertificate.fromJAXBImpl(impl.getOpaqueCertificate());

    if (temp instanceof PolicyCertificate == false) {
      throw new ParseException("Invalid cert type", -1);
    }

    cert = (PolicyCertificate) temp; 
  }
  
 /**
   * Produces a UploadPolicyCertificateRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the UploadPolicyCertificateRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    UploadPolicyCertificateRequestType impl = 
      type? Util.getObjectFactory().createUploadPolicyCertificateRequestType() 
          : Util.getObjectFactory().createUploadPolicyCertificateRequestGenerated();

    impl.setOpaqueCertificate(OpaqueCertificate.toJAXBImpl(getCertificate()));
    return impl;
  }
}
