package lbl.akenti.parsers;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
  * Tokenizes the exression. 
  *
  * @author Abdelilah Essiari
  */
final class 
ExpressionScanner implements TokenType {
  private ArrayList tokens = new ArrayList();
  private String currentToken = null;
  private int currentIndex = -1;

  /**
    * Construct with an expression.
    */
  ExpressionScanner(String expr) {
    StringTokenizer tokenizer = new StringTokenizer(expr, " &!|=><()", true);
    int count = tokenizer.countTokens();

    if (count == 0) {
      return;
    }

    String[]temp = new String[count];

    for (int i = 0; i < count; ++i) {
      temp[i] = tokenizer.nextToken();
    }

    int last = count - 1;

    for (int i = 0; i < last; ++i) {
      switch(temp[i].charAt(0)) {
        case '&':
        case '|':
          tokens.add(temp[i] + temp[i+1]);
          ++i;
          break;
        case '!':
        case '<':
        case '>':
          if (temp[i + 1].charAt(0) == '=') {
            tokens.add(temp[i] + temp[i+1]);
            i++;
          } else {
            tokens.add(temp[i]);
          }
          break;
        case ' ':
          // do nothing
          break;
        default:
          tokens.add(temp[i]);
      }
    }

    if (temp[last] != " ") {
      tokens.add(temp[last]);
    }
  }

  /**
    * Returns current token.
    */
  String getCurrentToken() {
    return currentToken;
  }

  /**
    * Advances to the next token.  
    */
  String advance() {
    if (!hasMoreTokens()) {
      currentToken = INVALID_STRING;
      return INVALID_STRING;
    }
 
    currentToken = (String) tokens.get(++currentIndex);
 
    if (TokenMap.tokenType(currentToken) != CONST) {
      return currentToken;
    } else if (peek() != CONST) {
      return currentToken;
    }

    StringBuffer sb = new StringBuffer(currentToken);

    do {
      sb.append(' ');
      sb.append(tokens.get(++currentIndex));
    } while (peek() == CONST);
    
    currentToken = sb.toString();
    return currentToken;
  }

  /**
    */
  private boolean hasMoreTokens() {
    return currentIndex + 1 < tokens.size();
  }

  /**
    * Returns the type of next token.
    *
    */
  private int peek() {
    int idx = currentIndex + 1;

    return (idx == tokens.size())? INVALID : TokenMap.tokenType((String) tokens.get(idx));
  }
 
  /**
    */
  public String toString() {
    return tokens.toString();
  }
}
