package lbl.akenti.parsers;

/**
  * 
  * @author Abdelilah Essiari
  */
final 
class TreeNode implements TokenType {
  private int type;
  private String data;
  private TreeNode leftChild;
  private TreeNode rightChild;

  /**
    */
  TreeNode(int type, String data) {
    this(type, data, null, null);
  }

  /**
    */
  TreeNode(int type, String data, TreeNode leftChild, TreeNode rightChild) {
    this.type = type;
    this.data = data;
    this.leftChild = leftChild;
    this.rightChild = rightChild;
  }

  /**
    */
  TreeNode deepCopy() {
    return new TreeNode(type, 
                        data, 
                        (this.leftChild != null) ? this.leftChild.deepCopy() : null,
                        (this.rightChild != null) ? this.rightChild.deepCopy() : null);
  }

  /**
    */
  int getTokenType() {
    return type;
  }

  /**
    */
  String getData() {
    return data;
  }

  /**
    */
  TreeNode getLeftChild() {
    return leftChild;
  }
 
  /**
    */
  TreeNode getRightChild() {
    return rightChild;
  }

  /**
    */
  boolean isNull() {
    return type == ENULL;
  }

  /**
    */
  public String toString() {
    if (isNull()) {
      return "()";
    }

    StringBuffer sb = new StringBuffer();

    toString(this, sb);
    return sb.toString();
  }

  /**
    */
  private void toString(TreeNode n, StringBuffer sb) {
    switch(n.getTokenType()) {
      case AND: case OR:
        sb.append("( ");
        toString(n.getLeftChild(), sb);
        sb.append(' ');
        sb.append(n.getData());
        sb.append(' ');
        toString(n.getRightChild(), sb);
        sb.append(" )");
        break;
      case NOT:
        n = n.getLeftChild();
 
        if (n.getTokenType() == NOT) {
          sb.append("! ( ");
          toString(n, sb);
          sb.append(" )");
        } else {
          sb.append("! ");
          toString(n, sb);
        } 
        break;
      case CONST:
        sb.append(n.getData());
        break;
      case INVALID:
        throw new RuntimeException();
      default:
        sb.append("( ");
        toString(n.getLeftChild() , sb);
        sb.append(" ");
        sb.append(n.getData());
        sb.append(" ");
        toString(n.getRightChild() , sb);
        sb.append(" )");
    }
  }
}
