package lbl.akenti.policyEngine;

import java.security.cert.X509Certificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.util.AkentiMessage;

/**
  * Holds information collected during CheckAccess/CheckPolicy requests.<BR>
  * 
  * @author Abdelilah Essiari
  */
public interface PolicyContext {

  /**
    * Returns the name of the resource.
    */
  String getResource();

  /**
    * Returns the resource alternative name.
    */
  String getResourceAltName();
  
  /**
    * Sets the name of the resource.
    */
  void setResource(String resource);

  /**
    * Sets the resource alternative name.
    */
  void setResourceAltName(String resourceAlt);
  
  /**
    * Returns the subject.
    */
  AkentiPrincipal getSubject();

  /**
    * Returns the subject's <code>X509</code> certificate.
    */
  X509Certificate getSubjectCertificate();

  /**
    * Returns the subjects <code>Capability</code>.
    */
  CapabilityCertificate getCapabilityCertificate();

  /**
    * Returns the resource policies.
    *
    * @see ResourcePolicy
    */
  ResourcePolicy[] toResourcePolicies();

  /**
    * Returns <code>X509</code> certificates.
    */
  X509Certificate[] toX509Certificates();
  
  /**
    * Returns the <code>Attribute</code> certificates.
    */
  AttributeCertificate[] toAttributeCertificates();

  /**
    * Returns the cache interval.
    */
  long getCacheInterval();

  /**
    * Returns certificates collected by the external
    * evaluator.
    *
    * @see ExternalEvaluator
    */
  Certificate[] toExternalCertificates();

  /**
    */
  String [] toDesiredActions();

  /**
    */
  AkentiMessage getAkentiMessage();

  void addResourcePolicy(ResourcePolicy policy);
  
}
