package lbl.akenti.policyEngine;
 
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AkentiPrincipal;

/**
  * Manages the trust material and is responsible for deciding whether 
  * certificates/subject are trusted. This involves locating X509 
  * certificates and verifying signatures.
  *
  * @see Certificate 
  * @see CertificateHolder 
  * @see AkentiPrincipal 
  * @author Abdelilah Essiari
  **/
public 
interface TrustVerifier {

  /**
    * Determines the level at which the <code>Certificate</code> wrapped in the 
    * <code>CertificateHolder</code> should be trusted.
    *
    * @see CertificateHolder 
    * @see CertificateHolder#getTrustLevel() 
    */
  boolean verify(CertificateHolder cert);

  /**
    */
  CertificateHolder verify(Certificate cert);

  /**
    * Determines whether the subject is to be trusted or not.
    *
    * @return null if the subject's certificate could not be locacted.
    *         a CertificateHolder containing the subject's otherwise.
    *
    * @see CertificateHolder 
    * @see CertificateHolder#getTrustLevel() 
    */
  CertificateHolder verifySubject(AkentiPrincipal ap);
}
