/*
 * CheckPolCertCollector.java
 *
 * Created on July 28, 2004, 10:03 AM
 */

package lbl.akenti.policyEngine.check;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Properties;
import java.util.StringTokenizer;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.policyEngine.impl.AkentiCertificateHolder;
import lbl.akenti.policyEngine.impl.AkentiResourcePolicy;
import lbl.akenti.policyEngine.impl.AkentiRuntime;
import lbl.akenti.policyEngine.impl.AkentiTrustVerifier;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ConfFileParser;
import lbl.akenti.util.UTCTime;

/**
 *
 * @author Tommaso Sciortino
 */
public class CheckPolCertCollector implements AkentiConstants {
  private AkentiRuntime runtime;
  private AkentiMessage message;
  private CertInfoCollector cinfoCol;
  
  
  /** Creates a new instance of CheckPolCertCollector */
  public CheckPolCertCollector(AkentiRuntime runtime, CertInfoCollector cinfoCol) {
    this.runtime = runtime;
    this.cinfoCol = cinfoCol;
  }
  
  public AkentiMessage getAkentiMessage() {
    return message;
  }
  
  public boolean collectPolicies() {
    DummyCertificate dummyCert = new DummyCertificate();
    
    String[] resources = parseResource(runtime.getPolicyContext().getResource());
    
    if (resources == null) {
      AkentiMessage mesg = new AkentiMessage(401, "Parsing resource name failed");
      CertInfo cinfo = new CertInfo(dummyCert,mesg,false);
      cinfoCol.addCertInfo(cinfo);
      cinfoCol.setStatus(false);
      return false;
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.length() == 0) {
      AkentiMessage mesg = new AkentiMessage(402, "root path unspecified");
      CertInfo cinfo = new CertInfo(dummyCert,mesg,false);
      cinfoCol.addCertInfo(cinfo);
      cinfoCol.setStatus(false);
      return false;
    }
    
    String encType = System.getProperty("AK_CERT_ENC_TYPE", AK_CERT_ENC_TYPE);
    String policyFileName = System.getProperty("PolicyFileName", PolicyFileName);
    
    String temp = base + '/' + resources[0] + '/' + policyFileName;
    
    File f = new File(temp);
    
    if (!f.isAbsolute()) {
      AkentiMessage mesg = new AkentiMessage(403, "root path is not absolute");
      CertInfo cinfo = new CertInfo(dummyCert,mesg,false);
      cinfoCol.addCertInfo(cinfo);
      cinfoCol.setStatus(false);
      return false;
    } else if (!f.exists()) {
      AkentiMessage mesg = new AkentiMessage(404, "root policy file does not exist");
      CertInfo cinfo = new CertInfo(dummyCert,mesg,false);
      cinfoCol.addCertInfo(cinfo);
      cinfoCol.setStatus(false);
      return false;
    }
    
    PolicyCertificate polCert = loadPolicyCertificate(temp);

    if (polCert == null) {
      AkentiMessage mesg = new AkentiMessage(405, "error reading root policy from " + temp);
      CertInfo cinfo = new CertInfo(dummyCert,mesg,false);
      cinfoCol.addCertInfo(cinfo);
      cinfoCol.setStatus(false);
      return false;
    }
    
    if (!polCert.getResource().equals(resources[0])) {
      AkentiMessage mesg = new AkentiMessage(406, "wrong root resource name ");
      int index = cinfoCol.lookup(polCert.getHashString(), "Policy");
      if (index < 0) {
        CertInfo cinfo = new CertInfo(polCert,mesg,false);
        cinfo.setHashCode(polCert.getHashString());
        cinfo.setResource(polCert.getResource());
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index, mesg);
      }
      cinfoCol.setStatus(false);
      return false;
    } else if (!polCert.containsCertificateAuthorityInfos()) {
      int index = cinfoCol.lookup(polCert.getHashString(), "Policy");
      AkentiMessage mesg = new AkentiMessage(407, "missing root cas in root policy");
      
      if (index < 0) {
        CertInfo cinfo = new CertInfo(polCert,mesg,false);
        cinfo.setHashCode(polCert.getHashString());
        cinfo.setResource(polCert.getResource());
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index, mesg);
      }
      cinfoCol.setStatus(false);
      return false;
    }
    
    int size = resources.length;
    
    if (!cinfoCol.isStatus()) {
      return false;
    } else {
      boolean leaf = false;
      
      if (size == 1) {
        leaf = true;
      } else if (size == 2) {
        f = new File(base + '/' + resources[1]);
        leaf = !f.isDirectory();
      }
      
      int index = cinfoCol.lookup(polCert.getHashString(), "Policy");
      AkentiMessage mesg = new AkentiMessage(101, "Collect policy certificates ok");
      
      if (index < 0) {
        CertInfo cinfo = new CertInfo(polCert,mesg,true);
        cinfo.setHashCode(polCert.getHashString());
        cinfo.setResource(polCert.getResource());
        cinfo.setLeaf(leaf);
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index,mesg);
      }
      
      AkentiResourcePolicy arp = new AkentiResourcePolicy(polCert);
      arp.setLeaf(leaf);
      runtime.getPolicyContext().addResourcePolicy(arp);
    }
    
    for (int i = 1; i < size; ++i) {
      temp = base + '/' + resources[i];
      f = new File(temp);
      if (f.isDirectory()) {
        temp += '/' + policyFileName;
        f = new File(temp);
        
        if (f.exists()) {
          if ((polCert = loadPolicyCertificate(temp)) == null) {
            int index = cinfoCol.lookup(polCert.getHashString(),"Policy");
            AkentiMessage mesg = new AkentiMessage(430, "unable to read policy " + resources[i]);
            
            if (index < 0) {
              CertInfo cinfo = new CertInfo(polCert,mesg,false);
              cinfo.setHashCode(polCert.getHashString());
              cinfo.setResource(polCert.getResource());
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }

            cinfoCol.setStatus(false);
          }
          
          if (!polCert.getResource().equals(resources[i])) {
            AkentiMessage mesg = new AkentiMessage(431, "unable to read policy " + polCert.getResource());
            int index = cinfoCol.lookup(polCert.getHashString(), "Policy");
            if (index < 0) {
              CertInfo cinfo = new CertInfo(polCert,mesg,false);
              cinfo.setHashCode(polCert.getHashString());
              cinfo.setResource(polCert.getResource());
              cinfo.setError(false);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else {
            boolean leaf = false;
            
            if (i == size - 1) {
              leaf = true;
            } else if (i == size - 2) {
              f = new File(base + '/' + resources[size - 1]);
              leaf = !f.isDirectory();
            }
            
            AkentiMessage mesg = new AkentiMessage(101, "Collect policy certificates ok");
            int index = cinfoCol.lookup(polCert.getHashString(), "Policy");
            if (index < 0) {
              CertInfo cinfo = new CertInfo(polCert,mesg,true);
              cinfo.setHashCode(polCert.getHashString());
              cinfo.setResource(polCert.getResource());
              cinfo.setLeaf(leaf);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
            }
            
            // TODO: leaf info must be kept here
            runtime.getPolicyContext().addResourcePolicy(new AkentiResourcePolicy(polCert));
          }
        }
        
      } else if (i != size - 1) {
        
        AkentiMessage mesg = new AkentiMessage(432, "bad resource tree");
        CertInfo cinfo = new CertInfo(dummyCert,mesg,true);
        cinfo.setResource(resources[i]);
        cinfoCol.addCertInfo(cinfo);
        cinfoCol.setStatus(false);
        break;
      }
    }
    
    message = new AkentiMessage(101, "Collect policy certificates ok");
    return true; // Necessary for Test case -doesnt really matter otherwise
    
  }
  
  boolean verifyPolicies() {
        AkentiTrustVerifier trustVerifier = (AkentiTrustVerifier) runtime.getTrustVerifier();
 
    trustVerifier.loadTrustMaterial();
    
    
    boolean ret = true;
    
    int size = cinfoCol.getCertInfos().length;
    int j = 1;
    
    for (int i = 0; i < size; ++i) {
      CertInfo cinfo = cinfoCol.getCertInfos()[i];
      if (cinfo.getType().equals("Policy")) {
        PolicyCertificate pcert = (PolicyCertificate) cinfo.getAkentiCertificate();
        AkentiResourcePolicy policy = new AkentiResourcePolicy(pcert);
        policy.setLeaf(cinfo.isLeaf());
        if (cinfo.isUsed()) {
          runtime.setPolicyLevel(j++);
          verifyPolicy(policy);
        }
      }
    }
    
    return true;
  }
  
  public boolean verifyPolicy(AkentiResourcePolicy policy) {
    PolicyCertificate pcert = policy.getPolicyCertificate();
    
    if (!pcert.checkValidity(new UTCTime())) {
      AkentiMessage mesg = new AkentiMessage(480, " Validation failed");
      int index = cinfoCol.lookup(pcert.getHashString(), "Policy");
      
      if (index < 0) { // doesnt exist, add it
        CertInfo cinfo = new CertInfo(pcert,mesg,false);
        cinfo.setHashCode(pcert.getHashString());
        cinfo.setResource(pcert.getResource());
        cinfo.setError(true);
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index, mesg);
        cinfoCol.setUsed(index, false);
        cinfoCol.setError(index, true);
      }
      cinfoCol.setStatus(false);
    }
    
    AkentiIssuer issuer = policy.getPolicyCertificate().getIssuer();
    boolean nameVerified = false;
    
    for (int i = 0; !nameVerified && i < policy.getPolicyCertificate().getUseCondIssuerGroups().length; ++i) {
      UseCondIssuerGroup group = (policy.getPolicyCertificate().getUseCondIssuerGroups())[i];
      
      nameVerified = group.containsUCIssuer(issuer.getIssuer());
    }
    
    if (!nameVerified) {
      
      AkentiMessage mesg = new AkentiMessage(480, " Validation failed");
      int index = cinfoCol.lookup(pcert.getHashString(), "Policy");
      if (index < 0) {
        
        CertInfo cinfo = new CertInfo(pcert,mesg,false);
        cinfo.setHashCode(pcert.getHashString());
        cinfo.setResource(pcert.getResource());
        cinfo.setError(true);
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index, mesg);
        cinfoCol.setUsed(index, false);
        cinfoCol.setError(index, true);
      }
      cinfoCol.setStatus(false);
    }
    
    AkentiCertificateHolder holder = new AkentiCertificateHolder(pcert);
    
    if (runtime.getTrustVerifier().verify(holder)) {
      
      AkentiMessage mesg = new AkentiMessage(102, "verifying policies ok");
      int index = cinfoCol.lookup(pcert.getHashString(), "Policy");
      if (index < 0) { // doesnt exist, add it
        
        CertInfo cinfo = new CertInfo(pcert,mesg,true);
        cinfo.setHashCode(pcert.getHashString());
        cinfo.setResource(pcert.getResource());
        cinfoCol.addCertInfo(cinfo);
      } else {
        cinfoCol.addMessage(index, mesg);
      }
      return true;
    }
    
    return true;
  }
  
  private PolicyCertificate
  loadPolicyCertificate(String fileName) {
    try {
      String encType = System.getProperty("AK_CERT_ENC_TYPE", AK_CERT_ENC_TYPE);
      AkentiCertificate cert = AkentiCertificateFactory.initCertFromFile(encType, fileName);
 
      if (cert instanceof PolicyCertificate) {
        return (PolicyCertificate) cert;
      }
    } catch(IOException ioe) {
      ioe.printStackTrace();
    } catch(ParseException pe) {
      pe.printStackTrace();
    }
 
    return null;
  }

  private static String[] parseResource(String resource) {
    StringTokenizer st = new StringTokenizer(resource, "/ ", true);
    boolean ret = false;
    StringBuffer name = new StringBuffer();
    
    ArrayList resourcesBuffer = new ArrayList();
    
    while (st.hasMoreTokens()) {
      String temp = st.nextToken();
      
      if (temp.equals("/") || temp.equals(" ")) {
        break;
      }
      
      name.append(temp);
      resourcesBuffer.add(name.toString());
      ret = true;
      
      if (!st.hasMoreTokens()) {
        break;
      }
      
      if (st.nextToken().equals(" ")) {
        ret = false;
        break;
      }
      
      name.append("/");
      ret = false;
    }
    
    if (!ret) {
      return null;
    } else {
      return (String[]) resourcesBuffer.toArray(new String[resourcesBuffer.size()]);
    }
  }
  
}
