/*
 * CheckUCCertCollector.java
 *
 * Created on July 29, 2004, 2:41 PM
 */

package lbl.akenti.policyEngine.check;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.dirServices.CertificateRetrievalFactory;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.policyEngine.impl.AkentiCertificateHolder;
import lbl.akenti.policyEngine.impl.AkentiResourcePolicy;
import lbl.akenti.policyEngine.impl.AkentiRuntime;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;

/**
 *
 * @author Tommaso Sciortino
 */
public class CheckUCCertCollector {
  private AkentiRuntime runtime;
  private AkentiMessage message;
  private CertInfoCollector cinfoCol;
  
  /**
   * Creates a new instance of CheckUCCertCollector
   **/
  public CheckUCCertCollector(AkentiRuntime runtime, CertInfoCollector cinfoCol) {
    this.runtime = runtime;
    this.cinfoCol = cinfoCol;
  }
  
  public AkentiMessage getAkentiMessage() {
    return message;
  }
  
  public AkentiCertificate[] getUCCertHolders(AkentiResourcePolicy policy, ArrayList dirVector) {
    String hash = policy.getPolicyCertificate().getHashString();
    
    ArrayList ucCertVec = new ArrayList();
    
    HashMap args = new HashMap();
    int numOfGroups = policy.getPolicyCertificate().getUseCondIssuerGroups().length;
    
    args.put("type", "UseCondition");
    args.put("hash", Util.createHashName(hash));
    
    ArrayList ucCerts = new ArrayList();
    ArrayList trustedUCCerts = new ArrayList();
    ArrayList dir1 = new ArrayList();
    ArrayList dir2 = new ArrayList();
    
    int groupIndex = 0;
    boolean foundValidUC;
    
    TrustVerifier certVerifier = runtime.getTrustVerifier();
    
    do {
      UseCondIssuerGroup group = (policy.getPolicyCertificate().getUseCondIssuerGroups())[groupIndex];
      Directory[] dirs = group.getDirectories();
      
      foundValidUC = false;
      
      for (int j = 0; !foundValidUC && j < dirs.length; ++j) {
        
        AkentiCertificate[] certs = null;
        try {
          
          certs = CertificateRetrievalFactory.retrieveAkentiCertificates(dirs[j].parseDirectory(), args);
        } catch (MalformedURLException ex) {
          throw new RuntimeException("I don't think this should happen");
        }
        
        int k;
        
        // We must find trusted uc cert for each group
        for (k = 0; !foundValidUC && k < certs.length; ++k) {
          UseCondCertificate cert = (UseCondCertificate) certs[k];
          AkentiIssuer issuer = cert.getIssuer();
          
        /* first check for fundamental error - maybe policy or resource
         * doesnt exist, in which case level wouldnt have been set
         * How did it get to this cert then ??? XXX
         */
          if (runtime.getPolicyLevel() <= 0) {
            
            AkentiMessage mesg = new AkentiMessage(501, "failed to find uc certs");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!cert.getHashString().equals(hash)) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Hash check failed");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition", cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!cert.checkValidity(new UTCTime())) {
            
            AkentiMessage mesg = new AkentiMessage(502,  "Validation failed");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index,mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!group.containsUCIssuer(issuer.getIssuer())) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Group check failed");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!cert.getCondition().isValid()) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Conditions check failed");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!certVerifier.verify(new AkentiCertificateHolder(cert))) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Trust check failed");
            int index = cinfoCol.lookup(cert.getHashString(),"UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else {
            foundValidUC = true;
            trustedUCCerts.add(cert);
            dir2.add(dirs[j].toString());
          }
        }
        
        // We will check the trust later
        for (int l = k; l < certs.length; ++l) {
          UseCondCertificate cert = (UseCondCertificate) certs[l];
          
          AkentiIssuer issuer = cert.getIssuer();
                    
          if (!cert.getHashString().equals(hash)) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Hash check failed");
            int index = cinfoCol.lookup(cert.getHashString(),"UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!cert.checkValidity(new UTCTime())) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Validation failed");
            int index = cinfoCol.lookup(cert.getHashString(), "UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!group.containsUCIssuer(issuer.getIssuer())) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Group check failed");
            int index = cinfoCol.lookup(cert.getHashString(),"UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else if (!cert.getCondition().isValid()) {
            
            AkentiMessage mesg = new AkentiMessage(502, "Conditions check failed");
            int index = cinfoCol.lookup(cert.getHashString(),"UseCondition",cert.getUniqueID());
            if (index < 0) { // doesnt exist, add it
              
              CertInfo cinfo = new CertInfo(cert,mesg,dirs[j].toString(),false);
              cinfo.setHashCode(cert.getHashString());
              cinfo.setResource(cert.getResource());
              cinfo.setError(true);
              cinfoCol.addCertInfo(cinfo);
            } else {
              cinfoCol.addMessage(index, mesg);
              cinfoCol.setUsed(index, false);
              cinfoCol.setError(index, true);
            }
            cinfoCol.setStatus(false);
          } else {
            ucCerts.add(cert);
            dir1.add(dirs[j].toString());
          }
        }
      }
    } while (foundValidUC && ++groupIndex < numOfGroups);
    
    if (foundValidUC) {
      int level = runtime.getPolicyLevel();
      for (int i = 0; i < trustedUCCerts.size(); ++i) {
        ucCertVec.add(trustedUCCerts.get(i));
        dirVector.add(dir2.get(i));
      }
      for (int i = 0; i < ucCerts.size(); ++i) {
        ucCertVec.add(ucCerts.get(i));
        dirVector.add(dir1.get(i));
      }
    }
    
    return (AkentiCertificate[]) ucCertVec.toArray(new AkentiCertificate[ucCertVec.size()]);
  }
  
  boolean collectUseCondCertificates(AkentiResourcePolicy policy) {
    ArrayList dirVector = new ArrayList();
    
    AkentiCertificate[] ucCertVec = getUCCertHolders(policy, dirVector);
    
    if (ucCertVec.length == 0) {
      
      message = new AkentiMessage(501, "failed to find uc certs for " + policy.getResource());
      
      cinfoCol.setStatus(false);
    }
    
    TrustVerifier certVerifier = runtime.getTrustVerifier();
    int count = 0;
    
    for (int i = 0; i < ucCertVec.length; ++i) {
      UseCondCertificate ucCert = (UseCondCertificate) ucCertVec[i];
      
      if (!ucCert.isScopeLocal() || policy.isLeaf()) {
        if (certVerifier.verify(new AkentiCertificateHolder(ucCertVec[i]))) {
          policy.addUseCondCertificate(ucCert);
          
          AkentiMessage mesg = new AkentiMessage(103, "use condition certs verified for " + policy.getResource() );
          int index = cinfoCol.lookup(ucCert.getHashString(),"UseCondition",ucCert.getUniqueID());
          if (index < 0) { // doesnt exist, add it
            
            CertInfo cinfo = new CertInfo(ucCert,mesg,dirVector.get(i).toString(),true);
            cinfo.setHashCode(ucCert.getHashString());
            cinfo.setResource(ucCert.getResource());
            cinfoCol.addCertInfo(cinfo);
          } else {
            cinfoCol.addMessage(index, mesg);
          }
          ++count;
        } else {// certificate could not be verified
          
          AkentiMessage mesg = new AkentiMessage(503, "cert could not be verified");
          int index = cinfoCol.lookup(ucCert.getHashString(),"UseCondition",ucCert.getUniqueID());
          if (index < 0) { // doesnt exist, add it
            
            CertInfo cinfo = new CertInfo(ucCert,mesg,dirVector.get(i).toString(),false);
            cinfo.setHashCode(ucCert.getHashString());
            cinfo.setResource(ucCert.getResource());
            cinfo.setError(true);
            cinfoCol.addCertInfo(cinfo);
          } else {
            cinfoCol.addMessage(index, mesg);
            cinfoCol.setUsed(index, false);
            cinfoCol.setError(index, true);
          }
          cinfoCol.setStatus(false);
        }
      } else {
        // comes here if verified but not used - like if there was
        // an error before this point.
        
        AkentiMessage mesg = new AkentiMessage(103, "use condition certs verified for " + policy.getResource());
        int index = cinfoCol.lookup(ucCert.getHashString(), "UseCondition",ucCert.getUniqueID());
        if (index < 0) { // doesnt exist, add it
          
          CertInfo cinfo = new CertInfo(ucCert,mesg,false);
          cinfo.setHashCode(ucCert.getHashString());
          cinfo.setResource(ucCert.getResource());
          cinfoCol.addCertInfo(cinfo);
        } else {
          cinfoCol.addMessage(index, mesg);
          cinfoCol.setUsed(index, false);
        }
      }
      
    }
    
    
    message = new AkentiMessage(103, "use condition certs verified for " + policy.getResource());
    
    return true; // Necessary for Test case - otherwise doesnt mean anything
  }
  
  boolean collectUseCondCertificates() {
    boolean ret = true;
    
    int size = cinfoCol.getCertInfos().length;
    
    for (int i = 0; i < size; ++i) {
      CertInfo cinfo = (cinfoCol.getCertInfos())[i];
      if (cinfo.getType().equals("Policy")) {
        AkentiCertificate akcert = cinfo.getAkentiCertificate();
        PolicyCertificate pcert = (PolicyCertificate) akcert;
        AkentiResourcePolicy policy  = new AkentiResourcePolicy(pcert);
        policy.setLeaf(cinfo.isLeaf());
        collectUseCondCertificates(policy);
      }
    }
    return true; // Necessary for Test case - otherwise doesnt mean anything
  }
  
}