/*
 * PolicyChecker.java
 *
 * Created on July 30, 2004, 1:45 PM
 */

package lbl.akenti.policyEngine.check;

import java.io.IOException;
import java.security.cert.X509Certificate;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.policyEngine.impl.AkentiCertificateHolder;
import lbl.akenti.policyEngine.impl.AkentiRuntime;
import lbl.akenti.policyEngine.impl.AkentiX509Certificate;
import lbl.akenti.util.ConfFileParser;

/**
 *
 * @author  Tommaso Sciortino
 */
public class PolicyChecker {
  private AkentiRuntime runtime;
  private AkentiMessage mesg;
  private CertInfoCollector ciCol;
  
  /** Creates a new instance of PolicyChecker */
  public PolicyChecker() {
    runtime = new AkentiRuntime();
    ciCol = new CertInfoCollector();
  }
  
  public AkentiMessage getMessage() {
    return mesg;
  }
  
  public CertInfoCollector checkPolicy(String resource) {
    runtime.getPolicyContext().setResource(resource);
    runtime.getPolicyContext().setResourceAltName(resource);
    
    collectAndVerifyPolicies();
    collectAndVerifyUseConds();
    verifyCAS();
    
    ciCol.CertInfoReorder();
    
    return ciCol;
  }
  
  boolean collectAndVerifyPolicies() {
    CheckPolCertCollector policyCollector = new CheckPolCertCollector(runtime,ciCol);
    
    if (!policyCollector.collectPolicies()) {
      mesg = policyCollector.getAkentiMessage();
    } else if (!policyCollector.verifyPolicies()) {
      mesg = policyCollector.getAkentiMessage();
    }
    
    return true;
  }
  
  boolean collectAndVerifyUseConds() {
    CheckUCCertCollector ucCollector = new CheckUCCertCollector(runtime,ciCol);
    
    if (!ucCollector.collectUseCondCertificates()) {
      mesg = ucCollector.getAkentiMessage();
    }
    
    return true;
  }
  
  boolean verifyCAS() {
    PolicyContext ctx = runtime.getPolicyContext();
    ResourcePolicy[] policies = ctx.toResourcePolicies();
    TrustVerifier verifier = runtime.getTrustVerifier();
    boolean ok = true;
    
    // SKIP ROOT SINCE IT HAS ALREADY BEEN CHECKED
    for (int i = 1; i < policies.length; ++i) {
      runtime.setPolicyLevel(i + 1);
      
      int count = 0;
      CertificateAuthorityInfo[] infos = policies[i].getPolicyCertificate().getCertificateAuthorityInfos();
      
      for (int j = 0; j < infos.length; j++) {
        X509Certificate[] caChain = infos[i].getChain();
        AkentiX509Certificate cert = new AkentiX509Certificate(caChain[caChain.length - 1]);

        boolean ret = verifier.verify(new AkentiCertificateHolder(cert));
        
        if (!ret) {
          ok = false;
        } else {
          count++;
        }
      }
      
      if (count == 0 && infos.length == 0) {
        mesg = new AkentiMessage(460, "BLAH");
        return false;
      }
    }
    
    if (!ok) {
      mesg = new AkentiMessage(151, "BALH");
    } else {
      mesg = new AkentiMessage(150, "BALH");
    }
    
    return true;
  }
    
  void loadResourceMap(String fileName) throws IOException {
    ConfFileParser.loadFromFile(fileName);
  }
  
}
