package lbl.akenti.policyEngine.impl;
 
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.parsers.Result;
import lbl.akenti.policyEngine.Authorizer;
import lbl.akenti.policyEngine.Runtime;

/**
  * @author Abdelilah Essiari
  **/
class AkentiAuthorizer implements Authorizer {
  private Runtime runtime;
  private Set grantedActions = new TreeSet();
  private Set minimalActions = new TreeSet();
  private List condActionsList = new ArrayList();

  AkentiAuthorizer(Runtime runtime) {
    this.runtime = runtime;
  }

  public boolean authorize() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext();
    Iterator resourcePolicies = ctx.getResourcePolicies();
    int policyLevel = 1;

    do {
      AkentiResourcePolicy rp = (AkentiResourcePolicy) resourcePolicies.next();
      List ucCerts = rp.getCriticalUseCondCertificates();

      runtime.setPolicyLevel(policyLevel++);

      if (ucCerts.size() != 0) {
        if (authorize(ucCerts) != ucCerts.size()) {
          ctx.setMessage(301, "did not satisfy critical uc for " + rp.getResource());
          return false;
        }
      }
    } while(resourcePolicies.hasNext());

    Set desiredActions = ctx.getDesiredActions();
    boolean stop = desiredActions.size() != 0 && grantedActions.containsAll(desiredActions);
 
    if (stop == true) {
      ctx.addGrantedActions(grantedActions);
      ctx.setMessage(104, "actions granted");
      return true;
    }

    resourcePolicies = ctx.getResourcePolicies();
    policyLevel = 1;

    while (!stop && resourcePolicies.hasNext()) {
      AkentiResourcePolicy rp = (AkentiResourcePolicy) resourcePolicies.next();
      List ucCerts = rp.getNonCriticalUseCondCertificates();

      runtime.setPolicyLevel(policyLevel++);
      
      if (ucCerts.size() != 0 ) {
        if (authorize(ucCerts) == 0 ) {
        } else {
        }
      }

      stop = desiredActions.size() != 0 && desiredActions.equals(grantedActions);
    }

    // if (grantedActions.size() == 0 && condActionsList.size() == 0) {
    if (minimalActions.size() == 0) {
      ctx.setMessage(302, "User is allowed no actions");
      return false;
    } 

    ctx.addGrantedActions(grantedActions);

    if (condActionsList.size() == 0 &&
        desiredActions.size() != 0 && 
        !grantedActions.containsAll(desiredActions)) {
      ctx.setMessage(106, "User cannot do all desired actions");
      return true;
    } else if (condActionsList.size() == 0) {
      ctx.setMessage(104, "actions granted");
      return true;
    }

    Iterator iter = condActionsList.iterator();

    minimalActions = grantedActions;
 
    do {
      ConditionalActions condActions = (ConditionalActions) iter.next();

      if (!condActions.isCritical()) {
        break;
      }

      Set actions = new TreeSet(); 
      String[] temp = condActions.getActions();

      for (int j = 0;j < temp.length; ++j) {
        actions.add(temp[j]); 
      }
 
      if (actions.removeAll(minimalActions)) {
        ctx.addConditionalActions(
               new ConditionalActions(condActions.getCondition(), 
                                     (String []) actions.toArray(new String[0]), 
                                      true));
      } else {
        ctx.addConditionalActions(condActions);
      }
 
      minimalActions.addAll(actions);
    } while (iter.hasNext());

    iter = condActionsList.iterator();

    // THESE ARE ALREADY AS COMPACT AS THEY CAN BE
    while (iter.hasNext()) {
      ConditionalActions condActions = (ConditionalActions) iter.next();

      if (!condActions.isCritical()) {
        String[] actions = condActions.getActions();
 
        for (int j = 0; j < actions.length; ++j) {
          minimalActions.add(actions[j]);
        }
 
        ctx.addConditionalActions(condActions);
      }
    }

    if (desiredActions.size() != 0 && !minimalActions.containsAll(desiredActions)) {
      ctx.setMessage(107, 
                    "Further conditions apply, only a subset of desired actions");
      return true;
    }
 
    ctx.setMessage(105, "Further conditions apply");
    return true;
  }

  private int authorize(List ucCertsList) {
    Iterator ucCerts = ucCertsList.iterator();
    int count = 0;
    
    do {
      UseCondCertificate cert = (UseCondCertificate) ucCerts.next();
      Condition condition = cert.getCondition();
      boolean critical = cert.isCritical();
      AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext();
      Set actions = new TreeSet();
      String[] temp = cert.getActions();


      for (int j = 0; j < temp.length; ++j) {
        actions.add(temp[j]); 
      }

      if (ctx.containsDesiredActions()) {
        actions.retainAll(ctx.getDesiredActions());
      } 

      actions.removeAll(minimalActions);

      if (!critical && actions.size() == 0) {
      } else {
        DefaultInternalEvaluator evaluator;

        evaluator = (DefaultInternalEvaluator) runtime.getInternalEvaluator();

        evaluator.setAttributeInfos(condition.getAttributeInfos());

        Result result = evaluator.reduce(condition.parseConstraint());
        
        if (result.isTrue()) {
          grantedActions.addAll(actions);
          minimalActions.addAll(actions);
          ++count;
        } else if (result.isFalse()) {
          if (critical) {
            break;
          }
        } else {
          Condition cond = new Condition(result.getValue().toString(),
                                         evaluator.getAttributeInfos());
          ConditionalActions 
             condActions = new ConditionalActions(cond, 
                                                  (String []) actions.toArray(new String[0]), 
                                                  critical);
          if (critical) {
            minimalActions.addAll(actions);
          }

          condActionsList.add(condActions);
          ++count;
        }
      } 

    } while (ucCerts.hasNext());

    return count;
  }
}
