package lbl.akenti.policyEngine.impl;

import java.io.FileInputStream;
import java.util.Properties;
import java.util.Set;
import lbl.akenti.certs.CapabilityCertGenerator;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.policyEngine.Authorizer;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.PolicyCollector;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.UseCondCollector;
import lbl.akenti.util.AkentiMessage;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;
import lbl.akenti.util.ConfFileParser;

public class AkentiEngine {
  /**
    */ 
  private Runtime runtime; 

  /**
    */
  protected AkentiEngine(Runtime runtime) {
    if (runtime == null) {
      throw new IllegalArgumentException();
    }
 
    this.runtime = runtime;
  }

  /**
    */
  public AkentiEngine() {
    this(new AkentiRuntime()); 
  }

  /**
    */
  public void setDesiredActions(Set desiredActions) {
    AkentiPolicyContext ctx = (AkentiPolicyContext) getPolicyContext();

    ctx.setDesiredActions(desiredActions);
  }

  public CapabilityCertificate getCapabilityCertificate() {
    return getPolicyContext().getCapabilityCertificate();
  }

  /**
    */
  public PolicyContext getPolicyContext() {
    return runtime.getPolicyContext();
  }

  /**
    */
  public AkentiMessage getAkentiMessage() {
    return getPolicyContext().getAkentiMessage();
  }

  /**
    */
  public boolean checkAccess(String resource, AkentiPrincipal ap) {
    AkentiPolicyContext ctx = (AkentiPolicyContext) getPolicyContext();
 
    ctx.setResource(resource);
    ctx.setSubject(ap);
 
    PolicyCollector policyCollector = runtime.getPolicyCollector();
 
    if (!policyCollector.collect()) {
      return false;
    }
 
    AkentiTrustVerifier trustVerifier = (AkentiTrustVerifier) runtime.getTrustVerifier();
 
    trustVerifier.loadTrustMaterial();
 
    if (!policyCollector.verify()) {
      return false;
    }
 
    UseCondCollector useCondCollector = runtime.getUseCondCollector();
 
    if (!useCondCollector.collectAndVerify()) {
      return false;
    }
 
    CertificateHolder holder = trustVerifier.verifySubject(ap);
 
    if (holder == null) {
      ctx.setMessage(306, "user did not verify");
      return false;
    }
 
    AkentiX509Certificate cert = (AkentiX509Certificate) holder.getCertificate();
 
    ctx.setSubjectCertificate(cert.getX509Certificate());
 
    Authorizer authorizer = runtime.getAuthorizer();
 
    if (!authorizer.authorize()) {
      return false;
    }
 
    generateCapabilityCertificate(ctx);
    return true;
  }

  /**
    */
  private void generateCapabilityCertificate(AkentiPolicyContext ctx) {
     CapabilityCertGenerator gen = new CapabilityCertGenerator();
     String resource = ctx.getResource();
 
     gen.setVersionNumber(2);
     gen.setUniqueID(Util.createUniqueID(Util.createHashName(resource)));
     gen.setNotBefore(new UTCTime());
     gen.setNotAfter(new UTCTime(ctx.getCacheInterval()));
     gen.setResource(resource);
     gen.setPrincipal(ctx.getSubject());
     gen.setIssuer(new AkentiIssuer());
     gen.setPublicKey(ctx.getSubjectCertificate().getPublicKey());
     gen.setUnConditionalActions(ctx.toUnconditionalActions());
     gen.setConditionalActions(ctx.toConditionalActions());
     ctx.setCapabilityCertificate((CapabilityCertificate) gen.generate());
  }

  public static boolean init(String fileName, StringBuffer sb) {
    try {
      
      ConfFileParser.loadFromFile(fileName);
      RootResources.loadFromFile(fileName);
      sb.append("Initializing engine ok");
      return true;
    } catch(RuntimeException ex) {
      throw ex;
    } catch(Exception ex) {
      sb.append("Initializing engine failed:" + ex);
      return false;
    }
  }
}
