/**
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.manager;

import com.sun.net.ssl.internal.ssl.Provider;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.security.Security;
import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import lbl.akenti.policyManager.widgets.ACMHelpMenu;
import lbl.akenti.policyManager.widgets.ConfFilePicker;
import lbl.akenti.policyManager.widgets.HTMLMenuMaker;
import lbl.akenti.policyManager.widgets.LBLDesktopMaker;
import lbl.akenti.policyManager.widgets.LBLDialogFactory;
import lbl.akenti.policyManager.widgets.LBLResourcePanel;
import lbl.akenti.policyManager.widgets.LBLSavePanel;
import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.SplashScreen;
import lbl.akenti.policyManager.widgets.VersionChecker;
import lbl.akenti.policyManager.widgets.viewer.CertInfoViewFrame;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.AttributeCertGenerator;

/**
 * SUN MICROSYSTEMS RECOMMENDS JAVA 1.4.x OR ABOVE!!
 */
public class AkentiCertificateManager extends JFrame implements ActionListener {
    private JDesktopPane desktop;
    
    private static final int HTMLMENUOFFSET_X = 1000;
    private static final int HTMLMENUOFFSET_Y = 800;
    private static final String OFFCIAL_VERSION = "0.9c";
    
    private static final String POLICY_CERTIFICATE = "Policy Certificate";
    private static final String USECONDITION_CERTIFICATE = "Use-Condition Certificate";
    private static final String ATTRIBUTE_CERTIFICATE = "Attribute Certificate";
    
    private ClassLoader loader = Thread.currentThread().getContextClassLoader();
    
    public AkentiCertificateManager() {
        super("Akenti Certificate Manager");
        
        //Make the big window be indented 150 pixels from each edge of the screen.
        int inset = 150;
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(inset, inset,
        screenSize.width  - inset*2,
        screenSize.height - inset*2);
        
        
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new
        WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                quit(0);
            }
        });
        
        //Set up the GUI.
        desktop = LBLDesktopMaker.getInstance();
        
        setContentPane(desktop);
        setJMenuBar(createMenuBar());
        
        //Make dragging a little faster but perhaps uglier.
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        
    }
    
    protected JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        
        JMenu menu = new JMenu("Manager");
        menu.setMnemonic(KeyEvent.VK_M);
        menuBar.add(menu);
        
        JMenu prefMenu = new JMenu("Preferences");
        prefMenu.setMnemonic(KeyEvent.VK_P);
        menuBar.add(prefMenu);
        
        JMenu genMenu = new JMenu("Generators");
        genMenu.setMnemonic(KeyEvent.VK_G);
        menuBar.add(genMenu);
        
        JMenu advEditMenu = new JMenu("Advance");
        advEditMenu.setMnemonic(KeyEvent.VK_A);
        menuBar.add(advEditMenu);
        
        JMenu advNewMenu = new JMenu("New");
        advEditMenu.add(advNewMenu);
        
        JMenu helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(helpMenu);
        
        JMenuItem advPolyCert = new JMenuItem("Policy Certificate");
        advPolyCert.setActionCommand("advPolyCert");
        advPolyCert.addActionListener(this);
        advNewMenu.add(advPolyCert);
        
        JMenuItem advUCCert = new JMenuItem("Use-Condition Certificate");
        advUCCert.setActionCommand("advUCCert");
        advUCCert.addActionListener(this);
        advNewMenu.add(advUCCert);
        
        JMenuItem advAttrCert = new JMenuItem("Attribute Certificate");
        advAttrCert.setActionCommand("advAttrCert");
        advAttrCert.addActionListener(this);
        advNewMenu.add(advAttrCert);
        
        JMenuItem polyCertController = new JMenuItem("Policy Certificate Generator");
        polyCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));
        polyCertController.setActionCommand("polyCertController");
        polyCertController.addActionListener(this);
        
        JMenuItem useCertController = new JMenuItem("Use Condition Certificate Generator");
        useCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.ALT_MASK));
        useCertController.setActionCommand("useCertController");
        useCertController.addActionListener(this);
        
        JMenuItem attrCertController = new JMenuItem("Attribute Certificate Generator");
        attrCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.ALT_MASK));
        attrCertController.setActionCommand("attrCertController");
        attrCertController.addActionListener(this);
        
        JMenuItem resrcViewController = new JMenuItem("Resource Manager");
        resrcViewController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4, ActionEvent.ALT_MASK));
        resrcViewController.setActionCommand("resrcViewController");
        resrcViewController.addActionListener(this);
        
        genMenu.add(polyCertController);
        genMenu.add(useCertController);
        genMenu.add(attrCertController);
        menu.add(resrcViewController);
      
        JMenuItem clearURLController = new JMenuItem("Clear URL History");
        clearURLController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.ALT_MASK));
        clearURLController.setActionCommand("clearURLController");
        clearURLController.addActionListener(this);
        
        JMenuItem clearP12Controller = new JMenuItem("Clear P12 History");
        clearP12Controller.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.ALT_MASK));
        clearP12Controller.setActionCommand("clearP12Controller");
        clearP12Controller.addActionListener(this);
        
        JMenuItem loadConfController = new JMenuItem("Load Config File");
        loadConfController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.ALT_MASK));
        loadConfController.setActionCommand("loadConfController");
        loadConfController.addActionListener(this);
        
        prefMenu.add(clearURLController);
        prefMenu.add(clearP12Controller);
        prefMenu.add(loadConfController);
        
        JMenuItem quitItem = new JMenuItem("Quit");
        quitItem.setMnemonic(KeyEvent.VK_Q);
        quitItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_Q, ActionEvent.ALT_MASK));
        quitItem.setActionCommand("quit");
        quitItem.addActionListener(this);
        menu.add(quitItem);
        
        JMenuItem advEditMenuItem = new JMenuItem("Certificate Editor");
        advEditMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_5, ActionEvent.ALT_MASK));
        advEditMenuItem.setActionCommand("advancedEdit");
        advEditMenuItem.addActionListener(this);
        advEditMenu.add(advEditMenuItem);
        
        JMenuItem helpItem = new JMenuItem("Help Contents");
        helpItem.setMnemonic(KeyEvent.VK_C);
        helpItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_C, ActionEvent.ALT_MASK));
        helpItem.setActionCommand("help");
        helpItem.addActionListener(this);
        helpMenu.add(helpItem);
        
        JMenuItem releaseItem = new JMenuItem("Release Notes");
        releaseItem.setMnemonic(KeyEvent.VK_R);
        releaseItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_R, ActionEvent.ALT_MASK));
        releaseItem.setActionCommand("release");
        releaseItem.addActionListener(this);
        helpMenu.add(releaseItem);
        
        JMenuItem licenseItem = new JMenuItem("License");
        licenseItem.setMnemonic(KeyEvent.VK_L);
        licenseItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_L, ActionEvent.ALT_MASK));
        licenseItem.setActionCommand("license");
        licenseItem.addActionListener(this);
        helpMenu.add(licenseItem);
        
        JMenuItem aboutItem = new JMenuItem("About Akenti Certificate Manager");
        aboutItem.setMnemonic(KeyEvent.VK_A);
        aboutItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_A, ActionEvent.ALT_MASK));
        aboutItem.setActionCommand("about");
        aboutItem.addActionListener(this);
        helpMenu.add(aboutItem);
        
        return menuBar;
    }
    
    // React to menu selections.
    public void actionPerformed(ActionEvent e) {
        if ("polyCertController".equals(e.getActionCommand())) {
            createPolyCertControllerFrame();
        } else if ("attrCertController".equals(e.getActionCommand())) {
            createAttrCertControllerFrame();
        } else if ("useCertController".equals(e.getActionCommand())) {
            createUCCertControllerFrame();
        } else if ("resrcViewController".equals(e.getActionCommand())) {
            createResrcViewFrame();
        } else if ("clearURLController".equals(e.getActionCommand())) {
            LBLResourcePanel temp = new LBLResourcePanel(new LBLTopFrame(""), "");
            temp.createNewURLHistoryFile(false);
        } else if ("clearP12Controller".equals(e.getActionCommand())) {
            LBLSavePanel temp = new LBLSavePanel(new LBLTopFrame(""), "");
            temp.createNewP12HistoryFile(false);
        } else if ("loadConfController".equals(e.getActionCommand())) {
            ConfFilePicker cfPanel = new ConfFilePicker();
            cfPanel.createDialog(this, "Please load a configuration file.").setVisible(true);
        } else if ("advPolyCert".equals(e.getActionCommand())) {
            createAdvCertificate(POLICY_CERTIFICATE);
        } else if ("advUCCert".equals(e.getActionCommand())) {
            createAdvCertificate(USECONDITION_CERTIFICATE);
        } else if ("advAttrCert".equals(e.getActionCommand())) {
            createAdvCertificate(ATTRIBUTE_CERTIFICATE);
        } else if ("quit".equals(e.getActionCommand())) {
            quit(0);
        } else if ("help".equals(e.getActionCommand())) {
            showHelpPane();
        } else if ("release".equals(e.getActionCommand())) {
            showInformationPane("Akenti Release Notes", loader.getResource("lbl/akenti/policyManager/docs/release_notes/ReleaseNotes.html").toString());
        } else if ("about".equals(e.getActionCommand())) {
            showAboutDialog();
        } else if ("license".equals(e.getActionCommand())) {
            if (loader == null) { // there may not be a context class loader
                loader = AkentiCertificateManager.class.getClassLoader();
            }
            showInformationPane("Akenti License", loader.getResource("lbl/akenti/policyManager/docs/license/license.txt").toString());
        } else if ("advancedEdit".equals(e.getActionCommand())) {
            advanceEditCertificate();
        } else {
            throw new RuntimeException("No such feature exists.");
        }
    }
    
    public void showHelpPane() {
        ACMHelpMenu frame = new ACMHelpMenu();
        frame.setSize(HTMLMENUOFFSET_X, HTMLMENUOFFSET_Y);
        frame.setVisible(true); // necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    public void showAboutDialog() {
        if (loader == null) { // there may not be a context class loader
            loader = AkentiCertificateManager.class.getClassLoader();
        }
        JOptionPane.showInternalMessageDialog(desktop,
        "Akenti Certificate Generator " + OFFCIAL_VERSION + " official build",
        "About Akenti Certificate Generator",
        JOptionPane.INFORMATION_MESSAGE,
        new ImageIcon(loader.getResource("lbl/akenti/policyManager/docs/pics/AkentiKeyS.png"))
        );
        
    }
    public void showInformationPane(String title, String url) {
        HTMLMenuMaker frame = new HTMLMenuMaker(title, url);
        frame.setSize(HTMLMENUOFFSET_X, HTMLMENUOFFSET_Y);
        frame.setVisible(true); // necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    protected void createPolyCertControllerFrame() {
        PolCertController polCertController = new PolCertController();
        polCertController.start();
        LBLTopFrame frame = polCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected void createAttrCertControllerFrame() {
        AttrCertController attrCertController = new AttrCertController();
        attrCertController.start();
        LBLTopFrame frame = attrCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected void createUCCertControllerFrame() {
        UCCertController ucCertController = new UCCertController();
        ucCertController.start();
        LBLTopFrame frame = ucCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected void createResrcViewFrame() {
        ResrcViewController resrcViewer = new ResrcViewController(desktop);
        resrcViewer.start();
        LBLTopFrame frame = resrcViewer.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
        
    }
    
    protected void createAdvCertificate(String cert) {
            CertInfoViewFrame frame;
            if (cert.equals(POLICY_CERTIFICATE)) {
                frame = new CertInfoViewFrame(POLICY_CERTIFICATE, new PolicyCertGenerator().generate(), this);
            } else if (cert.equals(USECONDITION_CERTIFICATE)) {
                frame = new CertInfoViewFrame(USECONDITION_CERTIFICATE, new UseCondCertGenerator().generate(), this);
            } else if (cert.equals(ATTRIBUTE_CERTIFICATE)) {
                frame = new CertInfoViewFrame(ATTRIBUTE_CERTIFICATE, new AttributeCertGenerator().generate(), this);               
            } else {
                throw new RuntimeException("UNKNOWN CERTIFICATE! PLEASE CHECK INPUT STRING.");
            }
            
            java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            frame.setBounds((screenSize.width-600)/2, (screenSize.height-400)/2, 600, 400);
            frame.setVisible(true);
            frame.setLocation(100, 100);
            desktop.add(frame);
            try {
                frame.setSelected(true);
            } catch (PropertyVetoException e) {}
        
    }
    
    protected void advanceEditCertificate() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogTitle("Select the certificate you want to edit.");
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        int clientResponse = fileChooser.showDialog(this, "Select");
        
        if (clientResponse == fileChooser.APPROVE_OPTION) {
            fileChooser.getSelectedFile().getAbsolutePath();
            File file = fileChooser.getSelectedFile();
            CertInfoViewFrame frame = new CertInfoViewFrame(file.getAbsolutePath(), this);
            
            java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            frame.setBounds((screenSize.width-600)/2, (screenSize.height-400)/2, 600, 400);
            frame.setVisible(true);
            frame.setLocation(100, 100);
            desktop.add(frame);
            try {
                frame.setSelected(true);
            } catch (PropertyVetoException e) {}
        }
    }
    
    protected static void quit(int status) {
        System.exit(status);
    }
    
    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private void createAndShowGUI() {
        AkentiCertificateManager frame = new AkentiCertificateManager();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        frame.setVisible(true);
        
        File splashScreen_lock = new File(System.getProperty("user.home") + System.getProperty("file.separator") + ".akenti" 
            + System.getProperty("file.separator") + ".splashscreen");
        
        if (splashScreen_lock.exists() && splashScreen_lock.isDirectory()) {
            throw new RuntimeException(".splashscreen is a directory and should be a file!");
        }
        if (!splashScreen_lock.exists()) {
            if (loader == null) { // there may not be a context class loader
                loader = AkentiCertificateManager.class.getClassLoader();
            }
            SplashScreen ss = new SplashScreen(loader.getResource("lbl/akenti/policyManager/docs/splash_screen/AkentiSplashScreen.html"));
            JDialog d = ss.createDialog(desktop, "Welcome to the Akenti Certificate Manager");
            java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            d.setBounds((screenSize.width-634)/2, (screenSize.height-440)/2, 634, 445);
            d.setResizable(true);
            d.setVisible(true);
            
            if(!ss.getSplashScreen()) {
                try {
                    splashScreen_lock.createNewFile();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        
        
    }
    
    public static void main(String[] args) {
        
        // check JVM version, must be 1.4.x or higher
        boolean versionOK = VersionChecker.checkJavaVersion();
        
        if(!versionOK) {
            quit(1);
        }
        
        String osName = System.getProperty("os.name");
        if (!osName.equals("SunOS")) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
            catch (Throwable t) {
                throw new RuntimeException(t.getMessage());
            }
        }
        
        Security.addProvider(new Provider());
        /**
         * Schedule a job for the event-dispatching thread:
         * creating and showing this application's GUI.
         * Allow ldap
         */
        
        URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {
            public URLStreamHandler
            createURLStreamHandler(String protocol) {
                
                if (protocol.equalsIgnoreCase("ldap")) {
                    return new URLStreamHandler() {
                        protected URLConnection
                        openConnection(URL url) throws IOException {
                            throw new IOException("open ldap connection unsupported");
                        }
                    };
                }
                
                return null;
            }
        });
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                new AkentiCertificateManager().createAndShowGUI();
            }
        });
    }
}