/**
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.manager;

import java.net.URL;
import javax.swing.JDesktopPane;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.policyManager.base.GenInfo;
import lbl.akenti.policyManager.widgets.AbstractController;
import lbl.akenti.policyManager.widgets.ConfFilePicker;
import lbl.akenti.policyManager.widgets.LBLDialogFactory;
import lbl.akenti.policyManager.widgets.LBLResourcePanel;
import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.viewer.LBLPolUCChoosePanel;
import lbl.akenti.policyManager.widgets.viewer.ResrcViewConsts;
import lbl.akenti.protocolHandler.LocalProtocolHandler;
import lbl.akenti.protocolHandler.ProtocolHandler;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.ConfFileParser;

public class ResrcViewController
extends AbstractController
implements ResrcViewConsts {
    
    private LBLTopFrame frame;
    // panels
    private LBLResourcePanel lblResourcePanel;
    private LBLPolUCChoosePanel lblPolUCChoosePanel;
    
    private JDesktopPane desktop;
    public ResrcViewController(JDesktopPane aDesktop) {
        // We will be using two panels.
        super(2);
        desktop = aDesktop;
    }
    
    public void start() {
        
        frame = LBLTopFrame.getInstance("Resource Viewer");
        
        frame.setController(this);
        
        // initialize panels
        lblResourcePanel = new LBLResourcePanel(frame, HEADERS[RESOURCE_INDEX]);
        lblPolUCChoosePanel = new LBLPolUCChoosePanel(frame, HEADERS[POL_UC_INDEX], desktop);
        
        // add panels in this specific order
        frame.addPanel(lblResourcePanel, "LBLResourcePanel");
        frame.addPanel(lblPolUCChoosePanel, "LBLPolUCChoosePanel");
        
    }
    
    public LBLTopFrame getFrame() {
        return frame;
    }
    
    public void clearLBLResourcePane() {
        lblResourcePanel.clear();
    }
    
    public boolean loadInputsIntoPanel(int index) throws Exception {
        switch (index) {
            case POL_UC_INDEX:
                URL base = new URL(lblResourcePanel.getBaseURL());
                String resource = lblResourcePanel.getResource();
                
                ProtocolHandler eng = null;
                if (base.getProtocol().equals("file")) {
                    if (ConfFileParser.getResources().length == 0) {
                        LBLDialogFactory.showInfoDialog(null,"No Config loaded!","You must load a configuration file with root resource mappings to browse locally");
                        ConfFilePicker cfPanel = new ConfFilePicker();
                        cfPanel.createDialog(frame, "Please load a configuration file.").setVisible(true);
                    }
                    eng = new LocalProtocolHandler();
                } else {
                    eng = new RemoteProtocolHandler(base);
                }
                CheckPolicyRequest cpr = new CheckPolicyRequest(RemoteProtocolHandler.getID(), resource);
                CheckPolicyResponse resp = eng.process(cpr);
                if (resp.getMessage().getCode() >= 200) {
                  LBLDialogFactory.showInfoDialog(null,"Error!","There was an error with your request (perhaps the resource you requested doesn't exist).");
                 return false;
                }
                
                //ResourcePolicy resPol = new ResourcePolicy();
                lblPolUCChoosePanel.clear();
                lblPolUCChoosePanel.setCheckPolicyResponse(resp);
                if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
                    // Make sure we load the GenInfo first.
                    GenInfo tempInfo = new GenInfo(resource, base);
                    tempInfo.loadResFileInfo();
                    tempInfo.loadPolicies();
                    this.info = tempInfo;
                    
                    lblPolUCChoosePanel.setCAInfos(info);
                }
                lblResourcePanel.save();
                return false;
            default:
                throw new RuntimeException();
        }
    }
    
    /**
     * Returns help message for frame
     *
     * @param index index of current frame
     * @return String containing help message for corresponding frame
     */
    protected String getHelpMessage(int index) {
        return helpMess[index];
    }
    
    public void showDialog(String title, String info) {
        frame.showDialog(title, info);
    }
    
    public char[] getPassword() {
        return null;
    }
    
    public String getP12Filename() {
        return null;
    }
    
    public String getSaveLocation() {
        return null;
    }
    
    public void cancel() {
        frame.cancel();
    }
    
    /**
     *   Used to call doEnable() on panels
     *
     *   param int index the index of the panel
     */
    public void checkEnableButton(int index) {
        switch(index) {
            case POL_UC_INDEX:
                break;
                
            default:
                
                throw new RuntimeException("Error in ResrcViewController.checkEnable");
        }
    }
    
}