package lbl.akenti.policyManager.widgets;

import java.io.FileOutputStream;
import java.util.Properties;
import lbl.akenti.certs.AkentiCertGenerator;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.policyManager.base.CredentialInfo;
import lbl.akenti.policyManager.base.GenCa;
import lbl.akenti.policyManager.base.GenInfo;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.Util;

/*
 * Provides common functionality to all controllers.
 */
abstract public class AbstractController {
    protected GenInfo info;
    protected AkentiCertGenerator gen; // This must be set by the subclass
    
    /*
     * constructor
     *
     * @param  numOfFrames number of frames in the whole interface
     */
    protected AbstractController(int numOfFrames)   {
        //Allow https urls
        System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
    }
    
    abstract protected boolean loadInputsIntoPanel(int nextFrameIndex) throws Exception;
    
    /**
     * Subclasses must return the help message corresponding
     * to the frame at index <code>ind</code>.
     *
     */
    abstract protected String getHelpMessage(int currentFrameIndex);
    abstract public char[] getPassword();
    abstract public String getP12Filename();
    abstract public String getSaveLocation();
    abstract public void clearLBLResourcePane();
    abstract public void cancel();
    abstract public void checkEnableButton(int index);
    abstract public void showDialog(String title, String info);
    /**
     */
    protected GenCa[] getGenCas() {
        return info.getGenCas();
    }

    protected void signAndSave() {
        try {
            P12Reader p12Reader = new P12Reader();
            CredentialInfo info = CredentialInfo.getInstance();
            
            if (info.isInitCredential()) {
                gen.setIssuer(
                new AkentiIssuer(
                new AkentiPrincipal(info.getSubjectDN(),info.getIssuerDN())
                )
                );
            } else {
                p12Reader.readP12File(getP12Filename(), getPassword());
                // XXX
                gen.setIssuer(
                new AkentiIssuer(
                new AkentiPrincipal(p12Reader.getSubjectDN(),p12Reader.getIssuerDN())
                )
                );
            }
            
            AkentiCertificate cert = gen.generate();
            
            String hash = Util.createHashName(cert.getHashString());
            
            gen.setUniqueID(Util.createUniqueID(hash));
            
            System.setProperty("MD5withRSA", "RSA-MD5");
            
            if (info.isInitCredential()) {
                gen.sign("MD5withRSA", info.getPrivateKey());
            } else {
                gen.sign("MD5withRSA", p12Reader.getPrivateKey());
            }
            
            String fileName = getSaveLocation();
            
            // XXX
            fileName += '.' + hash + ".";
            fileName = Util.createFileName(fileName);
            
            FileOutputStream out = new FileOutputStream(fileName);
            
            AkentiCertificateFactory.writeXMLCertToStream(cert, out);
            
            showDialog(
            "Info",
            "Certificate " + fileName + " has been saved"
            );
            
            cancel();
            
        } catch(RuntimeException e) {
            e.printStackTrace();
        } catch(Exception e) {
            e.printStackTrace();
            showDialog("Error", e.getMessage());
        }
    }
    
}