/*
 * AkentiResourceChooser.java
 *
 * Created on July 19, 2004, 10:00 AM
 */

package lbl.akenti.policyManager.widgets;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Properties;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileSystemView;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ConfFileParser;

/**
 *
 * @author Tommaso Sciortino
 */
public class AkentiResourceChooser extends JFileChooser {
  
  private final static String ps = System.getProperty("file.separator");
  
  public AkentiResourceChooser(URL base) throws IOException {
    super(new AkentiResourceSystemView(base));
    
    if (!base.getProtocol().equals("file") &&
    !base.getProtocol().equals("http") &&
    !base.getProtocol().equals("https")) {
      throw new IOException("COULD NOT PARSE URI PROTOCOL");
    }
    
    setDialogType(JFileChooser.OPEN_DIALOG);
    setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    setDialogTitle("Open " + base.toString());
  }
  
  public void setCurrentDirectory(String resource){
    setCurrentDirectory(new AkentiResourceFile(ps + resource,getBaseURL()));
  }
  
  public URL getBaseURL() {
    return ((AkentiResourceSystemView) getFileSystemView()).getBaseURL();
  }
  
  public String getBase() {
    return getBaseURL().toString();
  }
  
  public String getSelectedResourceName() throws IOException{
    File file = super.getSelectedFile();
    
    if (file == null) {
      return "";
    }
    
    String resName = file.getPath();
    if(resName.startsWith(ps) ) {
      resName = resName.substring(1);
    }
    return resName;
  }
  
}

class AkentiResourceSystemView extends FileSystemView {
  
  private final static String ps = System.getProperty("file.separator");
  private final AkentiResourceFile VISIBLE_ROOT;
  private URL baseURL;
  
  AkentiResourceSystemView(URL baseURL) {
    VISIBLE_ROOT = new AkentiResourceFile(ps, baseURL);
    this.baseURL = baseURL;
  }
  
  public boolean isRoot(File f) {
    return f.getPath() == ps;
  }
  
  public URL getBaseURL() {
    return baseURL;
  }
  
  public File createNewFolder(File dir) throws IOException {
    /* XXX: this was implemented but should never be called. To allow users to 
     * make new resources, change the canWrite method in AkentiResourceFile
     */
    if (baseURL.getProtocol().equals("file")) {
      AkentiResourceFile newFile = new AkentiResourceFile(dir, "NewFile", baseURL);
      if (!newFile.mkdir() ) {
        throw new IOException("\nCOULD NOT CREATE FILE");
      }
      return newFile;
    } else if (baseURL.getProtocol().equals("https")) {
      if (dir.getPath() == ps) {
        throw new IOException("\nROOT RESOURCES CANNOT BE CREATED WITH THIS TOOL");
      }
      
      String name = JOptionPane.showInputDialog("Name of Resource:");
      
      if (name == null || name.equals("")) {
        throw new IOException("\nRESOURCE MUST HAVE NAME");
      }
      
      String newResPath = dir.getAbsolutePath() + ps + name;
      if(newResPath.startsWith(ps) ) {
        newResPath = newResPath.substring(1);
      }
      
      RemoteProtocolHandler eng = new RemoteProtocolHandler(baseURL);
      CreateResourceRequest req = new CreateResourceRequest(RemoteProtocolHandler.getID(), newResPath);
      CreateResourceResponse resp = eng.process(req);
      
      if (resp.getMessage().getCode() > 200) {
        throw new IOException("\nCOULD NOT CREATE RESOURCE: \n" + resp.getMessage().getMessage());
      }
      
      return new AkentiResourceFile(newResPath,baseURL);
    } else if (baseURL.getProtocol().equals("http")) {
      throw new IOException("\nCLIENT MUST BE USING HTTPS TO CREATING RESOURCE");
    } else {
      throw new IOException("\nUNRECOGNIZED PROTOCOL");
    }
  }
  
  public File[] getRoots() {
    return new File[] { VISIBLE_ROOT };
  }
  
  public File getHomeDirectory() {
    return VISIBLE_ROOT;
  }
  
  public File getDefaultDirectory() {
    return VISIBLE_ROOT;
  }
  
  public File createFileObject(File dir, String filename) {
    return new AkentiResourceFile(dir, filename, baseURL);
  }
  
  public File createFileObject(String path) {
    return new AkentiResourceFile(path,baseURL);
  }
  
  public File getParentDirectory(File dir) {
    if (dir == null) {
      return null;
    }
    return dir.getParentFile();
  }
  
  int x = 0;
  
  public synchronized File[] getFiles(File dir, boolean useFileHiding) {
    /* HACK:
     * We get called three times during initialization (JFileChooser)
     * in three different thread and since our server isn't thread safe
     * we just skip listing the first two times. This may change as sun changes
     * the implementation for JFileChooser so we should keep this in mind.
     */
    
    if (x < 2) {
      x++;
      return new File[0];
    }
    
    return dir.listFiles();
  }
  
}

class AkentiResourceFile extends File implements AkentiConstants {
  private final static String ps =  System.getProperty("file.separator");
  private URL baseURL;
  private File realFile;
  
  AkentiResourceFile(File dir, String fileName, URL baseURL) {
    super(dir,fileName);
    this.baseURL = baseURL;
    if (baseURL.getProtocol().equals("file")) {
      realFile = new File(baseURL.getPath() + ps + dir.getPath(), fileName);
    }
  }
  
  AkentiResourceFile(String path, URL baseURL) {
    super(path);
    this.baseURL = baseURL;
    if (baseURL.getProtocol().equals("file")) {
      realFile = new File(baseURL.getPath() + ps + path);
    }
  }
  
  
  public boolean canWrite() {
    return false;
  }

  public boolean isDirectory() {
    return true;
  }
  
  public boolean exists() {
    if (baseURL.getProtocol().equals("file")) {
      return realFile.exists();
    } else {
      return true;
    }
  }
  
  public boolean isFile() {
    return false;
  }
  
  public File getParentFile() {
    if (super.getParentFile() == null) {
      return null;
    } else {
      return new AkentiResourceFile(super.getParentFile().getPath(), baseURL);
    }
  }
  
  public File[] listFiles(){
    if (baseURL.getProtocol().equals("file")) {
      return listFileResources();
    } else {
      try {
        return listHttpResources();
      } catch(Exception ex) {
        LBLDialogFactory.showInfoDialog(null,
        ex.getClass().getName(),
        ex.getMessage());
        return new File[0];
      }
    }
  }
  
  private File[] listFileResources() {
    File[] subs = realFile.listFiles();
    ArrayList temp = new ArrayList();
    
    if (!super.getPath().equals(ps)) {
      for (int i=0; i < subs.length; i++) {
        if (subs[i].isDirectory()) {
          String name = subs[i].getName();
          temp.add(new AkentiResourceFile(this, name, baseURL));
        }
      }
      
    } else {
      
      File f = new File(baseURL.getPath());
      String[] resNames = ConfFileParser.getPathResources(f);
      
      for (int i=0; i < resNames.length; i++) {
        temp.add(new AkentiResourceFile(this, resNames[i], baseURL));
      }
      
      if (temp.size() == 0) {
        String policyFileName = System.getProperty("PolicyFileName", PolicyFileName);
        
        for (int i=0; i < subs.length; i++) {
          File[] ls = subs[i].listFiles();
          if (ls == null) {
            break;
          }
          for (int j=0; j < ls.length; j++) {
            if (ls[j].getName().equals(policyFileName)) {
              String name = subs[i].getName();
              temp.add(new AkentiResourceFile(this, name, baseURL));
            }
          }
        }
      }
      
      if (temp.size() == 0) {
        for (int i=0; i < subs.length; i++) {
          if (subs[i].isDirectory()) {
            String name = subs[i].getName();
            temp.add(new AkentiResourceFile(this, name, baseURL));
          }
        }
      }
      
    }
    
    return (AkentiResourceFile[]) temp.toArray(new AkentiResourceFile[temp.size()]);
  }
  
  private File[] listHttpResources() throws IOException {
    String resName = super.getPath();
    if(resName.startsWith(ps) ) {
      resName = resName.substring(1);
    }
    
    RemoteProtocolHandler eng = new RemoteProtocolHandler(baseURL);
    ResourceBrowseRequest req = new ResourceBrowseRequest(RemoteProtocolHandler.getID(), resName);
    ResourceBrowseResponse resp = eng.process(req);
    
    if (resp.getMessage().getCode() > 200) {
      throw new IOException("THERE WAS AN ERROR COMMUNICATING WITH THE SERVER:\n"
      + resp.getMessage().getMessage());
    }
    
    String[] dirFileNames = resp.getResources();
    File[] dirFiles = new File[dirFileNames.length];
    
    for (int i=0; i < dirFiles.length; i++) {
      dirFiles[i] = new AkentiResourceFile(this, dirFileNames[i], baseURL);
    }
    
    return dirFiles;
  }
  
  public boolean mkdir() {
    if (baseURL.getProtocol().equals("file")) {
      return realFile.mkdir();
    } else if (baseURL.getProtocol().equals("http")) {
      
      try {
        if (super.getPath() == ps) {
          throw new IOException("\nROOT RESOURCES CANNOT BE CREATED WITH THIS TOOL");
        }
        
        String name = JOptionPane.showInputDialog("Name of Resource:");
        
        if (name == null || name.equals("")) {
          throw new IOException("\nRESOURCE MUST HAVE NAME");
        }
        
        AkentiResourceFile newRes = new AkentiResourceFile(this,name,baseURL);
        
        String newResPath = newRes.getPath();
        if(newResPath.startsWith(ps) ) {
          newResPath = newResPath.substring(1);
        }
        
        RemoteProtocolHandler eng = new RemoteProtocolHandler(baseURL);
        CreateResourceRequest req = new CreateResourceRequest(RemoteProtocolHandler.getID(), newResPath);
        CreateResourceResponse resp = eng.process(req);
        
        return resp.getMessage().getCode() < 200;
      } catch(Exception ex) {
        LBLDialogFactory.showInfoDialog(null,
        ex.getClass().getName(),
        ex.getMessage());
        return false;
      }
    } else {
      return false;
    }
  }
  
  public File getCanonicalFile() throws IOException {
    return this;
  }
  
}