/*
 * ConfFilePicker.java
 *
 * Created on August 9, 2004, 2:18 PM
 */

package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.util.ConfFileParser;

/**
 * This program allows the user to load a configuration file.
 * @author Tommaso Sciortino
 */
public class ConfFilePicker extends JOptionPane {
  private JList confList;
  private JTextField location;
  private String confFileName = "";
  private XMLHelper xmlHelper;
  private DefaultListModel model;
  
  /** Creates new form ConfFilePicker */
  
  public ConfFilePicker() {
    xmlHelper = new XMLHelper(System.getProperty("user.home") + System.getProperty("file.separator") + ".akenti"
    + System.getProperty("file.separator") + "akenti_CONFFILEHISTORY.xml");
    
    
    initComponents();
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents() {
    setLayout(new BorderLayout());
    
    GridBagConstraints gridBagConstraints;
    
    JPanel jPanel1 = new JPanel();
    location = new JTextField();
    JButton browse = new JButton();
    
    JPanel jPanel2 = new JPanel();
    JScrollPane listScroll = new JScrollPane();
    
    confList = getList();
    location.setText((String) confList.getSelectedValue());
    
    JPanel jPanel3 = new JPanel();
    JButton ok = new JButton();
    JButton cancel = new JButton();
    
    browse.setText("Browse");
    browse.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        browseActionPerformed(evt);
      }
    });
    
    jPanel1 = WrapUtil.wrapComponents(location,browse);
    
    confList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
        confListValueChanged(evt);
      }
    });
    
    listScroll.setViewportView(confList);
    confList.getInputMap().put(KeyStroke.getKeyStroke("DELETE"), "deleteItem");
    confList.getActionMap().put("deleteItem",
        new AbstractAction("deleteItem") {
            public void actionPerformed(ActionEvent evt) {
                deleteItem();
            }
        }
    );

    jPanel2 = WrapUtil.wrapVertical(jPanel1,listScroll);
    
    add(jPanel2, BorderLayout.CENTER);
    
    jPanel3.setLayout(new FlowLayout());
    
    ok.setText("OK");
    ok.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        okActionPerformed(evt);
      }
    });
    
    jPanel3.add(ok);
    
    cancel.setText("Cancel");
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        CancelActionPerformed(evt);
      }
    });
    
    jPanel3.add(cancel);
    
    add(jPanel3, BorderLayout.SOUTH);
    add(new JLabel("Please load a configuration file."), BorderLayout.NORTH);
    
  }
  
  private void confListValueChanged(ListSelectionEvent evt) {
    location.setText((String) confList.getSelectedValue());
  }
  
  private void CancelActionPerformed(ActionEvent evt) {
    setValue(GenConsts.AK_CANCEL_VALUE);
  }
  
  private void okActionPerformed(ActionEvent evt) {
    
    
    try {
      ConfFileParser.loadFromFile(location.getText());
      
      xmlHelper.encode(getAllPaths());
      
    } catch(IOException ex) {
      JOptionPane.showMessageDialog(this, "There was an error loading the configuration file.");
    }
    setValue(GenConsts.AK_CANCEL_VALUE);
  }
  
  private void browseActionPerformed(ActionEvent evt) {
    JFileChooser chooser = new JFileChooser(location.getText());
    
    if(chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      location.setText(chooser.getSelectedFile().toString());
    }
  }
  
  private JList getList() {
    model = new DefaultListModel();
    if (xmlHelper.exists()) {
      String[] data = (String[]) xmlHelper.decode();
      for (int i=0; i < data.length; i++) {
        model.addElement(data[i]);
      }
      
    }
    JList f = new JList(model);
    f.setSelectedIndex(0);
    return f;
  }
  
  private String[] getAllPaths() {
    ArrayList temp = new ArrayList();
    temp.add(location.getText());
    
    for (int i=0; i < model.getSize() && i < 10; i++) {
      String entry = (String) model.getElementAt(i);
      if (entry == null) {
        continue;
      } else if (!entry.equals(location.getText())) {
        temp.add(entry);
      }
    }
    
    return (String[]) temp.toArray(new String[temp.size()]);
  }
  
  private void deleteItem() {
    int pos = confList.getSelectedIndex();
    if (pos != -1) {
      model.remove(pos);
    }

  }
}