/*
 * HttpFileChooser.java
 *
 * Created on July 19, 2004, 10:00 AM
 */

package lbl.akenti.policyManager.widgets;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileSystemView;

/**
 * Pops a window for either a http or file view.
 *
 * @author Tommaso Sciortino
 */
public class HttpFileChooser extends JFileChooser{
  
  public HttpFileChooser(URL base) throws IOException{
    super(base.getPath());
    if (!base.getProtocol().equals("http") && !base.getProtocol().equals("file")) {
      throw new IOException("COULD NOT PARSE URI PROTOCOL");
    }
    
    if (base.getProtocol().equals("http")) {
      setFileSystemView(new HttpSystemView(base));
      setDialogType(JFileChooser.OPEN_DIALOG);
      setDialogTitle("Open " + base.toString());
      setCurrentDirectory(new HttpFile(base,true));
    }
  }
  
  public URL getSelectedURL() throws IOException{
    return super.getSelectedFile().toURL();
  }
  
}

class HttpSystemView extends FileSystemView {
  
  private final static String ps = System.getProperty("file.separator");
  private final HttpFile VISIBLE_ROOT;
  private URL baseURL;
  
  HttpSystemView(URL baseURL) {
    VISIBLE_ROOT = new HttpFile(baseURL, true);
    this.baseURL = baseURL;
  }
  
  public boolean isRoot(File f) {
    return f.getPath().equals(ps);
  }
  
  public URL getBaseURL() {
    return baseURL;
  }
  
  public File[] getRoots() {
    return new File[] { VISIBLE_ROOT };
  }
  
  public File getHomeDirectory() {
    return VISIBLE_ROOT;
  }
  
  public File getDefaultDirectory() {
    return VISIBLE_ROOT;
  }
  
  public File createFileObject(File dir, String fileName) {
    try {
      URL url = dir.toURL();
      return new HttpFile(url, url.getPath() + ps + fileName, true);
    } catch (MalformedURLException ex) {
      LBLDialogFactory.showInfoDialog(null,
      ex.getClass().getName(),
      ex.getMessage());
    }
    return null;
  }
  
  public File createFileObject(String path) {
    try {
      return new HttpFile(baseURL, path, true);
    } catch (MalformedURLException ex) {
      LBLDialogFactory.showInfoDialog(null,
      ex.getClass().getName(),
      ex.getMessage());
    }
    return null;
  }
  
  public File getParentDirectory(File dir) {
    if (dir == null) {
      return null;
    }
    
    return dir.getParentFile();
  }
  
  int x = 0;
  
  public synchronized File[] getFiles(File dir, boolean useFileHiding) {
    if (x < 2) {
      x++;
      return new File[0];
    }
    
    try {
      return ((HttpFile) dir).listHttpFiles();
    } catch(Exception ex) {
      LBLDialogFactory.showInfoDialog(null,
      ex.getClass().getName(),
      ex.getMessage());
    }
    
    return new File[0];
  }
  
  public File createNewFolder(File containingDir) throws IOException {
    return null;
  }
  
}

class HttpFile extends File {
  private final static String ps =  System.getProperty("file.separator");
  private URL url;
  private boolean isDir;
  
  HttpFile(URL url, boolean isDir) {
    super(url.getPath());
    this.url = url;
    this.isDir = isDir;
  }
  
  HttpFile(URL baseURL, String path, boolean isDir) throws MalformedURLException{
    super(path);
    this.url = new URL(baseURL.getProtocol(),baseURL.getHost(),baseURL.getPort(),path);
    this.isDir = isDir;
  }
  
  public boolean canWrite() {
    return false;
  }
  
  public boolean isDirectory() {
    return isDir;
  }
  
  public boolean exists() {
    return true;
  }
  
  public boolean isFile() {
    return !isDir;
  }
  
  public File getParentFile() {
    if (super.getParentFile() == null) {
      return null;
    } else {
      try {
        return new HttpFile(url, super.getParent(), true);
      } catch (MalformedURLException ex) {
        LBLDialogFactory.showInfoDialog(null,
        ex.getClass().getName(),
        ex.getMessage());
      }
    }
    return null;
  }
  
  public File[] listHttpFiles() throws MalformedURLException, IOException{
    BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
    
    ArrayList dirs = new ArrayList();
    String line;
    boolean parent  = true;
    
    while ((line = reader.readLine()) != null) {
      
      int idx1 = line.indexOf("HREF=\"");
      int idx2 = line.indexOf("\"", idx1 + 6);
      
      if(idx1 == -1 || idx2 == -1) {
        continue;
      }
      
      String child = line.substring(idx1 + 6, idx2);
      if (child.startsWith("http://") || child.startsWith("..") || child.startsWith("/")) {
        continue;
      }      
      
      if (line.indexOf("DIR") != -1) {
        dirs.add(new HttpFile(url, url.getPath() + ps + child, true));
      } else if (line.indexOf("[") != -1 && line.indexOf("]") != -1){
        dirs.add(new HttpFile(url, url.getPath() + ps + child, false));
      }
    }
    
    return (File[]) dirs.toArray(new File[0]);
    
  }
  
  public File getCanonicalFile() throws IOException {
    return this;
  }
  
  public URL toURL() {
    return url;
  }
}