package lbl.akenti.policyManager.widgets;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;

/**
 * Allows the user to create a url or a directory.
 *
 * @see javax.swing.JPanel
 */
public class LBLDirField extends JPanel {
  private String[] protocols;
  private JComboBox comboBox;
  
  private DocumentListener docListener;
  
  private LBLBuildDirPane optionPane;
  private JButton createButton = new JButton("Create");
  
  /**
   * Constructor
   */
  public LBLDirField() {
    this(null);
  }
  
  public LBLDirField(String label) {
    this(label, null, null);
  }
  
  public LBLDirField(String label, String []protocols, ActionListener listener) {
    if (protocols != null) {
      this.protocols = protocols;
    }
    
    comboBox = new JComboBox();
    comboBox.setEditable(true);
    
    createButton.setActionCommand("create");
    createButton.addActionListener(listener);
    LBLButtonPanel.enableBlueButton(createButton);
    
    if (label == null) {
      WrapUtil.wrapHorizontalIn(this, null, comboBox, createButton);
      return;
    }
    
    JLabel temp = new JLabel(label);
    
    temp.setForeground(Color.red);
    WrapUtil.wrapHorizontalIn(this, temp, comboBox, createButton);
  }
  /**
   * REMOVE HINT BUTTON FROM THE "Directory Build" pane.  This is normally
   * supposed to be visible except during the Resource Panel.
   */
  public LBLBuildDirPane getDirOptionPane() {
    if (optionPane == null) {
      optionPane = new LBLBuildDirPane(protocols);
      optionPane.setHintButtonStatus(false); // resource panel doesnt need hints
    }
    
    return optionPane;
  }
  
  public void addDocumentListener(DocumentListener docListener)  {
    JTextField field = (JTextField) comboBox.getEditor().getEditorComponent();
    
    field.getDocument().addDocumentListener(docListener);
  }
  
  void setDir(String dir) {
    comboBox.addItem(dir);
  }
  
  String getDir() {
    return (String) comboBox.getEditor().getItem();
  }
  
  /**
   */
  public void clear() {
    setDir("");
  }
  
  public void addItem(Object obj) {
    if (comboBox != null) {
      String path = (String) obj;
      comboBox.addItem(path);
    } else {
      throw new RuntimeException("Variable comboBox never instantiated, try constructor isComboBox set to true.");
    }
    
    
  }
  
  public JComboBox getComboBox() {
    return comboBox;
  }
}
