package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import javax.naming.NamingException;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import lbl.akenti.util.LDAPSearchUtil;

/**
 * Allows to select one or more entities and returns
 * the selected entities x509 certificates.
 */
public class LBLLdapPanel
extends JOptionPane
implements ActionListener {
  
  private JTextField baseTextField = new JTextField();
  private JTextField cnTextField = new JTextField(25);
  private JPanel searchPanel;
  
  private URL url;
  
  public LBLLdapPanel(String instructions) {
    setLayout(new BorderLayout(5, 5));
    
    JLabel labels[] = {new JLabel("SearchBase"), new JLabel("CommonName containins") };
    
    for(int i = 0; i < labels.length; i++) {
      labels[i].setForeground(Color.red);
    }
    
    baseTextField.setEditable(false);
    
    JButton searchButton = new JButton("Search");
    
    //searchButton.setBackground(Color.blue);
    //searchButton.setForeground(Color.white);
    searchButton.setActionCommand("search");
    searchButton.addActionListener(this);
    
    JComponent fields[] = {
      baseTextField,
      WrapUtil.wrapComponents(cnTextField, searchButton)
    };
    
    searchPanel = WrapUtil.wrapComponents(labels,fields);
    add(searchPanel, "Center");
    
    JButton cancelButton = new JButton("Cancel");
    JPanel cancelPanel = new JPanel();
    
    cancelPanel.add(cancelButton);
    cancelButton.setActionCommand("cancel");
    cancelButton.addActionListener(this);
    add(cancelPanel, "South");
  }
  
  /**
   */
  public void setURL(URL url) {
    String base = url.getFile();
    baseTextField.setText(url.getFile().substring(1));
    this.url = url;
  }
  
  /**
   */
  public URL getURL() {
    return url;
  }
  
  private static String PRINCIPAL_MESSAGE =
  "<html><font size=+1 color=blue>Looking for X509 certificates...</font>\n\n" +
  "<li><font color=blue>Select one more principals" +
  "<li><font color=blue>Click <i>ok</i> to load x509 certificates";
  
  /**
   *
   */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    
    if (cmd.equals("search")){
      try {
        StringBuffer sb = new StringBuffer(baseTextField.getText());
        
        String base = sb.toString();
        String temp = cnTextField.getText();
        
        String filter = null;
        
        if (temp.length() == 0) {
          filter = "CN=*";
        } else {
          filter = "CN=*" + temp + "*";
        }
        
        URL tempURL = new URL("ldap", url.getHost(), url.getPort(), base);
        String[] userList = LDAPSearchUtil.loadCns(tempURL, filter);
        
        if (userList.length == 0) {
          LBLDialogFactory.createInfoDialog(
          this,
          "Searching Failed",
          "<html><font color=blue>No user found<BR>" + "BASE:" + base + "<BR>FILTER:" + filter
          ).setVisible(true);
          
          return;
        }
        
        Object[] aps = LBLDialogFactory.showListDialog(
        this,
        "Principal Selection Dialog",
        PRINCIPAL_MESSAGE,
        userList
        );
        
        if (aps == null) {
          setValue(GenConsts.AK_CANCEL_VALUE);
          return;
        }
        
        ArrayList certList = new ArrayList();
        
        for (int i = 0; i < aps.length; ++i) {
          // TODO:
          // hack around the ldap tool runnind doe's ca
          // filter = "cn=" + aps[i];
          filter = "cn=*" + aps[i];
          
          X509Certificate[] certs = LDAPSearchUtil.loadX509Certs(tempURL, filter);
          
          if (certs.length != 0) {
            certList.add(certs[0]);
          } else {
            LBLDialogFactory.showInfoDialog(
            this,
            "Ldap Search Error",
            "<html><font color=blue>No X509 cert found for<BR>" + aps[i]
            );
          }
        }
        
        if (certList.size() != 0) {
          setValue(certList.toArray(new X509Certificate[0]));
        }
      } catch(MalformedURLException ex) {
        throw new RuntimeException(ex.getMessage());
      } catch(NamingException ex) {
        LBLDialogFactory.showInfoDialog(
        this,
        "Ldap Search Error",
        "<html><font color=blue>Searching failed ...<BR>" + ex.getMessage()
        );
      }
    } else { // cmd = "cancel"
      setValue(GenConsts.AK_CANCEL_VALUE);
    }
  }
  
}