package lbl.akenti.policyManager.widgets;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import java.awt.BorderLayout;

/**
 * Superclass of all panels that are associated with
 * a LBLTopFrame.
 *
 * A border layout is used.
 *
 * The north is used for details about the panel.
 */
abstract public class LBLPanel extends JPanel {
  
  private AbstractController controller;
  private LBLTopFrame parent;
  
  protected LBLPanel(LBLTopFrame parent, String header) {
    if (parent != null) {
      this.parent = parent;
    } else {
      throw new RuntimeException("Parent was null.");
    }
    
    this.setLayout(new BorderLayout(5, 5));
    this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    
    if (header != null) {
      this.add(LBLLabelFactory.createHtmlLabel(header), "North");
    }
  }
  
  /**
   */
  public void setController(AbstractController controller) {
    this.controller = controller;
  }
  
  public LBLTopFrame getParentFrame() {
    return parent;
  }
  
  /**
   */
  public AbstractController getController() {
    return controller;
  }
  
  
  /**
   */
  protected  void showError(String message) {
    showDialog("Error", message);
  }
  
  /**
   */
  protected  void showInfo(String message) {
    showDialog("Info", message);
  }
  
  /**
   */
  protected void showDialog(String title, String info) {
    LBLDialogFactory.createInfoDialog(
    this,
    title,
    (info == null? "" : info)
    ).setVisible(true);
  }
  
  /**
   * Creates a panel with a label followed by a component on each line
   * labels[0]  components[0]
   * labels[1]  components[1] ...
   */
  protected JPanel wrapComponents(JLabel []labels, JComponent []components) {
    return WrapUtil.wrapComponents(labels, components);
  }
  
  /**
   * Wraps two components horizontally next to each other
   */
  protected JPanel wrapComponents(JComponent comp1, JComponent comp2) {
    return WrapUtil.wrapComponents(comp1, comp2);
  }
  
  /**
   * wraps two components vertically on top of each other
   */
  protected JPanel wrapVertical(JComponent comp1, JComponent comp2) {
    return WrapUtil.wrapVertical(comp1, comp2);
  }
  
  /**
   */
  protected void enableBlueButton(String cmd) {
    parent.enableBlueButton(cmd);
  }
  
  /**
   */
  protected void disableBlueButton(String cmd) {
    parent.disableBlueButton(cmd);
  }
  
  /**
   */
  protected void enableRedButton(String cmd) {
    parent.enableRedButton(cmd);
  }
  
  /**
   */
  protected void disableRedButton(String cmd) {
    parent.disableRedButton(cmd);
  }
  
  protected void enableBlueButton(JButton button) {
    LBLButtonPanel.enableBlueButton(button);
  }
  
  /**
   */
  protected void disableBlueButton(JButton button) {
    LBLButtonPanel.disableBlueButton(button);
  }
  
  /**
   */
  protected  void enableRedButton(JButton button) {
    LBLButtonPanel.enableRedButton(button);
  }
  
  /**
   */
  protected void disableRedButton(JButton button) {
    LBLButtonPanel.disableRedButton(button);
  }
  
}