package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.policyManager.base.GenCa;

/**
 *
 */
public
class LBLPrincipalListPane
extends JOptionPane
implements ActionListener, ListDataListener, ListSelectionListener {
  
  private static String PANEL_HEADER  =
  "<html><font size=+1 color=blue>Issuer Editor";
  
  private JList issuerList;
  
  private JButton addIssuerButton = new JButton("Add");
  private JButton remIssuerButton = new JButton("Remove");
  
  private LBLDirPane dirPane;
  
  private JButton okButton = new JButton("OK");
  private JButton cancelButton = new JButton("CANCEL");;
  
  private LBLPrincipalPane issuerPanel = null;
  private boolean optional;
  
  /**
   */
  public LBLPrincipalListPane() {
    this(null);
  }
  
  /**
   */
  public LBLPrincipalListPane(String header) {
    this(header, null, null);
  }
  
  /**
   * Builds this component.
   *
   */
  public LBLPrincipalListPane(String header, String issuersLabel, String dirsLabel) {
    setLayout(new BorderLayout(5, 5));
    setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    
    if (header != null) {
      add(LBLLabelFactory.createHtmlLabel(header), "North");
    }
    
    DefaultListModel model = new DefaultListModel();
    
    model.addListDataListener(this);
    
    issuerList = new JList(model);
    issuerList.setSelectionMode(2);
    issuerList.addListSelectionListener(this);
    
    addIssuerButton.addActionListener(this);
    addIssuerButton.setActionCommand("addissuer");
    enableBlueButton(addIssuerButton);
    
    remIssuerButton.addActionListener(this);
    remIssuerButton.setActionCommand("remissuer");
    disableRedButton(remIssuerButton);
    
    if (dirsLabel == null) {
      dirPane = new LBLDirPane("Directories");
    } else {
      dirPane = new LBLDirPane(dirsLabel);
    }
    
    dirPane.setListSelectionListener(this);
    dirPane.setListDataListener(this);
    
    add(
    new JSplitPane(
    JSplitPane.VERTICAL_SPLIT,
    createListPanel(issuersLabel),
    dirPane
    ),
    "Center"
    );
    
    JPanel buttonPanel = new JPanel();
    
    okButton.setActionCommand("ok");
    okButton.addActionListener(this);
    disableRedButton(okButton);
    buttonPanel.add(okButton);
    
    cancelButton.addActionListener(this);
    cancelButton.setActionCommand("cancel");
    buttonPanel.add(cancelButton);
    add(buttonPanel, "South");
  }
  
  /**
   */
  private JPanel
  createListPanel(String issuersLabel) {
    JOptionPane panel = new JOptionPane();
    
    panel.setLayout(new BorderLayout());
    
    JLabel label;
    
    if (issuersLabel == null) {
      label = new JLabel("Issuers");
    } else {
      label = new JLabel(issuersLabel);
    }
    
    label.setForeground(Color.red);
    panel.add(label, "North");
    issuerList.setForeground(Color.blue);
    panel.add(new JScrollPane(issuerList), "Center");
    
    return WrapUtil.wrapComponents(
    panel,
    WrapUtil.wrapVertical(addIssuerButton, remIssuerButton)
    );
  }
  
  /**
   */
  public void setOKLabel(String label) {
    okButton.setLabel(label);
  }
  
  /**
   */
  public void setDirsOptional(boolean optional) {
    this.optional = optional;
  }
  
  /**
   */
  private LBLPrincipalPane getPrincipalPane() {
    if (issuerPanel == null) {
      issuerPanel = new LBLPrincipalPane();
    }
    
    return issuerPanel;
  }
  
  /**
   */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    
    if (cmd.equals("addissuer")) {
      LBLPrincipalPane issuerPanel = getPrincipalPane();
      
      issuerPanel.createDialog(this, "AddIssuerDialog").setVisible(true);
      
      Object test = issuerPanel.getValue();
      
      if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
        Object[] issuers = (Object[]) test;
        addIssuers((AkentiPrincipal[]) issuers);
      }
    } else if (cmd.equals("remissuer")) {
      int []indices = issuerList.getSelectedIndices();
      DefaultListModel model = (DefaultListModel) issuerList.getModel();
      
      for (int i = indices.length - 1; i > -1; --i) {
        model.removeElementAt(indices[i]);
      }
    } else if (cmd.equals("ok")) {
      DefaultListModel model = (DefaultListModel) issuerList.getModel();
      AkentiPrincipal[] principals = new AkentiPrincipal[model.getSize()];
      
      model.copyInto(principals);
      setValue(new Object[] { principals, dirPane.getDirs() });
    } else if (cmd.equals("cancel")) {
      setValue(GenConsts.AK_CANCEL_VALUE);
    } else {
      throw new RuntimeException(cmd + " not supported");
    }
  }
  
  public void setGenCAs(GenCa[] genCas) {
    LBLPrincipalPane issuerPanel = getPrincipalPane();
    issuerPanel.setGenCAs(genCas);
  }
  /**
   */
  public void valueChanged(ListSelectionEvent e) {
    doListEnable(issuerList.getSelectedIndex() != -1);
  }
  
  /**
   */
  public void contentsChanged(ListDataEvent e) {
  }
  
  /**
   */
  public void intervalAdded(ListDataEvent e) {
    doOkEnable(numOfIssuers() != 0 && (optional || numOfDirs() != 0));
  }
  
  /**
   */
  public void intervalRemoved(ListDataEvent e)  {
    doOkEnable(numOfIssuers() != 0 && (optional || numOfDirs() != 0));
    doListEnable(issuerList.getSelectedIndex() != -1);
  }
  
  /**
   */
  public void clear() {
    ((DefaultListModel) issuerList.getModel()).clear();
    dirPane.clear();
  }
  
  /**
   */
  private void doListEnable(boolean on) {
    if (on) {
      enableRedButton(remIssuerButton);
    } else {
      disableRedButton(remIssuerButton);
    }
  }
  
  /**
   */
  private void doOkEnable(boolean on) {
    if (on) {
      enableRedButton(okButton);
    } else {
      disableRedButton(okButton);
    }
  }
  
  /**
   */
  private void enableBlueButton(JButton button) {
    LBLButtonPanel.enableBlueButton(button);
  }
  
  /**
   */
  private void disableBlueButton(JButton button) {
    LBLButtonPanel.disableBlueButton(button);
  }
  
  /**
   */
  private void enableRedButton(JButton button) {
    LBLButtonPanel.enableRedButton(button);
  }
  
  /**
   */
  private void disableRedButton(JButton button) {
    LBLButtonPanel.disableRedButton(button);
  }
  
  /**
   */
  public int numOfIssuers() {
    return issuerList.getModel().getSize();
  }
  
  /**
   */
  public AkentiPrincipal[] getIssuers() {
    DefaultListModel model = (DefaultListModel) issuerList.getModel();
    AkentiPrincipal[] issuers = new AkentiPrincipal[model.getSize()];
    
    model.copyInto(issuers);
    return issuers;
  }
  
  /**
   */
  public boolean addIssuer(AkentiPrincipal issuer) {
    return addIssuers(new AkentiPrincipal[] { issuer }) != 0;
  }
  
  /**
   */
  public int addIssuers(AkentiPrincipal[] issuers) {
    DefaultListModel model = (DefaultListModel) issuerList.getModel();
    int added = 0;
    int size = model.getSize();
    
    issuerList.clearSelection();
    
    for (int i = 0; i < issuers.length; ++i) {
      int index = model.indexOf(issuers[i]);
      
      if (index != -1) {
        issuerList.addSelectionInterval(index, index);
      } else {
        model.addElement(issuers[i]);
        issuerList.addSelectionInterval(size, size++);
        ++added;
      }
    }
    
    return added;
  }
  
  /**
   */
  public int numOfDirs() {
    return dirPane.numOfDirs();
  }
  
  /**
   */
  public Directory[] getDirs() {
    return dirPane.getDirs();
  }
  
  /**
   */
  public boolean addDir(Directory dir) {
    return dirPane.addDir(dir);
  }
  
  /**
   */
  public int addDirs(Directory[] dirs) {
    return dirPane.addDirs(dirs);
  }
  
}