package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.cert.X509Certificate;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.policyManager.base.GenCa;
import lbl.akenti.util.ParsingException;

/**
 * Allows the user to enter an <code>AkentiPrincipal</code>.
 *
 * @see javax.swing.JPanel
 */
public class LBLPrincipalPane
extends JOptionPane
implements DocumentListener, ActionListener {
  
  private final static String PRINCIPAL_PANEL_MESSAGE =
  "<html><font size=+1 color=blue>Akenti Principal Panel" +
  "<li>Enter tne <i><font color=red>distinguished</i> name of the principal" +
  "<li>Enter the <i><font color=red>distinguished</i> name of the principal's CA" +
  "<li>click on <i><font color=red>search</i> to search for x509 certs";
  
  private JTextField dnField;
  private JTextField caDnField;
  
  private JButton okButton;
  private DocumentListener docListener;
  
  private X509CertRetrieval certRetrieval;
  
  private GenCa[] cas;
  
  public LBLPrincipalPane() {
    this(PRINCIPAL_PANEL_MESSAGE);
  }
  
  /**
   * Constructor
   */
  public LBLPrincipalPane(String header) {
    setLayout(new BorderLayout());
    
    if (header != null) {
      add(LBLLabelFactory.createHtmlLabel(header), "North");
    }
    
    JLabel dnLabel = new JLabel("Name");
    
    dnField = LBLTextFieldFactory.createTextField(this);
    
    JLabel caDnLabel = new JLabel("Ca Name");
    
    caDnField = LBLTextFieldFactory.createTextField(this);
    
    JPanel temp = WrapUtil.wrapComponents(
    new JLabel[] { dnLabel, caDnLabel },
    new JTextField[] { dnField, caDnField }
    );
    
    JButton searchButton = new JButton("SEARCH");
    
    searchButton.setActionCommand("search");
    searchButton.addActionListener(this);
    temp = WrapUtil.wrapComponents(temp, searchButton);
    temp.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    add(temp, "Center");
  }
  
  public JDialog createDialog(Component parentComponent, String title) {
    if (okButton == null) {
      okButton = new JButton("OK");
      okButton.addActionListener(this);
      
      JButton cancelButton = new JButton("CANCEL");
      
      cancelButton.addActionListener(this);
      
      JPanel tempPanel = new JPanel();
      
      tempPanel.add(WrapUtil.wrapComponents(okButton, cancelButton));
      add(tempPanel, "South");
      doEnable();
    }
    
    clear();
    return super.createDialog(parentComponent, title);
  }
  
  public void setDocumentListener(DocumentListener docListener) {
    this.docListener = docListener;
  }
  
  /**
   */
  public void changedUpdate(DocumentEvent evt) {
    if (docListener != null) {
      docListener.changedUpdate(evt);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void removeUpdate(DocumentEvent evt) {
    if (docListener != null) {
      docListener.removeUpdate(evt);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void insertUpdate(DocumentEvent evt) {
    if (docListener != null) {
      docListener.insertUpdate(evt);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void setGenCAs(GenCa[] genCas) {
    cas = genCas;
  }
  /**
   */
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();
    
    try {
      if (cmd.equals("search")) {
        if (certRetrieval == null) {
          certRetrieval = new X509CertRetrieval(this);
        }
        
        X509Certificate []certs;
        if (cas == null) {
          certs = certRetrieval.loadX509Certificates();
        } else {
          certs = certRetrieval.loadX509Certificates(cas);
        }
        
        if (certs == null) {
          setValue(GenConsts.AK_CANCEL_VALUE);
          return;
        }
        
        AkentiPrincipal[] aps = new AkentiPrincipal[certs.length];
        
        for (int i = 0; i < certs.length; ++i) {
          DistinguishedName dn = new DistinguishedName(certs[i].getSubjectDN());
          DistinguishedName caDn = new DistinguishedName(certs[i].getIssuerDN());
          
          aps[i] = new AkentiPrincipal(dn, caDn);
          dnField.setText(dn.to500String());
          caDnField.setText(caDn.to500String());
        }
        
        setValue(aps);
      } else if (cmd.equals("OK")) {
        setValue(new AkentiPrincipal[] { getAkentiPrincipal() });
      } else {
        setValue(GenConsts.AK_CANCEL_VALUE);
      }
    } catch(ParsingException  ex) {
      System.err.println(ex);
      throw new RuntimeException();
    }
  }
  
  /**
   */
  private void doEnable() {
    if (okButton != null) {
      try {
        getAkentiPrincipal();
        LBLButtonPanel.enableBlueButton(okButton);
      } catch(ParsingException ex) {
        LBLButtonPanel.disableBlueButton(okButton);
      }
    }
  }
  
  public AkentiPrincipal getAkentiPrincipal() throws ParsingException {
    return new AkentiPrincipal(getDN(), getCADN());
  }
  
  /**
   */
  public DistinguishedName getDN() throws ParsingException {
    return new DistinguishedName(dnField.getText());
  }
  
  /**
   */
  public  DistinguishedName getCADN() throws ParsingException {
    return new DistinguishedName(caDnField.getText());
  }
  
  /**
   */
  public void clear() {
    setValue(null);
    dnField.setText("");
    caDnField.setText("");
  }
  
}