package lbl.akenti.policyManager.widgets;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;
import lbl.akenti.policyManager.base.CredentialInfo;

/**
 * Allows the user to enter the name of their p12 file
 * the password, and the name of the file where the
 * newly generated certificate will be stored.
 */
public class LBLSavePanel
extends LBLPanel
implements ActionListener, ItemListener, DocumentListener {
  private static final String [] LABELS = {
    "Keyfile Name",
    "Keyfile Password",
    "CertificateFile",
    "Save to Server"
  };
  
  private static final ExtensionFileFilter []FORMAT_FILTERS = {
    new ExtensionFileFilter("Entrust Profile" , "epf"),
    new ExtensionFileFilter("Netscape P12 File", "p12")
  };
  
  private JPasswordField passwordField;
  private JComboBox keyFileBox;
  private JTextField certFileField;
  private JTextField serverLocationField;
  private XMLHelper xmlHelper = new XMLHelper(System.getProperty("user.home") + System.getProperty("file.separator") + ".akenti"
  + System.getProperty("file.separator") + "akenti_P12HISTORY.xml");
  private JComponent serverComponent;
  private JLabel serverLabel;
  
  private JFileChooser chooser;
  private File keyDir;
  private File certDir;
  private String[] serverDirs;
  
  /**
   * Builds this frame.
   *
   * @param parent this window's parent
   * @param header the title of this window
   */
  public LBLSavePanel(LBLTopFrame parent, String header) {
    super(parent, header);
    
    JLabel []labels = new JLabel[LABELS.length];
    
    for (int i = 0; i < labels.length; i++) {
      labels[i] = new JLabel(LABELS[i]);
      labels[i].setForeground(Color.red);
    }
    
    serverLabel = labels[3];
    
    JComponent []components  = new JComponent[LABELS.length];
    JButton browse = new JButton("Browse");
    
    browse.setActionCommand("Browse for KeyFile");
    browse.addActionListener(this);
    
    keyFileBox = new JComboBox();
    keyFileBox.setEditable(true);
    JTextField field = (JTextField) keyFileBox.getEditor().getEditorComponent();
    if (xmlHelper.exists()) {
      load();
    }
    field.getDocument().addDocumentListener(this);
    
    
    //components[0] = wrapComponents(keyFileField, browse);
    components[0] = wrapComponents(keyFileBox, browse);
    components[1] = passwordField = LBLTextFieldFactory.createPasswordField(this);
    
    browse = new JButton("Browse");
    browse.setActionCommand("Browse for CertFile");
    browse.addActionListener(this);
    certFileField = LBLTextFieldFactory.createFileTextField(this);
    
    components[2] = wrapComponents(certFileField, browse);
    
    serverLocationField = new JTextField();
    serverLocationField.setEditable(false);
    
    JCheckBox checkBox = new JCheckBox("", false);
    
    checkBox.setForeground(Color.blue);
    checkBox.setForeground(Color.white);
    checkBox.addItemListener(this);
    
    components[3] = serverComponent = wrapComponents(
    serverLocationField,
    checkBox
    );
    
    enableServerComponent(false);
    
    add(wrapComponents(labels, components), "Center");
    
    CredentialInfo credInfo = CredentialInfo.getInstance();
    
    if (credInfo.isInitCredential()) {
      passwordField.setText("******");
      passwordField.setEnabled(false);
    } else {
    }
    
  }
  
  /**
   */
  public void setServerDirectories(String[] dirs) {
    this.serverDirs = dirs;
    enableServerComponent(true);
  }
  
  /**
   */
  private void enableServerComponent(boolean enable) {
    serverComponent.setVisible(enable);
    
    if (enable) {
      serverLabel.setForeground(Color.red);
    } else {
      serverLabel.setForeground(serverLabel.getBackground());
    }
  }
  
  /**
   * Create a file chooser dialog box for selecting private-key.
   * Also used for selecting place to save certificate.
   */
  private JFileChooser getFileChooser() {
    if (chooser == null) {
      chooser = new JFileChooser();
      chooser.setFileHidingEnabled(false);
    }
    
    FileFilter filters[] =  chooser.getChoosableFileFilters();
    
    for (int i = 0; i < filters.length; i++) {
      chooser.removeChoosableFileFilter(filters[i]);
    }
    
    return chooser;
  }
  
  /**
   * Handles button clicking events (the browse buttons)
   * Opens up file choosers.
   */
  public void actionPerformed(ActionEvent evt) {
    JFileChooser chooser = getFileChooser();
    String cmd = evt.getActionCommand();
    if (cmd.equals("Browse for KeyFile")) {
      for (int i = 0; i < FORMAT_FILTERS.length; i++) {
        chooser.addChoosableFileFilter(FORMAT_FILTERS[i]);
      }
      
      chooser.setDialogTitle("Select the file that contains your private key");
      chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
      
      if (keyDir != null) {
        chooser.setCurrentDirectory(keyDir);
      }
      
      if (chooser.showDialog(this, "Select") == JFileChooser.APPROVE_OPTION) {
        File keyFile = chooser.getSelectedFile();
        
        if (keyFile != null) {
          //keyFileField.setText(keyFile.toString());
          keyFileBox.removeAllItems();
          keyFileBox.addItem(keyFile.toString());
          if (xmlHelper.exists()) {
            load();
          }
        }
      }
      
      keyDir = chooser.getCurrentDirectory();
      chooser.setSelectedFile(null);
    } else if ( cmd.equals("Browse for CertFile")) {
      chooser.setDialogTitle("Select the location to save the newly generated certificate");
      chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
      
      if (certDir != null) {
        chooser.setCurrentDirectory(certDir);
      }
      
      if (chooser.showDialog(this, "Save as") == JFileChooser.APPROVE_OPTION) {
        File certFile = chooser.getSelectedFile();
        
        if (certFile != null) {
          certFileField.setText(certFile.toString());
        }
      }
      
      certDir = chooser.getCurrentDirectory();
      chooser.setSelectedFile(null);
    } else {
      throw new RuntimeException();
    }
  }
  
  /**
   */
  public void changedUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void insertUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void removeUpdate(DocumentEvent e) {
    doEnable();
  }
  
  /**
   */
  public void doEnable() {
    boolean on;
    if (CredentialInfo.getInstance().isInitCredential() && getCertFileName().length() != 0) {
      on = true;
    } else {
      on = getKeyFileName().length() != 0 && getPassword().length != 0 && getCertFileName().length() != 0;
    }
    
    if (on) {
      enableRedButton("Sign and Save");
    } else {
      disableRedButton("Sign and Save");
    }
    
  }
  
  /**
   * Called when the checkbox is clicked
   */
  public void itemStateChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
      String loc = (String) LBLDialogFactory.showListDialog(
      this,
      "Server Location",
      "Select a location",
      serverDirs,
      ListSelectionModel.SINGLE_SELECTION
      );
      
      if (loc != null) {
        serverLocationField.setText(loc);
      } else {
        ((JCheckBox) e.getSource()).setSelected(false);
      }
    } else {
      serverLocationField.setText("");
    }
  }
  
  /**
   */
  public char[] getPassword() {
    return passwordField.getPassword();
  }
  
  /**
   */
  public String getFormat() {
    return "P12";
  }
  
  /**
   */
  public String getKeyFileName() {
    return (String) keyFileBox.getEditor().getItem();
  }
  
  /**
   */
  public String getCertFileName() {
    return certFileField.getText();
  }
  
  /**
   */
  public String getServerLocation() {
    String location = serverLocationField.getText();
    
    if (location.length() == 0) {
      return null;
    }
    
    return location;
  }
  
  /**
   */
  public void clear() {
    certFileField.setText("");
    serverDirs = null;
    enableServerComponent(false);
  }
  
  /**
   * Private class used with the file choosers to allow
   * selection of only certain types of files
   */
  private static class ExtensionFileFilter extends FileFilter {
    private String description;
    private String extension;
    
    ExtensionFileFilter(String description, String extension) {
      this.description = description;
      this.extension = extension;
    }
    
    public boolean accept(File file) {
      return file.isDirectory() || file.getName().endsWith("." + extension);
    }
    
    public String getDescription() {
      StringBuffer sb = new StringBuffer();
      
      sb.append(description);
      sb.append("(*.");
      sb.append(extension);
      sb.append(")");
      return sb.toString();
    }
  }
  
  public void load() {
    ArrayList items = (ArrayList) xmlHelper.decode();
    
    for (int x = 0; x <= items.size() - 1; x++) {
      keyFileBox.addItem(items.get(x));
    }
    
  }
  
  public void save() {
    if (xmlHelper.exists()) {
      ArrayList items = (ArrayList) xmlHelper.decode();
      if (!items.contains(keyFileBox.getSelectedItem())) {
        // add the last entry USED to the beginning of the list
        items.add(0, keyFileBox.getSelectedItem());
        xmlHelper.encode(items);
      } else {
        // add the last entry USED to the beginning of the list
        items.remove(items.indexOf(keyFileBox.getSelectedItem()));
        items.add(0, keyFileBox.getSelectedItem());
        xmlHelper.encode(items);
      }
      
    }
    else {
      createNewP12HistoryFile(true);
    }
  }
  
  public void createNewP12HistoryFile(boolean saveOne) {
    ArrayList toBeEncoded = new ArrayList();
    if (saveOne && keyFileBox != null) {
      toBeEncoded.add(keyFileBox.getSelectedItem());
    }
    xmlHelper.encode(toBeEncoded);
  }
  
}