/**
 *  JOptionPane widget which uses a JList/ListModel to hold data
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

public class MultipleInputPane extends JOptionPane {
  private JFileChooser fileChooser;
  private boolean userApproved;
  private DefaultListModel model = new DefaultListModel();
  private JButton add_button;
  private JPanel button_panel;
  private JButton cancel_button;
  private JPanel control_panel;
  private JButton edit_button;
  private JLabel label;
  private JList list;
  private JPanel main_panel;
  private JButton ok_button;
  private JButton remove_button;
  private JScrollPane scrollPane;
  // End of variables declaration
  
  public MultipleInputPane(String aLabel) {
    initComponents(aLabel);
  }
  
  private void initComponents(String aLabel) {
    setLayout(new BorderLayout());
    GridBagConstraints gridBagConstraints;
    
    button_panel = new JPanel();
    ok_button = new JButton();
    cancel_button = new JButton();
    main_panel = new JPanel();
    scrollPane = new JScrollPane();
    list = new JList(model);
    control_panel = new JPanel();
    add_button = new JButton();
    edit_button = new JButton();
    remove_button = new JButton();
    label = new JLabel();
    
    button_panel.setLayout(new GridBagLayout());
    
    ok_button.setText("OK");
    ok_button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        ok_buttonActionPerformed(evt);
      }
    });
    
    gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.insets = new Insets(10, 0, 10, 0);
    button_panel.add(ok_button, gridBagConstraints);
    
    cancel_button.setText("CANCEL");
    cancel_button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        cancel_buttonActionPerformed(evt);
      }
    });
    
    gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.insets = new Insets(10, 0, 10, 0);
    button_panel.add(cancel_button, gridBagConstraints);
    
    add(button_panel, BorderLayout.SOUTH);
    
    main_panel.setLayout(new GridLayout());
    
    scrollPane.setViewportView(list);
    
    main_panel.add(scrollPane);
    
    add(main_panel, BorderLayout.CENTER);
    
    control_panel.setLayout(new GridBagLayout());
    
    add_button.setText("Add");
    add_button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        add_buttonActionPerformed(evt);
      }
    });
    
    gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
    control_panel.add(add_button, gridBagConstraints);
    
    edit_button.setText("Edit");
    edit_button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        edit_buttonActionPerformed(evt);
      }
    });
    
    gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 2;
    control_panel.add(edit_button, gridBagConstraints);
    
    remove_button.setText("Remove");
    remove_button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        remove_buttonActionPerformed(evt);
      }
    });
    
    gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 3;
    gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
    control_panel.add(remove_button, gridBagConstraints);
    
    add(control_panel, BorderLayout.EAST);
    
    label.setText(aLabel);
    add(label, BorderLayout.WEST);
    
  }
  
  private void edit_buttonActionPerformed(ActionEvent evt) {
    InputPane inputPane = new InputPane("Folder Location: ");
    inputPane.setText((String) model.getElementAt(list.getSelectedIndex()));
    inputPane.createDialog(this, "Edit Folder Location").setVisible(true);
    // if list.getSelectedIndex() is -1 then nothing is selected JAVA 1.4 API
    if (inputPane.getUserApproved() && list.getSelectedIndex() != -1) {
      int location = list.getSelectedIndex();
      model.remove(location);
      model.insertElementAt(inputPane.getText(), location);
    }
  }
  
  private void cancel_buttonActionPerformed(ActionEvent evt) {
    userApproved = false;
    setValue(GenConsts.AK_CANCEL_VALUE);
  }
  
  private void ok_buttonActionPerformed(ActionEvent evt) {
    userApproved = true;
    setValue(GenConsts.AK_CANCEL_VALUE);
  }
  
  private void remove_buttonActionPerformed(ActionEvent evt) {
    // JAVA API 1.4.x: index = -1 if no items are selected!
    if (list.getSelectedIndex() >= 0) {
      model.remove(list.getSelectedIndex());
    }
  }
  
  private void add_buttonActionPerformed(ActionEvent evt) {
    LBLBuildDirPane optionPane = new LBLBuildDirPane(null);
    
    optionPane.clear();
    optionPane.createDialog(this, "Blah").setVisible(true);
    
    Object val = optionPane.getValue();
    
    if (val != null && val != GenConsts.AK_CANCEL_VALUE) {
      model.addElement((String) val);
    }
  }
  
  public void setFileChooserText(String title, int type) {
    fileChooser = getFileChooser();
    fileChooser.setDialogTitle(title);
    fileChooser.setFileSelectionMode(type);
  }
  
  public JFileChooser getFileChooser() {
    if (fileChooser == null) {
      fileChooser = new JFileChooser();
    }
    return fileChooser;
  }
  
  public boolean getUserApproved() {
    return userApproved;
  }
  
  public void setList(String[] aList) {
    for (int x = 0; x < aList.length; x++) {
      model.addElement(aList[x]);
    }
  }
  
  public String[] getList() {
    String [] listOfItems = new String[model.getSize()];
    for (int x = 0; x < model.getSize(); x++) {
      listOfItems[x] = (String) model.get(x);
    }
    return listOfItems;
  }
}