/**
 *  Provides a simple way to load/save java objects in/from XML files
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */
package lbl.akenti.policyManager.widgets;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Use this to convert just about any object to XML.
 */
public class XMLHelper {
  
  private String filename;
  
  /**
   * the constructor for this XMLHelper
   * @param aFilename fileName that will/does store the encoded/decoded object
   */
  public XMLHelper(String aFilename) {
    filename = aFilename;
  }
  
  /**
   * encode this object and store it
   * @param obj object to be encoded
   */
  public void encode(Object obj) {
    try {
      XMLEncoder e = new XMLEncoder(
      new BufferedOutputStream(
      new FileOutputStream(filename)));
      e.writeObject(obj);
      e.close();
    }
    catch (IOException exception) {
      exception.printStackTrace();
    }
    
  }
  
  /**
   * decode this object
   * @return Object that has been decoded
   */
  public Object decode() {
    Object obj = null;
    try {
      XMLDecoder reader = new XMLDecoder(
      new FileInputStream(filename));
      
      obj = reader.readObject();
      reader.close();
    }
    catch (IOException exception) {
    }
    return obj;
  }
  
  /**
   * Returns true if there is a file in the location where we wish
   * to encode/decode an object.
   * @return check to see if the specified file exists.
   */
  public boolean exists() {
    File file = new File(filename);
    return file.exists();
  }
  
}