package lbl.akenti.policyManager.widgets.pol;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JList;
import javax.swing.DefaultListModel;
import javax.swing.JScrollPane;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.security.cert.X509Certificate;

import lbl.akenti.policyManager.widgets.LBLButtonPanel;
import lbl.akenti.policyManager.widgets.X509CertRetrieval;
import lbl.akenti.policyManager.widgets.LBLTextFieldFactory;
import lbl.akenti.policyManager.widgets.LBLDialogFactory;
import lbl.akenti.policyManager.widgets.WrapUtil;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.policyManager.base.GenCa;
import java.text.ParseException;


/**
 */
public class CACertsList
extends JOptionPane
implements ListDataListener, ListSelectionListener, ActionListener {
    private JList certList;
    
    private ListSelectionListener listSelectionListener;
    private ListDataListener listDataListener;
    
    private JButton viewButton = new JButton("View");
    private JButton addButton = new JButton("Add");
    private JButton remButton = new JButton("Remove");
    
    private X509CertRetrieval certRetrieval;
    
    private GenCa[] cas;
    
    /**
     * Constructor
     */
    public CACertsList() {
        setLayout(new BorderLayout());
        
        JLabel label = new JLabel("X509 Chain");
        
        label.setForeground(Color.red);
        
        DefaultListModel model = new DefaultListModel();
        
        model.addListDataListener(this);
        
        certList = new JList(model);
        certList.setSelectionMode(2);
        certList.addListSelectionListener(this);
        
        addButton.setActionCommand("add");
        addButton.addActionListener(this);
        LBLButtonPanel.enableBlueButton(addButton);
        
        remButton.setActionCommand("remove");
        remButton.addActionListener(this);
        LBLButtonPanel.disableRedButton(remButton);
        
        viewButton.setActionCommand("view");
        viewButton.addActionListener(this);
        LBLButtonPanel.disableBlueButton(viewButton);
        
        add(
        WrapUtil.wrapComponents(
        WrapUtil.wrapVertical(
        new JComponent[] { label, new JScrollPane(certList) }
        ),
        WrapUtil.wrapVertical(
        new JButton[] { addButton, remButton, viewButton }
        )
        ),
        "Center"
        );
    }
    
    public void setGenCAs(GenCa[] genCas) {
        cas = genCas;
    }
    /**
     */
    public void setListSelectionListener(ListSelectionListener listSelectionListener) {
        this.listSelectionListener = listSelectionListener;
    }
    
    /**
     */
    public void setListDataListener(ListDataListener listDataListener) {
        this.listDataListener = listDataListener;
    }
    
    /**
     */
    public void actionPerformed(ActionEvent event) {
        if (event.getActionCommand().equals("view")) {
            DefaultListModel model = (DefaultListModel) certList.getModel();
            int index = certList.getSelectedIndex();
            X509Cert cert = (X509Cert) model.elementAt(index);
            
            LBLDialogFactory.showText(
            this,
            "X509 Certificate Viewer",
            "<html><font size=+1 color=blue>Certificate Authority X509 Certificate</font>\n\n",
            cert.toText()
            );
        } else if (event.getActionCommand().equals("add")) {
            if (certRetrieval == null) {
                certRetrieval = new X509CertRetrieval(this);
            }
            
            X509Certificate []certs;
            if (cas == null) {
                certs = certRetrieval.loadX509Certificates();
            }
            else {
                certs = certRetrieval.loadX509Certificates(cas);
            }
            
            if (certs != null) {
                addCerts(certs);
            }
        } else { // REMOVE
            int []indices = certList.getSelectedIndices();
            DefaultListModel model = (DefaultListModel) certList.getModel();
            
            for (int i = indices.length - 1; i > -1; --i) {
                model.removeElementAt(indices[i]);
            }
        }
    }
    
    /**
     */
    public void valueChanged(ListSelectionEvent e) {
        doEnable();
        
        if (listSelectionListener != null) {
            listSelectionListener.valueChanged(e);
        }
    }
    
    /**
     */
    public void contentsChanged(ListDataEvent e) {
        if (listDataListener != null) {
            listDataListener.contentsChanged(e);
        }
    }
    
    /**
     */
    public void intervalAdded(ListDataEvent e) {
        if (listDataListener != null) {
            listDataListener.intervalAdded(e);
        }
    }
    
    /**
     */
    public void intervalRemoved(ListDataEvent e)  {
        doEnable();
        
        if (listDataListener != null) {
            listDataListener.intervalRemoved(e);
        }
    }
    
    private void doEnable() {
        int []indices = certList.getSelectedIndices();
        
        if (indices.length != 0) {
            if (indices.length == 1) {
                LBLButtonPanel.enableBlueButton(viewButton);
            } else {
                LBLButtonPanel.disableBlueButton(viewButton);
            }
            
            LBLButtonPanel.enableRedButton(remButton);
        } else {
            LBLButtonPanel.disableBlueButton(viewButton);
            LBLButtonPanel.disableRedButton(remButton);
        }
    }
    
    /**
     */
    public void clear() {
        DefaultListModel model = (DefaultListModel) certList.getModel();
        
        model.clear();
    }
    
    /**
     */
    public int numOfCerts() {
        DefaultListModel model = (DefaultListModel) certList.getModel();
        
        return model.getSize();
    }
    
    /**
     */
    public int setCerts(X509Certificate []certs) {
        DefaultListModel model = (DefaultListModel) certList.getModel();
        
        model.clear();
        return addCerts(certs);
    }
    
    /**
     */
    public int addCerts(X509Certificate[] certs) {
        DefaultListModel model = (DefaultListModel) certList.getModel();
        int added = 0;
        int size = model.getSize();
        
        certList.clearSelection();
        
        for (int i = 0; i < certs.length; ++i) {
            int index = model.indexOf(new X509Cert(certs[i]));
            
            if (index != -1) {
                certList.addSelectionInterval(index, index);
            } else {
                model.addElement(new X509Cert(certs[i]));
                certList.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        
        return added;
    }
    
    public boolean addCert(X509Certificate cert) {
        return addCerts(new X509Certificate[] { cert }) != 0;
    }
    
    /**
     */
    public X509Certificate[] getCerts() {
        DefaultListModel model = (DefaultListModel) certList.getModel();
        X509Certificate[] certs = new X509Certificate[model.getSize()];
        
        for (int i = 0; i < certs.length; ++i) {
            certs[i] = ((X509Cert) model.elementAt(i)).dcert;
        }
        
        return certs;
    }
    
    private class X509Cert {
        X509Certificate dcert;
        
        X509Cert(X509Certificate dcert) {
            if (dcert == null) {
                throw new IllegalArgumentException();
            }
            
            this.dcert = dcert;
        }
        
        public String toString() {
            
            try {
                DistinguishedName dn =new  DistinguishedName(dcert.getSubjectDN());
                String temp = dn.getValue("CN");
                
                if (temp == null) {
                    return dn.to400String();
                }
                
                return temp;
            } catch(ParseException ex) {
                throw new RuntimeException();
            }
        }
        
        public boolean equals(Object o) {
            if (o instanceof X509Cert) {
                return dcert.equals(((X509Cert) o).dcert);
            }
            
            return false;
        }
        
        public String toText() {
            return dcert.toString();
        }
    }
}
