package lbl.akenti.policyManager.widgets.uc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;
import javax.naming.NamingException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.policyManager.base.GenAttribute;
import lbl.akenti.policyManager.base.GenCa;
import lbl.akenti.policyManager.base.GenValue;
import lbl.akenti.policyManager.base.GenX509Value;
import lbl.akenti.policyManager.widgets.GenConsts;
import lbl.akenti.policyManager.widgets.LBLAttrValPane;
import lbl.akenti.policyManager.widgets.LBLButtonPanel;
import lbl.akenti.policyManager.widgets.LBLCaListPane;
import lbl.akenti.policyManager.widgets.LBLDialogFactory;
import lbl.akenti.policyManager.widgets.LBLPanel;
import lbl.akenti.policyManager.widgets.LBLPrincipalListPane;
import lbl.akenti.policyManager.widgets.LBLTextFieldFactory;
import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.X509CertRetrieval;
import lbl.akenti.util.LDAPSearchUtil;
import lbl.akenti.util.ParsingException;

/**
 *
 */
public class UCExpressionPanel
extends LBLPanel
implements ActionListener , DocumentListener, ItemListener {
    
    private static String[] TYPES = { "AKENTI", "X509", "SYSTEM" };
    private static String[] OPS = { "=", "<", "<=", ">", ">="};
    
    private JComboBox types;
    private JComboBox ops;
    private JTextField attrField;
    private JTextField valField;
    
    private JPanel attrValPanel;
    private JTextArea expressionArea;
    
    private Vector attributes = new Vector();
    private Stack stack= new Stack();
    private Hashtable buttons = new Hashtable();
    private int count;
    private boolean notHasBeenSeen;
    
    private GenAttribute[] hintAttributes = new GenAttribute[0];
    
    /**
     * Constructor
     * sets up the GUI components of the frame.
     * All lists and text fields are empty.
     *
     * @param title title of the frame
     * @param instructions header/instructions to be displayed at top of frame
     * @param listener event handler to take care of Akenti Policy interaction
     */
    public UCExpressionPanel(LBLTopFrame parent, String header) {
        super(parent, header);
        
        JButton nextButton = parent.getButton("Next");
        nextButton.setLabel("Next");
        
        //nextButton.setBackground(Color.blue);
        //nextButton.setForeground(Color.white);
        buttons.put("Next", nextButton);
        
        JPanel temp = new JPanel(new BorderLayout(5, 5));
        
        attrField = LBLTextFieldFactory.createTextField(this);
        types = new JComboBox(TYPES);
        types.addItemListener(this);
        ops = new JComboBox(OPS);
        ops.addItemListener(this);
        valField = LBLTextFieldFactory.createTextField(this);
        
        JLabel []labels = {
            new JLabel("Attribute Type:"),
            new JLabel("Attribute Name:"),
            new JLabel("Operator:"),
            new JLabel("Value:")
        };
        
        for (int i = 0; i < labels.length; ++i) {
            labels[i].setForeground(Color.red);
        }
        
        JComponent [] comps = {
            types,
            attrField,
            ops,
            valField
        };
        
        JButton hintButton = new JButton("Use Hints");
        
        //hintButton.setBackground(Color.blue);
        //hintButton.setForeground(Color.white);
        hintButton.addActionListener(this);
        buttons.put("Use Hints", hintButton);
        attrValPanel = wrapComponents(wrapComponents(labels, comps), hintButton);
        
        temp.add(attrValPanel, "North");
        
        Border border = BorderFactory.createEmptyBorder(5, 5, 5, 5);
        
        expressionArea = new JTextArea(3, 0);
        expressionArea.setBorder(border);
        
        JPanel panel = createSecondPanel(expressionArea);
        
        temp.add(panel, "South");
        add(temp, "Center");
        doEnable();
    }
    
    private JPanel createSecondPanel(JTextArea exprArea) {
        JPanel panel = new JPanel();
        
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        
        String [] labels = { "(", "AND", "OR", "NOT", ")", "Add" , "Undo", "Clear"};
        LBLButtonPanel buttonPanel = new LBLButtonPanel(labels);
        
        JButton b = buttonPanel.getButton("Add");
        
        //b.setBackground(Color.blue);
        //b.setForeground(Color.white);
        b = buttonPanel.getButton(")");
        //b.setBackground(Color.blue);
        //b.setForeground(Color.white);
        
        buttonPanel.setActionListener(this);
        panel.add(buttonPanel);
        
        for (int i = 0; i < labels.length; i++) {
            buttons.put(labels[i], buttonPanel.getButton(labels[i]));
        }
        
        panel.add(Box.createVerticalStrut(5));
        
        JPanel temp = new JPanel(new BorderLayout(5, 5));
        JLabel label = new JLabel("Expression");
        
        label.setForeground(Color.black);
        temp.add(label, "North");
        exprArea.setEditable(false);
        temp.add(new JScrollPane(exprArea), "Center");
        panel.add(temp);
        panel.add(Box.createVerticalStrut(5));
        return panel;
    }
    
    /**
     */
    public void setHintAttributes(GenAttribute[] attrs) {
        if (attrs == null) {
            throw new IllegalArgumentException();
        }
        
        this.hintAttributes = attrs;
        doEnable();
    }
    
    /**
     */
    private void enableButton(String label, boolean enabled) {
        ((JButton) buttons.get(label)).setVisible(enabled);
    }
    
    /**
     */
    private void doEnable() {
        boolean enabled = attrValPanel.isVisible();
        
        enableButton("(", enabled && !notHasBeenSeen);
        enableButton(")", !enabled && count != 0);
        enableButton("NOT", enabled && !notHasBeenSeen && types.getSelectedItem().equals("X509"));
        enableButton("AND", !enabled);
        enableButton("OR", !enabled);
        enableButton("Next", !enabled && count == 0 || getExpression().equals(""));
        enableButton("Add", !attrField.getText().equals("") && !valField.getText().equals(""));
        enableButton("Undo", !stack.isEmpty());
        enableButton("Clear", !stack.isEmpty());
        enableButton("Use Hints", hintAttributes.length != 0);
    }
    
    /**
     */
    public void changedUpdate(DocumentEvent evt) {
        doEnable();
    }
    
    /**
     */
    public void removeUpdate(DocumentEvent evt) {
        doEnable();
    }
    
    /**
     */
    public void insertUpdate(DocumentEvent evt) {
        doEnable();
    }
    
    /**
     */
    public void itemStateChanged(ItemEvent evt) {
        if (notHasBeenSeen) {
            types.setSelectedItem("X509");
        }
        
        if (!types.getSelectedItem().equals("SYSTEM")) {
            ops.setSelectedItem("=");
        }
        
        doEnable();
    }
    
    /**
     * Handles button events.
     */
    public void actionPerformed(ActionEvent evt) {
        String cmd = evt.getActionCommand();
        
        if (cmd.equals("Add")) {
            String type = (String) types.getSelectedItem();
            String attrName = attrField.getText();
            String val = valField.getText();
            GenAttribute attr;
            
            if (type.equalsIgnoreCase("SYSTEM")) {
                attr = new GenAttribute(type, attrName);
                attr.addValue(new GenValue(val));
            } else if (type.equalsIgnoreCase("AKENTI")) {
                attr = createAkentiAttribute(attrName, val);
                
                if (attr == null) {
                    return;
                }
            } else if (type.equalsIgnoreCase("X509")) {
                Object source = evt.getSource();
                
                if (source instanceof GenCa) {
                    attr = createX509Attribute(attrName, val, (GenCa) source);
                } else {
                    attr = createX509Attribute(attrName, val, null);
                }
                
                if (attr == null) {
                    return;
                }
            } else {
                throw new RuntimeException(type + " not supported");
            }
            
            String op = (String) ops.getSelectedItem();
            
            appendToExpression(attrName + ' ' + op + ' ' + val);
            notHasBeenSeen = false;
            attrField.setText("");
            valField.setText("");
            attrValPanel.setVisible(false);
            attributes.addElement(attr);
            doEnable();
        } else if (cmd.equals("AND")) {
            appendToExpression(" && ");
            attrValPanel.setVisible(true);
            doEnable();
        } else if (cmd.equals("OR")) {
            appendToExpression(" || ");
            attrValPanel.setVisible(true);
            doEnable();
        } else if (cmd.equals(")")) {
            count--;
            appendToExpression(" )");
            doEnable();
        } else if (cmd.equals("(")) {
            count++;
            appendToExpression("( ");
            doEnable();
        } else if (cmd.equals("NOT")) {
            notHasBeenSeen = true;
            appendToExpression(" ! ");
            doEnable();
        } else if (cmd.equals("Clear")) {
            clear();
        } else if (cmd.equals("Undo")) {
            String str = ((String) stack.pop()).trim();
            
            if (str.equals("(")) {
                count--;
            } else if (str.equals(")")) {
                count++;
            } else if (str.equals("&&") || str.equals("||")) {
                attrValPanel.setVisible(false);
            } else if (str.equals("!")) {
                notHasBeenSeen = false;
            } else {
                notHasBeenSeen = !stack.isEmpty() && stack.peek().equals("NOT");
                attrValPanel.setVisible(true);
                attributes.removeElementAt(attributes.size() - 1);
            }
            
            editExpression();
            doEnable();
        } else if (cmd.equals("Use Hints")) {
            useHints();
        } else {
            throw new RuntimeException(cmd);
        }
    }
    
    /**
     * Clears all fields and lists in the frame.
     * Called when the cancel button is pressed
     */
    public void clear() {
        count = 0;
        notHasBeenSeen = false;
        stack.clear();
        attributes.clear();
        attrValPanel.setVisible(true);
        attrField.setText("");
        valField.setText("");
        expressionArea.setText("");
        doEnable();
    }
    
    /**
     */
    public String getExpression() {
        return expressionArea.getText();
    }
    
    /**
     */
    public AttributeInfo[] getAttributeInfos() {
        int size = attributes.size();
        AttributeInfo [] infos = new AttributeInfo[size];
        
        for (int i = 0; i < size; ++i) {
            GenAttribute attr = (GenAttribute) attributes.elementAt(i);
            
            if (attr.hasType("SYSTEM")) {
                GenValue genVal = (GenValue) attr.getValues()[0];
                
                infos[i] = new AttributeInfo(attr.getName(), genVal.getValue());
            } else if (attr.hasType("X509")) {
                GenX509Value genVal = (GenX509Value) attr.getValues()[0];
                
                infos[i] = new AttributeInfo(
                attr.getName(),
                genVal.getValue(),
                genVal.getCaDns()
                );
            } else if (attr.hasType("AKENTI")) {
                GenValue genVal = (GenValue) attr.getValues()[0];
                
                infos[i] = new AttributeInfo(
                attr.getName(),
                genVal.getValue(),
                genVal.getAkentiPrincipals(),
                genVal.getDirs()
                );
            } else {
                throw new RuntimeException();
            }
        }
        
        return infos;
    }
    
    /**
     * Appends an expression into the Expression text area.
     */
    private void appendToExpression(String str) {
        stack.push(str);
        expressionArea.append(str);
    }
    
    private void editExpression() {
        expressionArea.setText("");
        
        for (int i = 0; i < stack.size(); ++i) {
            expressionArea.append((String) stack.elementAt(i));
        }
    }
    
    /**
     */
    private LBLPrincipalListPane principalListPane;
    
    private static String PRINC_LIST_HEADER =
    "<html><font size=+1 color=blue>Attribute Issuers" +
    "<li><font color=blue>Must specify at least one attribute issuer" +
    "<li><font color=blue>Can specify one or more attributes directories";
    
    /**
     */
    private LBLPrincipalListPane getLBLPrincipalListPane() {
        if (principalListPane == null) {
            principalListPane = new LBLPrincipalListPane(
            PRINC_LIST_HEADER,
            "Attribute Issuers",
            "Attribute Directories"
            );
            
            principalListPane.setDirsOptional(true);
        }
        
        return principalListPane;
    }
    
    /**
     */
    private GenAttribute createAkentiAttribute(String name, String val) {
        LBLPrincipalListPane principalListPane = getLBLPrincipalListPane();
        JDialog dialog = principalListPane.createDialog(this, "Attribute Issuers");
        
        principalListPane.clear();
        dialog.pack();
        
        for (int i = 0; i < hintAttributes.length; ++i) {
            if (hintAttributes[i].hasType("AKENTI") && hintAttributes[i].hasName(name)) {
                Object[] values = hintAttributes[i].getValues();
                
                for (int j = 0; j < values.length; ++j) {
                    GenValue tempVal = (GenValue) values[j];
                    
                    principalListPane.addIssuers(tempVal.getAkentiPrincipals());
                    principalListPane.addDirs(tempVal.getDirs());
                }
            }
        }
        
        for (int i = 0; i < attributes.size(); ++i) {
            GenAttribute tempAttr = (GenAttribute) attributes.elementAt(i);
            
            if (tempAttr.hasType("AKENTI") && tempAttr.hasName(name)) {
                GenValue tempVal = (GenValue) tempAttr.getValues()[0];
                
                if (tempVal.hasValue(val)) {
                    principalListPane.addIssuers(tempVal.getAkentiPrincipals());
                    principalListPane.addDirs(tempVal.getDirs());
                }
            }
        }
        
        dialog.setVisible(true);
        
        Object ret = principalListPane.getValue();
        
        if (ret == null || ret == GenConsts.AK_CANCEL_VALUE) {
            return null;
        }
        
        GenAttribute attr = new GenAttribute("AKENTI", name);
        GenValue value = new GenValue(val);
        Object []temp = (Object[]) ret;
        
        value.addAkentiPrincipals((AkentiPrincipal[]) temp[0]);
        value.addDirs((Directory[]) temp[1]);
        attr.addValue(value);
        return attr;
    }
    
    /**
     */
    private LBLCaListPane caListPane;
    
    /**
     */
    private LBLCaListPane getLBLCaListPane() {
        if (caListPane == null) {
            caListPane = new LBLCaListPane();
        }
        
        return caListPane;
    }
    
    /**
     */
    private GenAttribute createX509Attribute(String name, String val, GenCa ca) {
        LBLCaListPane caListPane = getLBLCaListPane();
        
        caListPane.clear();
        
        if (ca != null) {
            caListPane.addCaDns(new DistinguishedName[] { ca.getDistinguishedName() } );
        }
        
        for (int i = attributes.size() - 1; i > -1; i--) {
            GenAttribute tempAttr = (GenAttribute) attributes.elementAt(i);
            
            if (tempAttr.hasName(name)) {
                GenX509Value tempVal = (GenX509Value) tempAttr.getValues()[0];
                
                if (tempVal.hasValue(val)) {
                    caListPane.addCaDns(tempVal.getCaDns());
                }
            }
        }
        
        caListPane.createDialog(this, "Certificate Authorities").setVisible(true);
        
        Object ret = caListPane.getValue();
        
        if (ret == null || ret == GenConsts.AK_CANCEL_VALUE) {
            return null;
        }
        
        DistinguishedName[] caDns = (DistinguishedName[]) ret;
        GenAttribute attr = new GenAttribute("X509", name);
        GenX509Value value = new GenX509Value(val);
        
        for (int i = 0; i < caDns.length; ++i) {
            value.addGenCa(new GenCa(caDns[i]));
        }
        
        attr.addValue(value);
        return attr;
    }
    
    /**
     */
    private LBLAttrValPane attrValPane;
    
    private static String ATTR_VAL_MESSAGE =
    "<html><font size=+1 color=blue>Attribute/Value Hints" +
    "<li><font color=blue>First select an attribute" +
    "<li><font color=blue>Then select a value";
    
    /**
     */
    private LBLAttrValPane getLBLAttrValPane() {
        if (attrValPane == null) {
            attrValPane = new LBLAttrValPane(
            ATTR_VAL_MESSAGE,
            false
            );
        }
        
        return attrValPane;
    }
    
    private static String X509_ATTR_MESSAGE =
    "<html><font size=+1 color=blue>H509 Attribute Hints" +
    "<li><font color=blue>select an attribute";
    
    
    private X509CertRetrieval certRetrieval;
    
    private X509CertRetrieval getX509CertRetrieval() {
        if (certRetrieval == null) {
            certRetrieval = new X509CertRetrieval(this);
        }
        
        return certRetrieval;
    }
    
    /**
     */
    private void useHints() {
        String type = (String) types.getSelectedItem();
        
        if (type.equalsIgnoreCase("X509")) {
            Vector temp = new Vector();
            
            for (int i = 0; i < hintAttributes.length; ++i) {
                if (hintAttributes[i].hasType("X509")) {
                    temp.addElement(hintAttributes[i]);
                }
            }
            
            GenAttribute[] attrs = new GenAttribute[temp.size()];
            
            temp.toArray(attrs);
            
            
            GenAttribute attr = (GenAttribute) LBLDialogFactory.showListDialog(
            this,
            "X509 Attributes",
            X509_ATTR_MESSAGE,
            attrs,
            ListSelectionModel.SINGLE_SELECTION
            );
            
            if (attr == null) {
                return;
            }
            
            GenX509Value x509Value = (GenX509Value) attr.getValues()[0];
            GenCa[] cas = x509Value.getGenCas();
            
            if (cas.length == 0) {
                showInfo("<html><font color=red>No Certificate Authority Infos");
                return;
            }
            
            GenCa ca = (GenCa) LBLDialogFactory.showListDialog(
            this,
            "Certificate Authorities",
            "<html><font color=blue> Select a Certificate Authority",
            cas,
            0
            );
            
            if (ca == null) {
                return;
            }
            
            Directory []dirs = ca.getDirs();
            
            if (dirs.length == 0) {
                showInfo("<html><font color=red> No Directories for selected CA");
                return;
            }
            
            Directory dir  = (Directory) LBLDialogFactory.showListDialog(
            this,
            "Directories",
            "<html><font color=blue> Select a directory",
            dirs,
            0
            );
            
            if (dir == null) {
                return;
            }
            
            try {
                URL url = new URL(dir.toString());
                String attrName = attr.getName();
                
                if (attrName.equalsIgnoreCase("OU")) {
                    if (url.getProtocol().equalsIgnoreCase("ldap")) {
                        String[] orgUnits = LDAPSearchUtil.loadOrgUnits(url);
                        String val = (String) LBLDialogFactory.showListDialog(
                        this,
                        "Organizational Units",
                        "<html><font color=blue>Select an organization unit",
                        orgUnits,
                        ListSelectionModel.SINGLE_SELECTION
                        );
                        
                        if (val != null) {
                            attrField.setText(attrName);
                            valField.setText(val);
                            actionPerformed(new ActionEvent(ca, 0, "Add"));
                        }
                        
                    } else {
                        showInfo(
                        "<html><font color=blue>Don't know how to load org units from<BR>" + url
                        );
                    }
                } else if (attrName.equalsIgnoreCase("CN")) {
                    X509CertRetrieval certRetrieval = getX509CertRetrieval();
                    X509Certificate[] certs = certRetrieval.loadX509Certificates(url);
                    
                    if (certs == null || certs.length == 0) {
                        return;
                    }
                    
                    DistinguishedName dn = new DistinguishedName(certs[0].getSubjectDN());
                    
                    String val = dn.getValue(attrName);
                    
                    attrField.setText(attrName);
                    valField.setText(val);
                    actionPerformed(new ActionEvent(ca, 0, "Add"));
                } else {
                    showInfo(
                    "<html><font color=blue>Don't know how to handle attribute " + attrName
                    );
                }
            } catch(NamingException ex) {
                showError(
                "<html><font color=red>Unable to load org units<BR>" + ex
                );
            } catch(ParsingException ex) {
                throw new RuntimeException();
            } catch(MalformedURLException ex) {
                throw new RuntimeException();
            }
            
            return;
        }
        
        LBLAttrValPane attrValPane = getLBLAttrValPane();
        
        attrValPane.clear();
        
        JDialog dialog = attrValPane.createDialog(this, "AttrValPane");
        
        dialog.pack();
        
        for (int i = 0; i < hintAttributes.length; ++i) {
            if (hintAttributes[i].hasType(type)) {
                attrValPane.addAttribute(hintAttributes[i]);
            }
        }
        
        dialog.setVisible(true);
        
        Object ret = attrValPane.getValue();
        
        if (ret != null && ret != GenConsts.AK_CANCEL_VALUE) {
            GenAttribute attribute = (GenAttribute) ret;
            
            attrField.setText(attribute.getName());
            valField.setText(attribute.getValues()[0].toString());
            actionPerformed(new ActionEvent(this, 0, "Add"));
        }
    }
}
