/**
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.widgets.viewer;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileOutputStream;
import java.security.PrivateKey;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent.EventType;
import java.awt.Component;
import lbl.akenti.certs.AkentiCertGenerator;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.policyManager.base.GenInfo;
import lbl.akenti.policyManager.base.HtmlCertWriter;
import lbl.akenti.policyManager.widgets.InputPane;
import lbl.akenti.policyManager.widgets.MultipleInputPane;
import lbl.akenti.policyManager.widgets.P12Retriever;
import lbl.akenti.policyManager.widgets.TimeInputPane;
import lbl.akenti.policyManager.widgets.ComboBoxPane;
import lbl.akenti.policyManager.widgets.ManageActionsPane;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;

import lbl.akenti.policyManager.widgets.AdvEditInputPane;
import lbl.akenti.policyManager.widgets.ManageAttributeValuePane;
import lbl.akenti.policyManager.widgets.LBLPrincipalPane;
import lbl.akenti.policyManager.widgets.GenConsts;
import lbl.akenti.policyManager.base.GenAttribute;
import lbl.akenti.policyManager.base.GenCa;

public class CertInfoViewFrame
extends JInternalFrame implements ActionListener, ViewerConsts {
    private JEditorPane editorPane;
    private JLabel mesg;
    private CertificateInfo certInfo;
    private JButton cancelButton;
    private JButton signAndSave_button;
    private AkentiCertificate akentiCert;
    private String p12Filename;
    private PrivateKey pKey;
    private GenInfo info;
    private CertificateAuthorityInfo[] cais;
    private UseCondIssuerGroup[] ucigs;
    private Component parent;
    
    public CertInfoViewFrame(String title, AkentiCertificate cert, Component parentComponent) {
        super(title,
        true,
        true,
        true,
        true);
        
        parent = parentComponent;
        this.certInfo = null;
        akentiCert = cert;
        
        initComponents();
    }
    
    public CertInfoViewFrame(String path, Component parentComponent) {
        super("",
        true,
        true,
        true,
        true);
        
        parent = parentComponent;
        this.certInfo = null;
        akentiCert = getAkentiCertificate(path);
        
        initComponents();
    }
    
    public CertInfoViewFrame(CertificateInfo certInfo, Component parentComponent) {
        super(certInfo.toString(),
        true,
        true,
        true,
        true);
        
        parent = parentComponent;
        
        if (certInfo.getAkentiMessages().length == 0) {
            mesg = new JLabel("");
        } else {
            StringBuffer buffer = new StringBuffer("<html><body>");
            AkentiMessage[] temp = certInfo.getAkentiMessages();
            for (int i=0; i < temp.length; i++) {
                buffer.append(temp[i].getMessage());
                if (i != temp.length -1) buffer.append("<br>");
            }
            buffer.append("</body></html>");
            mesg = new JLabel(buffer.toString());
        }
        
        akentiCert = certInfo.getAkentiCertificate();
        
        initComponents();
    }
    
    private void initComponents() {
        
        getContentPane().setLayout(new BorderLayout());
        
        editorPane = new JEditorPane();
        editorPane.setEditable(false);
        editorPane.setContentType("text/html");
        
        
        // reset the p12Filename
        p12Filename = null;
        pKey = null;
        
        if(akentiCert != null) {
            displayCertificate(akentiCert);
        }
        
        editorPane.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent evt) {
                hyperLinkHandler(evt);
            }
        });
        
        JScrollPane sPane =  new JScrollPane(editorPane);
        sPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        cancelButton = new JButton("Cancel");
        cancelButton.setEnabled(true);
        cancelButton.setActionCommand("Cancel");
        cancelButton.addActionListener(this);
        
        signAndSave_button = new JButton("Sign and Save");
        signAndSave_button.setEnabled(true);
        signAndSave_button.setActionCommand("Sign and Save");
        signAndSave_button.addActionListener(this);
        
        JPanel certRespPane = new JPanel();
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(cancelButton);
        buttonPanel.add(signAndSave_button);
        
        if (certInfo != null) {
            getContentPane().add(mesg, BorderLayout.NORTH);
        }
        getContentPane().add(sPane, BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    }
    private void hyperLinkHandler(HyperlinkEvent evt) {
        // if user clicks on the link
        if (evt.getEventType() == EventType.ACTIVATED) {
            
            if (akentiCert instanceof PolicyCertificate) {
                editPolicyCertificate(evt.getDescription());
            } else if (akentiCert instanceof UseCondCertificate) {
                editUseCondCertificate(evt.getDescription());
            } else if (akentiCert instanceof AttributeCertificate) {
                editAttributeCertificate(evt.getDescription());
            } else {
                throw new RuntimeException("Unknown Certificate");
            }
        }
    }
    
    public void editPolicyCertificate(String editType) {
        
        // prime the generator
        PolicyCertificate polCert = (PolicyCertificate) akentiCert;
        PolicyCertGenerator polGen = new PolicyCertGenerator();
        polGen.setVersionNumber(2);
        polGen.setUniqueID("UID");
        polGen.setIssuer(polCert.getIssuer());
        polGen.setNotBefore(polCert.getNotBefore());
        polGen.setNotAfter(polCert.getNotAfter());
        polGen.setResource(polCert.getResource());
        polGen.setCertificateAuthorityInfos(polCert.getCertificateAuthorityInfos());
        polGen.setUseCondIssuerGroups(polCert.getUseCondIssuerGroups());
        polGen.setAttributeDirectories(polCert.getAttributeDirectories());
        polGen.setCacheTime(polCert.getCacheTime());
        
        if (editType.equals(ISSUER)) {
            P12Retriever retriever = new P12Retriever();
            retriever.createDialog(this, "P12 File Authentication").setVisible(true);
            
            if(retriever.getUserApproved()) {
                P12Reader reader = new P12Reader();
                try {
                    
                    p12Filename = retriever.getP12AbsolutePath();
                    reader.readP12File(p12Filename, retriever.getP12Password());
                    pKey = reader.getPrivateKey();
                    
                    AkentiPrincipal issuer = new AkentiPrincipal(reader.getSubjectDN(), reader.getIssuerDN());
                    
                    if (issuer != null) {
                        polGen.setIssuer(new AkentiIssuer(issuer));
                    }
                } catch (java.security.GeneralSecurityException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.text.ParseException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.io.IOException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                }
            }
        } else if (editType.equals(VALIDITY_PERIOD)) {
            TimeInputPane timeInputPane = new TimeInputPane();
            timeInputPane.createDialog(this, "Enter Cache Time").setVisible(true);
            if(timeInputPane.getUserApproved()) {
                UTCTime utc = new UTCTime();
                polGen.setNotBefore(utc);
                polGen.setNotAfter(new UTCTime(utc, timeInputPane.getLifeTime()));
            }
        } else if (editType.equals(RESOURCE)) {
            InputPane inputPane = new InputPane("Enter new resource: ");
            inputPane.setText("");
            inputPane.createDialog(this, "Edit Resource").setVisible(true);
            
            if (inputPane.getUserApproved()) {
                polGen.setResource(inputPane.getText());
            }
        } else if (editType.equals(MAXIMUM_ALLOWED_CERTIFICATE_CACHING_TIME)) {
            InputPane inputPane = new InputPane("Enter new cache time: ");
            inputPane.setText("" + polCert.getCacheTime());
            inputPane.createDialog(this, "Maximum Cache Time").setVisible(true);
            
            if (inputPane.getUserApproved()) {
                polGen.setCacheTime(Long.valueOf(inputPane.getText()).longValue());
            }
        } else if (editType.equals(ATTRIBUTE_DIRECTORIES)) {
            MultipleInputPane multipleInputPane = new MultipleInputPane("Current Directories: ");
            multipleInputPane.setFileChooserText("Select an Attribute folder", JFileChooser.DIRECTORIES_ONLY);
            String[] attrsList = new String[polCert.getAttributeDirectories().length];
            for (int x = 0; x < polCert.getAttributeDirectories().length; x++) {
                // file is the protocol we're using
                attrsList[x] = polCert.getAttributeDirectories()[x].toString();
            }
            multipleInputPane.setList(attrsList);
            multipleInputPane.createDialog(this, "Attribute Directories").setVisible(true);
            
            if (multipleInputPane.getUserApproved()) {
                // get and set the new Attribute Directories
                if (multipleInputPane.getList().length > 0) {
                    attrsList = multipleInputPane.getList();
                    Directory[] directories = new Directory[attrsList.length];
                    for (int x = 0; x < attrsList.length; x++) {
                        if (!attrsList[x].startsWith("file:")) {
                            attrsList[x] = "file:" + attrsList[x];
                        }
                        directories[x] = new Directory(attrsList[x]);
                    }
                    polGen.setAttributeDirectories(directories);
                }
            }
        } else if (editType.equals(CERTIFICATE_AUTHORITY_INFOS)) {
            EditorCAInfosPanel caPanel = new EditorCAInfosPanel(EditorCAInfosPanel.CERTIFICATE_AUTHORITY_INFOS);
            
            if(cais == null) {
                if (info != null) {
                    caPanel.setGenCAs(info.getGenCas());
                    caPanel.setCAInfos(info.getCertificateAuthorityInfos());
                } else {
                    caPanel.setGenCAs(new GenCa[0]);
                    caPanel.setCAInfos(polCert.getCertificateAuthorityInfos());
                }
                
            }
            else {
                caPanel.setCAInfos(cais);
            }
            
            caPanel.createDialog(this, "Certificate Authority Information").setVisible(true);
            
            if (caPanel.getUserApproved()) {
                cais = caPanel.getCAInfos();
                polGen.setCertificateAuthorityInfos(cais);
            }
        } else if (editType.equals(USECONDITION_ISSUER_GROUPS)) {
            EditorCAInfosPanel caPanel = new EditorCAInfosPanel(EditorCAInfosPanel.USE_CONDITION_ISSUER_GROUPS);
            if (ucigs == null) {
                if (info != null) {
                    caPanel.setGenCAs(info.getGenCas());
                    caPanel.setUCGroups(info.getUseCondIssuerGroups());
                    
                } else {
                    caPanel.setGenCAs(new GenCa[0]);
                    caPanel.setUCGroups(polCert.getUseCondIssuerGroups());
                }
                
            } else {
                
                caPanel.setGenCAs(info.getGenCas());
                caPanel.setUCGroups(ucigs);
            }
            
            caPanel.createDialog(this, "Use Condition Issuer Group(s)").setVisible(true);
            
            if (caPanel.getUserApproved()) {
                ucigs = caPanel.getUseCondIssuerGroups();
                polGen.setUseCondIssuerGroups(ucigs);
            }
        } else {
            throw new RuntimeException("Unknown edit type!");
        }
        
        akentiCert = polGen.generate();
        displayCertificate(akentiCert);
    }
    
    public void editUseCondCertificate(String editType) {
        // prime the generator
        UseCondCertificate ucCert = (UseCondCertificate) akentiCert;
        UseCondCertGenerator ucGen = new UseCondCertGenerator();
        ucGen.setVersionNumber(2);
        ucGen.setUniqueID("UID");
        ucGen.setIssuer(ucCert.getIssuer());
        ucGen.setNotBefore(ucCert.getNotBefore());
        ucGen.setNotAfter(ucCert.getNotAfter());
        
        ucGen.setResource(ucCert.getResource());
        ucGen.setLocalScope(ucCert.isScopeLocal());
        
        ucGen.setCritical(ucCert.isCritical());
        ucGen.setCondition(ucCert.getCondition());
        ucGen.setActions(ucCert.getActions());
        
        if (editType.equals(ISSUER)) {
            P12Retriever retriever = new P12Retriever();
            retriever.createDialog(this, "P12 File Authentication").setVisible(true);
            
            if(retriever.getUserApproved()) {
                P12Reader reader = new P12Reader();
                try {
                    
                    p12Filename = retriever.getP12AbsolutePath();
                    reader.readP12File(p12Filename, retriever.getP12Password());
                    pKey = reader.getPrivateKey();
                    
                    AkentiPrincipal issuer = new AkentiPrincipal(reader.getSubjectDN(), reader.getIssuerDN());
                    
                    if (issuer != null) {
                        ucGen.setIssuer(new AkentiIssuer(issuer));
                    }
                } catch (java.security.GeneralSecurityException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.text.ParseException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.io.IOException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                }
            }
        } else if (editType.equals(VALIDITY_PERIOD)) {
            TimeInputPane timeInputPane = new TimeInputPane();
            timeInputPane.createDialog(this, "Enter Cache Time").setVisible(true);
            if(timeInputPane.getUserApproved()) {
                UTCTime utc = new UTCTime();
                ucGen.setNotBefore(utc);
                ucGen.setNotAfter(new UTCTime(utc, timeInputPane.getLifeTime()));
            }
        } else if (editType.equals(RESOURCE)) {
            InputPane inputPane = new InputPane("Enter new resource: ");
            inputPane.setText("");
            inputPane.createDialog(this, "Edit Resource").setVisible(true);
            
            if (inputPane.getUserApproved()) {
                ucGen.setResource(inputPane.getText());
            }
        } else if (editType.equals(SCOPE)) {
            ComboBoxPane comboBoxPane = new ComboBoxPane("Scope type: ");
            comboBoxPane.addItem("local");
            comboBoxPane.addItem("sub-tree");
            comboBoxPane.createDialog(this, "Edit Scope").setVisible(true);
            
            if (comboBoxPane.getUserApproved()) {
                String item = (String) comboBoxPane.getSelectedItem();
                if (item.equals("local")) {
                    ucGen.setLocalScope(true);
                } else if (item.equals("sub-tree")) {
                    ucGen.setLocalScope(false);
                } else {
                    throw new RuntimeException("Unknown Scope!");
                }
            }
        } else if (editType.equals(ACCESS)) {
            ComboBoxPane comboBoxPane = new ComboBoxPane("Access type: ");
            comboBoxPane.addItem("Critical");
            comboBoxPane.addItem("Not critical");
            comboBoxPane.createDialog(this, "Edit Access").setVisible(true);
            
            if (comboBoxPane.getUserApproved()) {
                String item = (String) comboBoxPane.getSelectedItem();
                if (item.equals("Critical")) {
                    ucGen.setCritical(true);
                } else if (item.equals("Not critical")) {
                    ucGen.setCritical(false);
                } else {
                    throw new RuntimeException("Unknown Access type!");
                }
            }
        } else if (editType.equals(CONSTRAINT)) {
            ExpressionBuilder ucep;
            if (info != null) {
                ucep = new ExpressionBuilder(info.getGenCas());
                ucep.setHintAttributes(info.getGenAttributes());
            } else {
                ucep = new ExpressionBuilder(new GenCa[0]);
            }
            javax.swing.JDialog d = ucep.createDialog(this, "Expression Builder");
            d.setResizable(true);
            d.setVisible(true);
            
            if (ucep.getUserApproved()) {
                String expression;
                if (ucep.getExpression() == null || ucep.getExpression().length() <= 0) {
                    expression = "()";
                } else {
                    expression = ucep.getExpression();
                }
                
                ucGen.setCondition(new lbl.akenti.certs.elements.Condition(expression));
            }
            
        } else if (editType.equals(ACTIONS)) {
            ManageActionsPane map = new ManageActionsPane();
            // actions found in UC Certificate
            map.addList(ucCert.getActions());
            if (info != null) {
                // actions from Res Attr file
                map.addList(info.getActions());
            }
            javax.swing.JDialog d = map.createDialog(this, "Manage Actions");
            d.setResizable(true);
            d.setVisible(true);
            
            if (map.getUserApproved()) {
                ucGen.setActions(map.getList());
            }
            
        } else {
            throw new RuntimeException("Unknown edit type!");
        }
        
        akentiCert = ucGen.generate();
        displayCertificate(akentiCert);
        
    }
    
    public void editAttributeCertificate(String editType) {
        AttributeCertificate attrCert = (AttributeCertificate) akentiCert;
        AttributeCertGenerator attrGen = new AttributeCertGenerator();
        attrGen.setVersionNumber(2);
        attrGen.setUniqueID("UID");
        attrGen.setIssuer(attrCert.getIssuer());
        attrGen.setNotBefore(akentiCert.getNotBefore());
        attrGen.setNotAfter(akentiCert.getNotAfter());
        
        attrGen.setPrincipal(attrCert.getPrincipal());
        attrGen.setAttribute(attrCert.getAttribute());
        attrGen.setValue(attrCert.getValue());
        
        if (editType.equals(ISSUER)) {
            P12Retriever retriever = new P12Retriever();
            retriever.createDialog(this, "P12 File Authentication").setVisible(true);
            
            if(retriever.getUserApproved()) {
                P12Reader reader = new P12Reader();
                try {
                    
                    p12Filename = retriever.getP12AbsolutePath();
                    reader.readP12File(p12Filename, retriever.getP12Password());
                    pKey = reader.getPrivateKey();
                    
                    AkentiPrincipal issuer = new AkentiPrincipal(reader.getSubjectDN(), reader.getIssuerDN());
                    
                    if (issuer != null) {
                        attrGen.setIssuer(new AkentiIssuer(issuer));
                    }
                } catch (java.security.GeneralSecurityException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.text.ParseException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                } catch (java.io.IOException e) {
                    // reset the p12Filename and p12Password variables
                    p12Filename = null;
                    pKey = null;
                    
                    showMessage(e.getMessage());
                }
            }
        } else if (editType.equals(VALIDITY_PERIOD)) {
            TimeInputPane timeInputPane = new TimeInputPane();
            timeInputPane.createDialog(this, "Enter Cache Time").setVisible(true);
            if(timeInputPane.getUserApproved()) {
                UTCTime utc = new UTCTime();
                attrGen.setNotBefore(utc);
                attrGen.setNotAfter(new UTCTime(utc, timeInputPane.getLifeTime()));
            }
        } else if (editType.equals(USER)) {
            if (info == null) {
                AdvEditInputPane advPane = new AdvEditInputPane("Name", "CaName");
                advPane.setFirstTextField(attrCert.getPrincipal().getName().to500String());
                advPane.setSecondTextField(attrCert.getPrincipal().getGuarantor().to500String());
                javax.swing.JDialog d = advPane.createDialog(this, "Enter User information");
                d.setResizable(true);
                d.setVisible(true);
                
                if (advPane.getUserApproved()) {
                    try {
                        attrGen.setPrincipal(new AkentiPrincipal(new DistinguishedName(advPane.getFirstTextField()), new DistinguishedName(advPane.getSecondTextField())));
                    } catch (lbl.akenti.util.ParsingException e) {
                        showMessage(e.getMessage());
                    }
                }
            } else {
                LBLPrincipalPane issuerPanel = new LBLPrincipalPane();
                issuerPanel.setGenCAs(info.getGenCas());
                issuerPanel.createDialog(this, "Search for a User").setVisible(true);
                Object test = issuerPanel.getValue();
                
                if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
                    // only 1 principal (USER) will be used
                    attrGen.setPrincipal(((AkentiPrincipal[]) test)[0]);
                }
            }
        } else if (editType.equals(ATTRIBUTE_VALUE_PAIR)) {
            ManageAttributeValuePane attrValPane = new ManageAttributeValuePane(null, true);
            if (info != null ) {
                
                
                GenAttribute[] attributes = info.getGenAttributes();
                
                for (int i = 0; i< attributes.length; i++) {
                    if (!attributes[i].isX509Attribute()) {
                        attrValPane.addAttribute(attributes[i]);
                    }
                }
            }
            javax.swing.JDialog d = attrValPane.createDialog(this, "Select Attribute Value Pair");
            d.setResizable(true);
            d.setVisible(true);
            
            Object test = attrValPane.getValue();
            if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
                attrGen.setAttribute(attrValPane.getAttributeName());
                attrGen.setValue(attrValPane.getValueName());
            }
        }
        
        akentiCert = attrGen.generate();
        displayCertificate(akentiCert);
    }
    
    public AkentiCertificate getAkentiCertificate(String pathToCert) {
        AkentiCertificate cert = null;
        try {
            cert = AkentiCertificateFactory.initCertFromFile("xml", pathToCert);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return cert;
        
    }
    
    public void displayCertificate(AkentiCertificate aCert) {
        editorPane.setText(new HtmlCertWriter().toHtml(aCert));
    }
    
    public void setCAInfos(GenInfo genInfo) {
        info = genInfo;
    }
    public void showMessage(String message) {
        JOptionPane.showMessageDialog(this, message, "", JOptionPane.INFORMATION_MESSAGE);
    }
    
    public CertificateInfo getCertInfo() {
        return certInfo;
    }
    
    public void actionPerformed(ActionEvent event) {
        String cmd = event.getActionCommand();
        
        if (cmd.equals("Cancel")) {
            dispose();
        } else if (cmd.equals("Sign and Save")) {
            if (akentiCert instanceof PolicyCertificate) {
                if (p12Filename == null || pKey == null) {
                    editPolicyCertificate(ISSUER);
                }
                
                if (p12Filename != null && pKey != null) {
                    signAndSave(new PolicyCertGenerator());
                }
            } else if (akentiCert instanceof UseCondCertificate) {
                if (p12Filename == null || pKey == null) {
                    editUseCondCertificate(ISSUER);
                }
                
                if (p12Filename != null && pKey != null) {
                    signAndSave(new UseCondCertGenerator());
                }
            } else if (akentiCert instanceof AttributeCertificate) {
                if (p12Filename == null || pKey == null) {
                    editAttributeCertificate(ISSUER);
                }
                
                if (p12Filename != null && pKey != null) {
                    signAndSave(new AttributeCertGenerator());
                }
            } else {
                throw new RuntimeException("Unkown Certificate!");
            }
        } else {
            throw new RuntimeException(cmd + " not supported");
        }
        
    }
    
    private void signAndSave(AkentiCertGenerator generator) {
        P12Reader reader = new P12Reader();
        boolean useHash = true;
        try {
            AkentiCertGenerator gen = generator;
            gen.setVersionNumber(2);
            gen.setUniqueID("UID");
            
            gen.setNotBefore(akentiCert.getNotBefore());
            gen.setNotAfter(akentiCert.getNotAfter());
            
            gen.setIssuer(akentiCert.getIssuer());
            
            if (akentiCert instanceof PolicyCertificate) {
                useHash = false;
                PolicyCertificate polCert = (PolicyCertificate) akentiCert;
                PolicyCertGenerator polGen = (PolicyCertGenerator) gen;
                
                polGen.setVersionNumber(2);
                polGen.setUniqueID("UID");
                polGen.setNotBefore(polCert.getNotBefore());
                polGen.setNotAfter(polCert.getNotAfter());
                polGen.setIssuer(polCert.getIssuer());
                polGen.setCertificateAuthorityInfos(polCert.getCertificateAuthorityInfos());
                polGen.setIssuer(polCert.getIssuer());
                polGen.setResource(polCert.getResource());
                polGen.setUseCondIssuerGroups(polCert.getUseCondIssuerGroups());
                polGen.setAttributeDirectories(polCert.getAttributeDirectories());
                polGen.setCacheTime(polCert.getCacheTime());
                
                akentiCert = polGen.generate();
            } else if (akentiCert instanceof UseCondCertificate) {
                UseCondCertificate ucCert = (UseCondCertificate) akentiCert;
                UseCondCertGenerator ucGen = (UseCondCertGenerator) gen;
                
                ucGen.setVersionNumber(2);
                ucGen.setUniqueID("UID");
                ucGen.setIssuer(ucCert.getIssuer());
                ucGen.setNotBefore(ucCert.getNotBefore());
                ucGen.setNotAfter(ucCert.getNotAfter());
                
                ucGen.setResource(ucCert.getResource());
                ucGen.setLocalScope(ucCert.isScopeLocal());
                
                ucGen.setCritical(ucCert.isCritical());
                ucGen.setCondition(ucCert.getCondition());
                ucGen.setActions(ucCert.getActions());
                
                akentiCert = ucGen.generate();
            } else if (akentiCert instanceof AttributeCertificate) {
                AttributeCertificate attrCert = (AttributeCertificate) akentiCert;
                AttributeCertGenerator attrGen = (AttributeCertGenerator) gen;
                
                attrGen.setPrincipal(attrCert.getPrincipal());
                attrGen.setAttribute(attrCert.getAttribute());
                attrGen.setValue(attrCert.getValue());
                
                akentiCert = attrGen.generate();
            } else {
                throw new RuntimeException("Unknown Certificate!");
            }
            
            gen.sign("MD5withRSA", pKey);
            
            String hash = Util.createHashName(akentiCert.getHashString());
            
            gen.setUniqueID(Util.createUniqueID(hash));
            JFileChooser fileChooser = new JFileChooser();
            fileChooser.setDialogTitle("Save Certificate as");
            fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            
            int clientResponse = fileChooser.showDialog(this, "Save as");
            if (clientResponse == JFileChooser.APPROVE_OPTION) {
                
                String fileName = fileChooser.getSelectedFile().getAbsolutePath();
                if (useHash) {
                    fileName += '.' + hash + ".";
                    fileName = Util.createFileName(fileName);
                }
                
                FileOutputStream out = new FileOutputStream(fileName);
                
                AkentiCertificateFactory.writeXMLCertToStream(akentiCert, out);
                
                showMessage("Certificate was successfully created!");
            }
        } catch(RuntimeException e) {
            e.printStackTrace();
            
            // reset the p12Filename and p12Password variables
            p12Filename = null;
            pKey = null;
            
            showMessage(e.getMessage());
        } catch(Exception e) {
            e.printStackTrace();
            // reset the p12Filename and p12Password variables
            p12Filename = null;
            pKey = null;
            
            showMessage(e.getMessage());
            
        }
        
    }
}