/**
 *  @author Jordan Thompson (JordanCThompson@yahoo.com)
 */

package lbl.akenti.policyManager.widgets.viewer;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.cert.X509Certificate;
import javax.swing.DefaultListModel;
import lbl.akenti.util.ParsingException;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.policyManager.base.GenCa;
import lbl.akenti.policyManager.widgets.GenConsts;
import lbl.akenti.policyManager.widgets.LBLPrincipalListPane;
import lbl.akenti.policyManager.widgets.pol.CAInfoPanel;
import lbl.akenti.policyManager.widgets.InputPane;

public class EditorCAInfosPanel extends JOptionPane {
    // Variables declaration - do not modify
    public static final int CERTIFICATE_AUTHORITY_INFOS = 0;
    public static final int USE_CONDITION_ISSUER_GROUPS = 1;
    
    private boolean userApproved;
    private LBLPrincipalListPane ucGroupPanel;
    private UseCondIssuerGroup[] ucigs;
    private CertificateAuthorityInfo[] newCAs;
    private int mode;
    private GenCa[] cas;
    private CAInfoPanel caInfoPanel = new CAInfoPanel();;
    private DefaultListModel model = new DefaultListModel();
    private JButton add_button;
    private JPanel buttonPanel;
    private JButton cancel_button;
    private JPanel controller_panel;
    private JPanel confirmation_panel;
    private JList list;
    private JButton ok_button;
    private JButton remove_button;
    private JScrollPane sPane;
    private JButton view_button;
    // End of variables declaration
    
    private static String UC_GROUP_PANEL_HEADER  =
    "<html><font size=+1 color=blue>UseConditionGroup Editor</font>\n\n" +
    "<li><font color=blue>Must provide at least one UseCondition issuer" +
    "<li><font color=blue>Must provide at least one UseCondition directory\n";
    ;
    
    public EditorCAInfosPanel(int aMode) {
        initComponents();
        mode = aMode;
        
    }
    
    private void initComponents() {
        setLayout(new BorderLayout());
        
        sPane = new JScrollPane();
        list = new JList(model);
        list.setSelectionMode(2);
        buttonPanel = new JPanel();
        controller_panel = new JPanel();
        add_button = new JButton();
        view_button = new JButton();
        remove_button = new JButton();
        confirmation_panel = new JPanel();
        ok_button = new JButton();
        cancel_button = new JButton();
        
        sPane.setViewportView(list);
        
        add(sPane, BorderLayout.CENTER);
        
        buttonPanel.setLayout(new BorderLayout());
        
        controller_panel.setLayout(new GridBagLayout());
        
        add_button.setText("Add");
        add_button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                add_buttonActionPerformed(evt);
            }
        });
        
        controller_panel.add(add_button);
        
        view_button.setText("View");
        view_button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                view_buttonActionPerformed(evt);
            }
        });
        
        controller_panel.add(view_button);
        
        remove_button.setText("Remove");
        remove_button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                remove_buttonActionPerformed(evt);
            }
        });
        
        controller_panel.add(remove_button);
        
        buttonPanel.add(controller_panel, BorderLayout.NORTH);
        
        confirmation_panel.setLayout(new GridBagLayout());
        
        ok_button.setText("OK");
        ok_button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ok_buttonActionPerformed(evt);
            }
        });
        
        confirmation_panel.add(ok_button);
        
        cancel_button.setText("Cancel");
        cancel_button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancel_buttonActionPerformed(evt);
            }
        });
        
        confirmation_panel.add(cancel_button);
        
        buttonPanel.add(confirmation_panel, BorderLayout.SOUTH);
        
        add(buttonPanel, BorderLayout.SOUTH);
        
    }
    
    private void cancel_buttonActionPerformed(ActionEvent evt) {
        userApproved = false;
        setValue(GenConsts.AK_CANCEL_VALUE);
    }
    
    private void ok_buttonActionPerformed(ActionEvent evt) {
        userApproved = true;
        setValue(GenConsts.AK_CANCEL_VALUE);
    }
    
    private void remove_buttonActionPerformed(ActionEvent evt) {
        DefaultListModel model = (DefaultListModel) list.getModel();
        int []indices = list.getSelectedIndices();
        
        for (int i = indices.length - 1; i > -1; --i) {
            model.removeElementAt(indices[i]);
        }
    }
    
    private void view_buttonActionPerformed(ActionEvent evt) {
        if (mode == CERTIFICATE_AUTHORITY_INFOS) {
            
            
            CAInfoPanel caInfoPanel = getCAInfoPanel();
            caInfoPanel.setGenCAs(cas);
            if (list.getSelectedIndex() != -1) {
                int index = list.getSelectedIndex();
                DefaultListModel model = (DefaultListModel) list.getModel();
                CertificateAuthorityInfo info = (CertificateAuthorityInfo) model.getElementAt(index);
                
                caInfoPanel.setOKLabel("Apply Changes");
                caInfoPanel.setCAInfo(info);
                caInfoPanel.createDialog(this, "CAInfoViewer").setVisible(true);
                
                Object val = caInfoPanel.getValue();
                
                if ((val != null) && !(val.equals(GenConsts.AK_CANCEL_VALUE))) {
                    model.removeElementAt(index);
                    model.insertElementAt(val, index);
                    list.addSelectionInterval(index, index);
                }
            }
        } else if (mode == USE_CONDITION_ISSUER_GROUPS) {
            LBLPrincipalListPane ucGroupPanel = getUCGroupPanel();
            
            ucGroupPanel.setGenCAs(cas);
            if (list.getSelectedIndex() != -1) {
                
                int index = list.getSelectedIndex();
                DefaultListModel model = (DefaultListModel) list.getModel();
                UCGroup group = (UCGroup) model.getElementAt(index);
                UseCondIssuerGroup dgroup = group.getGroup();
                AkentiPrincipal []stakeholders = dgroup.getUCIssuers();
                Directory []ucDirs = dgroup.getDirectories();
                
                ucGroupPanel.clear();
                ucGroupPanel.addIssuers(stakeholders);
                ucGroupPanel.addDirs(ucDirs);
                ucGroupPanel.setOKLabel("Apply Changes");
                ucGroupPanel.createDialog(this, "UCGroupViewer").setVisible(true);
                
                Object test = ucGroupPanel.getValue();
                
                if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
                    Object[] val = (Object[]) test;
                    AkentiPrincipal[] principals = (AkentiPrincipal[]) val[0];
                    Directory[] dirs = (Directory[]) val[1];
                    
                    group.setGroup(new UseCondIssuerGroup(principals, dirs));
                    
                    // Force the changes to appear graphically.
                    model.removeElementAt(index);
                    model.insertElementAt(group, index);
                    list.setSelectedIndex(index);
                }
            }
        }
    }
    
    private void add_buttonActionPerformed(ActionEvent evt) {
        if (mode == CERTIFICATE_AUTHORITY_INFOS) {
            CAInfoPanel caInfoPanel = getCAInfoPanel();
            caInfoPanel.setGenCAs(cas);
            caInfoPanel.clear();
            caInfoPanel.setOKLabel("Add CA Info");
            caInfoPanel.createDialog(this, "CAInfoBuilder").setVisible(true);
            
            Object val = caInfoPanel.getValue();
            
            if ((val != null) && !(val.equals(GenConsts.AK_CANCEL_VALUE))) {
                addCAInfo((CertificateAuthorityInfo) val);
            }
            
            
        } else if (mode == USE_CONDITION_ISSUER_GROUPS) {
            LBLPrincipalListPane ucGroupPanel = getUCGroupPanel();
            
            ucGroupPanel.setOKLabel("Add Group");
            ucGroupPanel.setGenCAs(cas);
            ucGroupPanel.clear();
            ucGroupPanel.createDialog(this, "UCGroup Panel").setVisible(true);
            
            Object test = ucGroupPanel.getValue();
            
            if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)){
                Object[] val = (Object[]) test;
                AkentiPrincipal[] principals = (AkentiPrincipal[]) val[0];
                Directory[] dirs = (Directory[]) val[1];
                
                addUCGroup(new UseCondIssuerGroup(principals, dirs));
            }
        }
    }
    
    private LBLPrincipalListPane getUCGroupPanel() {
        if (ucGroupPanel == null) {
            ucGroupPanel = new LBLPrincipalListPane(
            UC_GROUP_PANEL_HEADER,
            "UseCond Issuers",
            "UseCond Directories"
            );
        }
        
        return ucGroupPanel;
    }
    public GenCa[] getGenCAs() throws ParsingException {
        GenCa[] genCA = new GenCa[caInfoPanel.getCACerts().length];
        for (int x = 0; x < caInfoPanel.getCACerts().length; x++) {
            X509Certificate x509Temp = caInfoPanel.getCACerts()[x];
            GenCa caTemp = new GenCa(new DistinguishedName(x509Temp.getIssuerDN()));
        }
        return null;
    }
    public void setUCGroups(UseCondIssuerGroup[] groups) {
        ucigs = groups;
        addUCGroups(ucigs);
    }
    public UseCondIssuerGroup[] getUseCondIssuerGroups() {
        DefaultListModel model = (DefaultListModel) list.getModel();
        int added = 0;
        int size = model.getSize();
        
        ucigs = new UseCondIssuerGroup[model.size()];
        for (int i = 0; i < model.size(); ++i) {
            ucigs[i] = ((UCGroup) model.getElementAt(i)).getGroup();
        }
        return ucigs;
    }
    public void setGenCAs(GenCa[] genCAs) {
        cas = genCAs;
    }
    
    public void setCAInfos(CertificateAuthorityInfo[] infos) {
        addCAInfos(infos);
    }
    
    public CAInfoPanel getCAInfoPanel() {
        if (caInfoPanel == null) {
            caInfoPanel = new CAInfoPanel();
        }
        
        return caInfoPanel;
    }
    /**
     */
    public int addCAInfos(CertificateAuthorityInfo[] infos) {
        DefaultListModel model = (DefaultListModel) list.getModel();
        int added = 0;
        int size = model.getSize();
        
        list.clearSelection();
        
        for (int i = 0; i < infos.length; ++i) {
            int index = model.indexOf(infos[i]);
            
            if (index != -1) {
                list.addSelectionInterval(index, index);
            } else {
                model.addElement(infos[i]);
                list.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        return added;
    }
    
    public int addUCGroups(UseCondIssuerGroup[] groups) {
        DefaultListModel model = (DefaultListModel) list.getModel();
        
        int added = 0;
        int size = model.getSize();
        
        list.clearSelection();
        
        for (int i = 0; i < groups.length; ++i) {
            int index = model.indexOf(groups[i]);
            
            if (index != -1) {
                list.addSelectionInterval(index, index);
            } else {
                model.addElement(new UCGroup(groups[i]));
                list.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        return added;
    }
    
    public boolean addUCGroup(UseCondIssuerGroup group) {
        return addUCGroups(new UseCondIssuerGroup[] { group }) != 0;
    }
    
    public boolean addCAInfo(CertificateAuthorityInfo info) {
        return addCAInfos(new CertificateAuthorityInfo[] { info }) != 0;
    }
    
    public CertificateAuthorityInfo[] getCAInfos() {
        newCAs = new CertificateAuthorityInfo[model.getSize()];
        for (int x = 0; x < model.getSize(); x ++) {
            newCAs[x] = (CertificateAuthorityInfo) model.getElementAt(x);
        }
        return newCAs;
        
    }
    
    public boolean getUserApproved() {
        return userApproved;
    }
    
    private class UCGroup {
        private UseCondIssuerGroup group;
        
        UCGroup(UseCondIssuerGroup group) {
            this.group = group;
        }
        
        void setGroup(UseCondIssuerGroup group) {
            this.group = group;
        }
        
        UseCondIssuerGroup getGroup() {
            return group;
        }
        
        public String toString() {
            StringBuffer sb = new StringBuffer();
            
            sb.append("issuers");
            sb.append('=');
            sb.append(group.getUCIssuers().length);
            sb.append(',');
            sb.append("ucdirs");
            sb.append('=');
            sb.append(group.getDirectories().length);
            return sb.toString();
        }
    }
}
