package lbl.akenti.servers.AKMonitor.gui;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Insets;

/*
 * This panel shows progress by adding a StatusCanvas 
 * to the left of the panel and removes the last canvas
 * if this canvas is no longer visible. 
 *
 **/
public class ProgressPanel extends StatusPanel {
  /* size of canvases */
  protected static Dimension defaultDimension = new Dimension(5,5);
  
  /* preferred height. */
  protected int height;

  /*
   * used to avoid creating new <code>StatusPanel </code> widgets.
   */
  private StatusCanvas removed;
  
  /*
   * Constructor.
   **/
  public ProgressPanel(String title) { 
    progressPanelInit(title);
  }
  
  /*
   * mainly computes height.
   **/
  protected void progressPanelInit(String title) {
    BorderLayout layout = (BorderLayout) getLayout();
    
    layout.setVgap(3);

    Insets insets;

    setTitle(title);
    height = label.getPreferredSize().height;
    insets = getInsets();
    height += layout.getVgap();
    height += insets.top + insets.bottom;
    insets = panel.getInsets();
    height += insets.top + insets.bottom;
    height += defaultDimension.height;

    /* a couple more pixels! */
    height += 2; 
  }

  /*
   * add canvas to the left of the panel.
   * remove last canvas if it is not visiblei;
   *
   **/
  public synchronized StatusCanvas addStatusCanvas(StatusEvent event) {
    StatusCanvas canvas;
   
    if (removed != null) {
      canvas = removed;
      canvas.setStatusEvent(event);
      removed = null;
    } else {
      canvas = new StatusCanvas(event);
      canvas.setPreferredSize(defaultDimension);
    }

    panel.add(canvas, 0);
    panel.invalidate();
    panel.validate();
    panel.repaint();

    int count = panel.getComponentCount();

    if (count-- > 1) {
      Rectangle rect = canvas.getBounds();
      Component acanvas = panel.getComponent(count);
      
      /* 
       * Note that test isVisible() does not work.
       * So we ask the last canvas for its bounds and 
       * since we have a Flowlayout Manager the last canvas
       * gets positioned at a different height.
       */

      if (rect.y != acanvas.getBounds().y) {
        removed = (StatusCanvas) acanvas;
        removed.setStatusEvent(null);
      }
    }
    return canvas;
  }
  
  /*
   * we just make sure that the height stays the same.
   **/
  public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    
    dim.height = height;
    return dim;
  } 

  /*
   * @see getPreferredSize
   **/
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /*
   * @see getPreferredSize
   **/
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }

  /*
   * remove canvases. 
   **/
  public synchronized void clear() {
    super.clear();
    removed = null;
    validate();
    repaint();
  }
}
  
   
  
