package lbl.akenti.servers.AKMonitor.gui;

/**
  * Resource.java
  * by Abdelilah Essiari
  *
  **/
import java.util.Vector;
import java.util.Enumeration;

/**
  * This object is used to represent a resource.
  *  
  * A resource consists of a name and of a number of 
  * <code>StatusEvent</code> events generated by 
  * the access control mechanism.  
  */
public class Resource {
  /* name of a resource */
  protected String name;
  
  /* used to store the events */
  protected Vector events = new Vector();
  
  /* 
   * set to null while events are still being collected (default).
   *        true if access to the resource succeeds.
   *        false if access to the resource fails.
   */
  protected Boolean status;

  public Resource(String name) {
    this.name = name;
  }

  /*
   * @return name of the resource
   **/
  public String getName() {
    return name;
  }

  /*
   * @return name of the resource
   **/
  public String toString() {
    return getName();
  }

  /**
    * add an event associated with a resource.
    */
  public void addStatusEvent(StatusEvent event) {
    events.addElement(event);
  }

  public StatusEvent lastStatusEvent() {
    return (StatusEvent) events.lastElement();
  }

  public StatusEvent getStatusEventAt(int index) {
    return (StatusEvent) events.elementAt(index);
  }

  /**
    * This should be set when all events associted with a resource are added.
    *
    * @param status   true if access to a resource has been granted. 
    *                 false otherwise.
    */
  public void setStatus(boolean status) {
    this.status = status ? Boolean.TRUE : Boolean.FALSE;
  }

  /**
    * @return whether status has been set.
    * @see setStatus(boolean)
    */
  public Boolean getStatus() {
    return status;
  }

  /**
    * @return events associated with the resource.
    * @see setStatus(boolean)
    */
  public Enumeration events() {
    return events.elements();
  }
} 

