package lbl.akenti.servers.AKMonitor.gui;

/**
  * ResourceList.java
  * by Abdelilah Essiari
  *
  **/
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.DefaultListModel;
import javax.swing.ListCellRenderer;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.util.Enumeration;

/**
  * This a graphical list of resources.
  * 
  **/
public class ResourceList extends JList {
  /*
   * Constructor.
   *
   */
  public ResourceList() {
    super(new DefaultListModel());
    setCellRenderer(getListCellRenderer());
  }
  
  /*
   * add a resource to the list;
   *
   * @return index where resource was added.
   **/
  public int addResource(Resource resource) {
    DefaultListModel model = (DefaultListModel) getModel();

    int index = model.getSize();

    model.addElement(resource);
    ensureIndexIsVisible(index);
    return index;
  } 

  /*
   * removes only resources that are done.
   */
  public void clear() {
    DefaultListModel model = (DefaultListModel) getModel();
    int index = 0;

    while (index != model.size()) {
      Resource resource = (Resource) model.elementAt(index);

      if (resource.getStatus() == null) {
        index++;
      } else {
        model.removeElementAt(index);
      }
    }
  }

  public void clearAll() {
    DefaultListModel model = (DefaultListModel) getModel();
    int index = 0;

    while (index != model.size()) {
      model.removeElementAt(index);
    }
  }

  /* borders used by the Cell Renderer */
  protected static Border pborder = new EmptyBorder(5, 5, 5, 5);
  protected static Border lborder = new BevelBorder(BevelBorder.LOWERED);

  /*
   * graphical component to render each cell of the list.
   *
   **/
  protected ListCellRenderer getListCellRenderer() {
    return new CellRenderer();
  } 

  protected class CellRenderer extends JPanel implements ListCellRenderer {
    protected JLabel label;

    protected CellRenderer() {
      setLayout(new GridLayout(1, 1));
      setBorder(pborder);
      label = new JLabel();
      label.setOpaque(true);
      label.setBorder(lborder);
      add(label);
    }

    public Component
    getListCellRendererComponent(JList list,
                                 Object value,
                                 int index,
                                 boolean isSelected,
                                 boolean hasFocus) {

      Resource res = (Resource) value;

      Boolean status = res.getStatus();
      Color fg;

      /* 
       * set the foreground colors.
       **/ 
       if (status == null) {
         fg = Color.black;                  /* still receving logs */
       } else if(status.booleanValue()) {
         fg = Color.green;                  /* access granted      */
       } else {
         fg = Color.red;                    /* acces denied        */
       }
        
       /* handle selection differently */
       if (isSelected) {
         label.setBackground(fg);
         label.setForeground(Color.white);
       } else {
         label.setBackground(Color.white);
         label.setForeground(fg);
       }

       label.setText(res.getName());
       label.setEnabled(list.isEnabled());
       return this;
    }
  }
}
  

