package lbl.akenti.servers.CacheManager;

import java.net.ServerSocket;
import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.File;
import lbl.akenti.util.*;

/**
  *
  */
public class CacheServer implements AkentiConstants {

  private ServerSocket serverSocket;
  
  private File baseDir;
  
  /**
    *
    */
  public CacheServer(int port, File baseDir) throws IOException {
    if (!baseDir.isDirectory()) {
      throw new IllegalArgumentException();
    }
    
    this.baseDir = baseDir;
    serverSocket = new ServerSocket(port);
  }
  
  /**
    *
    */
  public void start() {
    try {
      Collector collector = new Collector(baseDir);

      collector.start();

      while (true) {
        new Thread(
          new CacheHandler(
            serverSocket.accept(),
            baseDir 
          )
        ).start();
      }
    } catch(IOException ex) {
      System.err.println(ex);
    }
  }

  /**
    *
    */
  public static void main(String args[]) {
    try {
      
      if (args.length == 1) {
        System.err.println("LOADING CONFIG FILE");
        ConfFileParser.loadFromFile(args[0]);
      }

      File baseDir = new File(System.getProperty("CacheBase", CacheBase));

      baseDir = new File(baseDir, System.getProperty("CacheDirName", CacheDirName));

      if (!baseDir.isDirectory() && !baseDir.mkdirs()) {
        System.err.println("UNABLE TO CREATE CACHE BASE DIR");
        System.exit(1);
      }

      System.err.println("SERVER CACHING IN DIR " + baseDir.getAbsolutePath());

      String port = System.getProperty("CachePort", CachePort);
      CacheServer s = new CacheServer(new Integer(port).intValue(), baseDir);

      System.err.println("SERVER RUNNING at port " + port);
      s.start(); 
    } catch(IOException ex) {
      System.err.println(ex);
    }
  }
}
