package lbl.akenti.servers.CacheManager;

import java.io.File;

/**
  *
  **/
public class Collector extends Thread {

  public static long DEFAULT_PERIOD = 30 * 1000;

  private File baseDir;

  private long period;
  
  public Collector(File baseDir) {
    if (!baseDir.isDirectory() || !baseDir.canWrite()) {
      throw new IllegalArgumentException();
    }

    this.baseDir = baseDir;
    period = DEFAULT_PERIOD;
  }

  public Collector(File baseDir, long period) {
    this(baseDir);
    
    if (period < 0) {
      throw new IllegalArgumentException();
    }

    this.period = period;
  }
  
  public void run() {
    while (true) {
      try {
        Cache[] caches = Cache.getCaches();
        UtcTime now = new UtcTime();

        for (int i = 0; i < caches.length; ++i) {
          File dir = new File(baseDir, caches[i].getName());

          synchronized(caches[i]) {
            CacheEntry e = caches[i].firstCacheEntry();

          
            while (e != null && !e.checkValidity(now)) {
              caches[i].removeCacheEntry(e);
              new File(new File(dir, e.getDirName()), e.getKey()).delete();
              e = caches[i].firstCacheEntry();
            }
          }
        }

        sleep(period);
      } catch(InterruptedException e) {
        System.err.println(e);
      }
    }
  } 
}
