
/**
  * Test suite for the generating an Attribute Certificate. 
  * 
  * @author Willie Chin
  */

package lbl.akenti.test.certs;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.text.ParseException;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.UTCTime;

public class AttributeCertGeneratorTestCase extends TestCase {

  /**
    * Constructs this testcase.
    */
  public AttributeCertGeneratorTestCase(String name) {
    super(name);
  }

  /**
    * Test Suite.
    */
  public static Test suite() {
    return new TestSuite(AttributeCertGeneratorTestCase.class);
  }

  /**
    *
    */
  public void 
  testGenerateAttributeCertCertGenerator()
    throws IOException, ParseException, GeneralSecurityException {

    UTCTime now = new UTCTime();
    UTCTime after = new UTCTime(now, 365 * 24 * 60 * 60);

    DistinguishedName issuer = new DistinguishedName("/C=US/OU=LBL/CN=Stake Holder I");
    DistinguishedName ca = new DistinguishedName("/C=US/OU=LBL/CN=TEST CA I");
    AkentiPrincipal guarantor = new AkentiPrincipal(issuer, ca);
    DistinguishedName user = new DistinguishedName("/C=US/OU=LBL/CN=User I");
    AkentiPrincipal principal = new AkentiPrincipal(user, ca);
    
    AttributeCertGenerator gen = new AttributeCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID("I");
    gen.setNotBefore(now);
    gen.setNotAfter(after);
    gen.setIssuer(new AkentiIssuer(guarantor));
    gen.setAttribute("JOB");
    gen.setValue("SUPERVISOR");
    gen.setPrincipal(principal);

    String constraint = "(TIME < 10 pm)";

    AttributeInfo ai = new AttributeInfo("TIME", "10 pm");
    Condition cond = new Condition(constraint, new AttributeInfo[] { ai } );

    gen.setCondition(cond);

    P12Reader p12 = new P12Reader();
    p12.readP12File("testCAI.p12", "testing");

    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey);

    Certificate cert = p12.getX509Certificate();
    PublicKey publicKey = cert.getPublicKey();
    AttributeCertificate attrCert = (AttributeCertificate) gen.generate();

    assertTrue(attrCert.verify(publicKey));

    FileOutputStream fos = new FileOutputStream("attrCert.xml");
    AkentiCertificateFactory.writeCertToStream("XML", attrCert, fos);

    fos.close();

    FileInputStream fis = new FileInputStream("attrCert.xml");

    AkentiCertificate tempAttrCert = AkentiCertificateFactory.initCertFromStream(
      "XML", fis
    );

    fis.close();     
    assertTrue(tempAttrCert.verify(publicKey));

    fos = new FileOutputStream("attrCert.pem");
    AkentiCertificateFactory.writeCertToStream("PEM", tempAttrCert, fos);
    fos.close();
    fis = new FileInputStream("attrCert.pem");
    tempAttrCert = AkentiCertificateFactory.initCertFromStream("PEM", fis);
    fis.close();
    assertTrue(tempAttrCert.verify(publicKey));
  }
}    
