package lbl.akenti.test.certs.elements;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.ParseException;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory; 

/**
  * This tests the AkentiIssuer class.
  * 
  * @author Willie Chin
  */
public class AkentiIssuerTestCase extends TestCase {

  /**
    *
    */
  public AkentiIssuerTestCase(String name) {
    super(name);
  }  

  /**
    *
    */
  public static Test suite() {
    return new TestSuite(AkentiIssuerTestCase.class);
  }

  /**
    * Testing <i>AkentiIssuer</i>.
    */
  public void testAkentiIssuer() 
    throws IOException, ParseException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    AkentiOutputStream aos = new AkentiOutputStream(baos);
    AkentiPrincipal issuer = new AkentiPrincipal(
      "/C=US/O=LBNL/OU=DSD/CN=User I",
      "/C=US/O=LBNL/OU=DSD/CN=Test CA I"
    );

    Directory dir = new Directory("file:/home/g1/id");
    Directory[] identityDirs = new Directory[] { dir };
    AkentiIssuer ai = new AkentiIssuer(issuer, identityDirs);

    for (int i = 0; i < 5; ++i) {  
      ai.writeObject(aos);
      baos.write(' ');
    }

    aos.close();

    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
    AkentiInputStream ais = new AkentiInputStream(bais);

    for (int i = 0; i < 5; ++i) {
      AkentiIssuer tai = new AkentiIssuer(ais);

      assertTrue(tai.getIssuer().equals(issuer)); 
      
      Directory[] dirs = tai.getIdentityDirs();

      assertTrue(dirs.length == 1);
      assertTrue(dirs[0].equals(dir));
    }     
  }
}
