package lbl.akenti.test.messaging;

import com.sun.net.ssl.internal.ssl.Provider;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.security.Security;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.messaging.AkentiRequests;
import lbl.akenti.messaging.AkentiResponses;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.PolicyInfo;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;

/**
  * Test the messaging elements.
  */
public class MessagingTestCaseWithCerts extends TestCase {

  private static boolean VERBOSE = false;

  private static String capCertString = 
    "-----BEGIN AKENTI CAPABILITY CERTIFICATE-----\n"+
    "Q2FwYWJpbGl0eSBWMiBib3NzaG9nLmxibC5nb3YjZGFjMGMyMjgjRnJpXCBBcHJc\n"+
    "IDlcIDA3OjQ1OjExXCAyMDA0IC9DTj1OT19OQU1FIC9DTj1OT19OQU1FIDAgMDQw\n"+
    "NDA5MDc0NTExWiAwNDA0MDkwODE1MTFaIEFrMUNhbkFsZyBOT19TSUdfQUxHIC9D\n"+
    "PVVTL09VPUxCTC9DTj1Vc2VyXCBJIC9DPVVTL09VPUxCTC9DTj1URVNUQ0FcIElB\n"+
    "IDkyIDBaMA0GCSqGSIb3DQEBAQUAA0kAMEYCQQCwRNs88nojwQQRW/EsdWu5KYun\n"+
    "WfhrVjti7TrnvqXf/+YjWQ2Rbgr1gS48L9YzYIPET8T8LE92nuGn5eceaycxAgED\n"+
    "IFIxIDEgUkVBRCAwIDYgTk9fU0lH\n"+
    "-----END AKENTI CAPABILITY CERTIFICATE-----";

  private static String polCertString = 
    "-----BEGIN AKENTI POLICY CERTIFICATE-----\n"+
    "UG9saWN5IFYyIDEgL0M9VVMvT1U9TEJML0NOPVN0YWtlXCBIb2xkZXJcIEkgL0M9\n"+
    "VVMvT1U9TEJML0NOPVRFU1RDQVwgSUEgMCAwMzA3MjUxMDM3MTNaIDA0MDcyNDEw\n"+
    "MzcxM1ogQWsxQ2FuQWxnIFJTQS1NRDUgUjEgMSAxIDMzMiAwggFIMIHzoAMCAQIC\n"+
    "AQEwDQYJKoZIhvcNAQEFBQAwLjELMAkGA1UEBhMCVVMxDDAKBgNVBAsTA0xCTDER\n"+
    "MA8GA1UEAxMIVEVTVENBIEkwHhcNMDMwNzI1MTAzNzExWhcNMDQwNTIwMTAzNzEx\n"+
    "WjAuMQswCQYDVQQGEwJVUzEMMAoGA1UECxMDTEJMMREwDwYDVQQDEwhURVNUQ0Eg\n"+
    "STBaMA0GCSqGSIb3DQEBAQUAA0kAMEYCQQCjhXU9beGXx7JDZyDRw/ep5GL3lwjn\n"+
    "ehaMF5QwfA5iTgisch6GIxKVEf5abTxQFSdlF3SqDKXbLxistF7zLG91AgEDMA0G\n"+
    "CSqGSIb3DQEBBQUAA0EAWROgLDGfULB/V3O7YN/u50AIqlu6iT9d2UfSQTDAQS1i\n"+
    "rGmF4ubv98TW49+QbGiyYWKZf+3nx0ipYYyLxhJXeyAxIGZpbGU6L2hvbWUvZzEv\n"+
    "dXNlcnMvYWVzL2FrZW50aS9idWlsZC9saW51eC1nY2MzL3Rlc3RjYXNlL2xpYnNy\n"+
    "Yy9jZXJ0cy8uLi8uLi9yZXNvdXJjZVRyZWUvaWRDZXJ0cy8gMCAxIDEgL0M9VVMv\n"+
    "T1U9TEJML0NOPVN0YWtlXCBIb2xkZXJcIEkgL0M9VVMvT1U9TEJML0NOPVRFU1RD\n"+
    "QVwgSUEgMSBmaWxlOi9ob21lL2cxL3VzZXJzL2Flcy9ha2VudGkvYnVpbGQvbGlu\n"+
    "dXgtZ2NjMy90ZXN0Y2FzZS9saWJzcmMvY2VydHMvLi4vLi4vcmVzb3VyY2VUcmVl\n"+
    "L2FrZW50aUNlcnRzLyAxIGZpbGU6L2hvbWUvZzEvdXNlcnMvYWVzL2FrZW50aS9i\n"+
    "dWlsZC9saW51eC1nY2MzL3Rlc3RjYXNlL2xpYnNyYy9jZXJ0cy8uLi8uLi9yZXNv\n"+
    "dXJjZVRyZWUvYWtlbnRpQ2VydHMvIDE4MDAgNjQgesHsSCCeALlDtj1JQrzCycO8\n"+
    "kFI5169cvGTZGoXr592m4BPRtGa8oRQDEO2DD0HJqYL1yyd/GtaLNeAgS5ADtA==\n"+
    "-----END AKENTI POLICY CERTIFICATE-----";

  private static String ucCertString = 
    "-----BEGIN AKENTI USECONDITION CERTIFICATE-----\n"+
    "VXNlQ29uZGl0aW9uIFYyIDEgL0M9VVMvT1U9TEJML0NOPVN0YWtlXCBIb2xkZXJc\n"+
    "IEkgL0M9VVMvT1U9TEJML0NOPVRFU1RDQVwgSUEgMCAwMzA3MjUxMDM3MTNaIDA0\n"+
    "MDcyNDEwMzcxM1ogQWsxQ2FuQWxnIFJTQS1NRDUgUjEgMCAxIE9VXCA9TEJMXCAm\n"+
    "JlwgSk9CXCA9XCBTVVBFUlZJU09SIDIgMSBPVSBMQkwgMSAvQz1VUy9PVT1MQkwv\n"+
    "Q049VEVTVENBXCBJQSAyIEpPQiBTVVBFUlZJU09SIDEgL0M9VVMvT1U9TEJML0NO\n"+
    "PVN0YWtlXCBIb2xkZXJcIEkgL0M9VVMvT1U9TEJML0NOPVRFU1RDQVwgSUEgMCAx\n"+
    "IFJFQUQgNjQglXPdI2DH/Zhe4p96z9zF79UqG7QYIakkebyN3Uz1gV8BZNEZe2qY\n"+
    "xwlbeiASnR9rVFc/OLNekZXZyQg51c3g4Q==\n"+
    "-----END AKENTI USECONDITION CERTIFICATE-----";

  /**
    */
  public MessagingTestCaseWithCerts(String name) {
    super(name);
  }  

  /**
    */
  public static Test suite() {
    return new TestSuite(MessagingTestCaseWithCerts.class);
  }

  public void test4() throws Exception {
    Security.addProvider(new Provider());

    ByteArrayInputStream bais = new ByteArrayInputStream(capCertString.getBytes());
    AkentiCertificate akCert = AkentiCertificateFactory.initCertFromStream("PEM", bais);
    CapabilityCertificate capCert = (CapabilityCertificate) akCert;
    AkentiMessage mesg = new AkentiMessage(101, "you got game");
    CheckAccessResponse resp = new CheckAccessResponse(0, mesg, capCert);
    AkentiResponses aresps = new AkentiResponses(resp);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    aresps.toStream(baos);

    String out1 = new String(baos.toByteArray());

    bais = new ByteArrayInputStream(baos.toByteArray());
    aresps = new AkentiResponses(bais);
    baos = new ByteArrayOutputStream();
    aresps.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      aresps.toStream(System.out);
    }
  }

  /**
    */
  public void test6() throws Exception {
    ByteArrayInputStream bais = new ByteArrayInputStream(polCertString.getBytes());
    AkentiCertificate cert = AkentiCertificateFactory.initCertFromStream("PEM", bais);
    UploadPolicyCertificateRequest 
      ucreq = new UploadPolicyCertificateRequest(5, (PolicyCertificate) cert);

    AkentiRequests areqs = new AkentiRequests(ucreq);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    areqs.toStream(baos);

    String out1 = new String(baos.toByteArray());

    bais = new ByteArrayInputStream(baos.toByteArray());
    areqs = new AkentiRequests(bais);

    baos = new ByteArrayOutputStream();
    areqs.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      areqs.toStream(System.out);
    }

  }

  /**
    */
  public void test5() throws Exception {
    ByteArrayInputStream bais = new ByteArrayInputStream(polCertString.getBytes());
    AkentiCertificate polCert = AkentiCertificateFactory.initCertFromStream("PEM", bais);

    bais = new ByteArrayInputStream(ucCertString.getBytes());

    AkentiCertificate ucCert = AkentiCertificateFactory.initCertFromStream("PEM", bais);
    AkentiMessage[] mesgArray = { new AkentiMessage(198, "found policy ok") };
    CertificateInfo cr1 = new CertificateInfo(polCert, mesgArray, "http://www.blab.com", true);
    CertificateInfo cr2 = new CertificateInfo(ucCert, mesgArray, "file:/home/dumb/", false);
   

    PolicyInfo rp = new PolicyInfo(new CertificateInfo[] { cr1, cr2});
    AkentiMessage mesg = new AkentiMessage(404, "missing critical uc");
    CheckPolicyResponse cpr = new CheckPolicyResponse(20, mesg, rp);

    AkentiResponses aresps = new AkentiResponses(cpr);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    aresps.toStream(baos);

    String out1 = new String(baos.toByteArray());

    bais = new ByteArrayInputStream(baos.toByteArray());
    aresps = new AkentiResponses(bais);
    baos = new ByteArrayOutputStream();
    aresps.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      aresps.toStream(System.out);
    }
  }
}
