package lbl.akenti.test.remotePolicyEngine;

import java.io.ByteArrayInputStream;
import java.net.URL;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;

/**
  * Test the remote policy engine.
  */
public class RemotePolicyEngineTestCase extends TestCase {

  private static boolean VERBOSE = true;

  private static String polCertString =
    "-----BEGIN AKENTI POLICY CERTIFICATE-----\n"+
    "UG9saWN5IFYyIDEgL0M9VVMvT1U9TEJML0NOPVN0YWtlXCBIb2xkZXJcIEkgL0M9\n"+
    "VVMvT1U9TEJML0NOPVRFU1RDQVwgSUEgMCAwMzA3MjUxMDM3MTNaIDA0MDcyNDEw\n"+
    "MzcxM1ogQWsxQ2FuQWxnIFJTQS1NRDUgUjEgMSAxIDMzMiAwggFIMIHzoAMCAQIC\n"+
    "AQEwDQYJKoZIhvcNAQEFBQAwLjELMAkGA1UEBhMCVVMxDDAKBgNVBAsTA0xCTDER\n"+
    "MA8GA1UEAxMIVEVTVENBIEkwHhcNMDMwNzI1MTAzNzExWhcNMDQwNTIwMTAzNzEx\n"+
    "WjAuMQswCQYDVQQGEwJVUzEMMAoGA1UECxMDTEJMMREwDwYDVQQDEwhURVNUQ0Eg\n"+
    "STBaMA0GCSqGSIb3DQEBAQUAA0kAMEYCQQCjhXU9beGXx7JDZyDRw/ep5GL3lwjn\n"+
    "ehaMF5QwfA5iTgisch6GIxKVEf5abTxQFSdlF3SqDKXbLxistF7zLG91AgEDMA0G\n"+
    "CSqGSIb3DQEBBQUAA0EAWROgLDGfULB/V3O7YN/u50AIqlu6iT9d2UfSQTDAQS1i\n"+
    "rGmF4ubv98TW49+QbGiyYWKZf+3nx0ipYYyLxhJXeyAxIGZpbGU6L2hvbWUvZzEv\n"+
    "dXNlcnMvYWVzL2FrZW50aS9idWlsZC9saW51eC1nY2MzL3Rlc3RjYXNlL2xpYnNy\n"+
    "Yy9jZXJ0cy8uLi8uLi9yZXNvdXJjZVRyZWUvaWRDZXJ0cy8gMCAxIDEgL0M9VVMv\n"+
    "T1U9TEJML0NOPVN0YWtlXCBIb2xkZXJcIEkgL0M9VVMvT1U9TEJML0NOPVRFU1RD\n"+
    "QVwgSUEgMSBmaWxlOi9ob21lL2cxL3VzZXJzL2Flcy9ha2VudGkvYnVpbGQvbGlu\n"+
    "dXgtZ2NjMy90ZXN0Y2FzZS9saWJzcmMvY2VydHMvLi4vLi4vcmVzb3VyY2VUcmVl\n"+
    "L2FrZW50aUNlcnRzLyAxIGZpbGU6L2hvbWUvZzEvdXNlcnMvYWVzL2FrZW50aS9i\n"+
    "dWlsZC9saW51eC1nY2MzL3Rlc3RjYXNlL2xpYnNyYy9jZXJ0cy8uLi8uLi9yZXNv\n"+
    "dXJjZVRyZWUvYWtlbnRpQ2VydHMvIDE4MDAgNjQgesHsSCCeALlDtj1JQrzCycO8\n"+
    "kFI5169cvGTZGoXr592m4BPRtGa8oRQDEO2DD0HJqYL1yyd/GtaLNeAgS5ADtA==\n"+
    "-----END AKENTI POLICY CERTIFICATE-----";

  /**
    */
  public RemotePolicyEngineTestCase(String name) {
    super(name);
  }  

  /**
    *
    */
  public static Test suite() {
    return new TestSuite(RemotePolicyEngineTestCase.class);
  }

  /**
    */
  public void test1() throws Exception {
    AkentiPrincipal ap = new AkentiPrincipal("/CN=Joe", "/CN=Ca");
    CheckAccessRequest careq = new CheckAccessRequest(1, ap, "R2");
    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
 
    CheckAccessResponse resp = eng.process(careq);
    CapabilityCertificate cert = resp.getCapabilityCertificate();

    assertTrue(cert == null);
  }

  public void test2() throws Exception {
    AkentiPrincipal ap = new AkentiPrincipal("/C=US/OU=LBL/CN=User I", "/C=US/OU=LBL/CN=TESTCA IA");
    CheckAccessRequest careq = new CheckAccessRequest(2, ap, "R2");
    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
    
    CheckAccessResponse resp = eng.process(careq);
    CapabilityCertificate cert = resp.getCapabilityCertificate();

    assertTrue(cert != null);
  }

  public void test3() throws Exception {
    CheckPolicyRequest cpr = new CheckPolicyRequest(3, "R1");
    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);

    CheckPolicyResponse resp = eng.process(cpr);
  }

  public void test4() throws Exception {
    ResourceBrowseRequest rbreq1 = new ResourceBrowseRequest(1, "R1");
    ResourceBrowseRequest rbreq2 = new ResourceBrowseRequest(2, "R1");
    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
 
    ResourceBrowseResponse resp = eng.process(rbreq1);
    ResourceBrowseResponse resp2 = eng.process(rbreq2);
  }

  public void test5() throws Exception {
    CreateResourceRequest crreq = new CreateResourceRequest(5, "R1/S3");
    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
 
    CreateResourceResponse resp = eng.process(crreq);
  }

  public void test6() throws Exception {
    ByteArrayInputStream bais = new ByteArrayInputStream(polCertString.getBytes());
    AkentiCertificate cert = AkentiCertificateFactory.initCertFromStream("PEM", bais);
    UploadPolicyCertificateRequest 
      ucreq = new UploadPolicyCertificateRequest(200, (PolicyCertificate) cert);

    URL url = new URL("http://localhost:8444");
    RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
 
    UploadPolicyCertificateResponse resp = eng.process(ucreq);
  }

}
