/*
 * RemoteResourceBrowse.java
 *
 * Created on July 7, 2004, 3:22 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;

/**
 * example: RemoteResourceBrowse -F conf.txt -r R1/S1
 *   -F arg          - akenti client config file
 *   [-r arg]        - resource
 *   [-v]            - verbose
 *   [-h]            - prints this message
 *
 * @author Tommaso Sciortino
 */
public class RemoteResourceBrowse implements AkentiConstants {
  
  /**
   * Use this program to remotely browse a resource.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String resource = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'F', 'r', 'v', 'h'} );
      
      confFileName = parser.getOptionArg('F');
      resource = parser.getOptionArg('r');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      }
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      ConfFileParser.loadFromFile(confFileName);
     
      String host = System.getProperty("AkentiHost", AkentiHost);
      String sslMode = System.getProperty("SslMode", SslMode);
      int port = 0;
      URL url;
      
      if (sslMode.equals("on")) {
        SSLUtil.initSSL();
        port = new Integer(System.getProperty("AkentiPort", AkentiPortSSL)).intValue();
        url = new URL("https", host, port,"/");
      } else {
        port = new Integer(System.getProperty("AkentiPort", AkentiPort)).intValue();
        url = new URL("http", host, port,"/");
      }
      
      if (resource == null) {
        resource = "";
      }
      
      RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
      ResourceBrowseRequest req = new ResourceBrowseRequest(RemoteProtocolHandler.getID(), resource);
      ResourceBrowseResponse resp = null;
      
      try {
        resp = eng.process(req);
      } catch(Exception ex) {
        System.err.println("There was an error connecting to " + url.toString());
        System.exit(3);
      }
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      String[] resources = resp.getResources();
      for (int i=0; i < resources.length; i++) {
        System.out.println(resources[i]);
      }
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */
  static void printUsage() {
    System.err.println("example: RemoteResourceBrowse -F conf.txt -r R1/S1");
    System.err.println("options:");
    System.err.println("  -F arg      - akenti client config file");
    System.err.println("  [-r arg]    - resource");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}