/*
 * RemoteUploadPolCert.java
 *
 * Created on July 7, 2004, 3:22 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;

/**
 * example: RemoteUploadPolCert -F conf.txt -p polCert
 *   -F arg          - akenti client config file
 *   -p arg          - policy certificate file
 *   [-v]            - verbose
 *   [-h]            - prints this message
 *
 * @author Tommaso Sciortino
 */
public class RemoteUploadPolCert implements AkentiConstants {
  
  /**
   * Use this program to remotely uploads a policy certificate.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String polCertName = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'F', 'p', 'v', 'h'} );
      
      confFileName = parser.getOptionArg('F');
      polCertName = parser.getOptionArg('p');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      } else if (polCertName == null || polCertName.equals("")) {
        System.err.println("Must provide a Policy Certificate");
        System.exit(2);
      }
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      ConfFileParser.loadFromFile(confFileName);
     
      String host = System.getProperty("AkentiHost", AkentiHost);
      String sslMode = System.getProperty("SslMode", SslMode);
      int port = 0;
      URL url;
      
      if (sslMode.equals("on")) {
        SSLUtil.initSSL();
        port = new Integer(System.getProperty("AkentiPort", AkentiPortSSL)).intValue();
        url = new URL("https", host, port,"/");
      } else {
        port = new Integer(System.getProperty("AkentiPort", AkentiPort)).intValue();
        url = new URL("http", host, port,"/");
      }
      
      AkentiCertificate polCert = AkentiCertificateFactory.initCertFromFile(null,polCertName);
      if (!(polCert instanceof PolicyCertificate)) {
        System.err.println("Must provide a Certificate of type Policy");
        System.exit(2);
      }
      
      RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
      UploadPolicyCertificateRequest req = new UploadPolicyCertificateRequest(RemoteProtocolHandler.getID(), (PolicyCertificate) polCert);
      UploadPolicyCertificateResponse resp = null;
      
      try {
        resp = eng.process(req);
      } catch(Exception ex) {
        System.err.println("There was an error connecting to " + url.toString());
        System.exit(3);
      }
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: RemoteUploadPolCert -F conf.txt -r R1/S1");
    System.err.println("options:");
    System.err.println("  -F arg      - akenti client config file");
    System.err.println("  -p arg      - policy certificate file");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}