/*
 * ResourceBrowse.java
 *
 * Created on July 7, 2004, 3:22 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.protocolHandler.*;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;

/**
 * example: ResourceBrowse -F conf.txt -r R1/S1
 *   -f arg          - akenti client config file
 *   [-r arg]        - resource
 *   [-v]            - verbose
 *   [-h]            - prints this message
 *
 * @author Tommaso Sciortino
 */
public class ResourceBrowse {
  
  /**
   * Use this program to browse a resource.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String resource = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'f', 'r', 'v', 'h'} );
      
      confFileName = parser.getOptionArg('f');
      resource = parser.getOptionArg('r');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      }
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      
      ConfFileParser.loadFromFile(confFileName);
      
      if (resource == null) {
        resource = "";
      }
      
      LocalProtocolHandler eng = new LocalProtocolHandler();
      ResourceBrowseRequest req = new ResourceBrowseRequest(RemoteProtocolHandler.getID(), resource);
      ResourceBrowseResponse resp = eng.process(req);
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      String[] resources = resp.getResources();
      for (int i=0; i < resources.length; i++) {
        System.out.println(resources[i]);
      }
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: ResourceBrowse -f conf.txt -r R1/S1");
    System.err.println("options:");
    System.err.println("  -f arg      - akenti client config file");
    System.err.println("  [-r arg]    - resource");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}