/*
 */

package lbl.akenti.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Properties;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.P8KeyStore;
import lbl.akenti.util.P8Reader;
import lbl.akenti.util.X509CertReader;

public class SSLUtil implements AkentiConstants {
  
  public static void initSSL() throws GeneralSecurityException, IOException {
     P8Reader p8Reader = new P8Reader();
     
     String keyFileName = System.getProperty("ClientKeyFile");
    
     if (keyFileName == null) {
       throw new GeneralSecurityException("Must provide a private key in pkcs8");
     }

     p8Reader.readPrivateKey(keyFileName);

     if (p8Reader.getPrivateKey() == null) {
       throw new GeneralSecurityException("unable to read p8 key from " + keyFileName);
     }

      String certFileName = System.getProperty("ClientCertFile");

     if (certFileName == null) {
       throw new GeneralSecurityException("Must provide x509 client chain");
     }

     p8Reader.readX509Chain(certFileName);
     Certificate[] chain = p8Reader.getX509CertificateChain();

     if (chain.length == 0) {
       throw new GeneralSecurityException("No x509 client chain in " + certFileName);
     }

     KeyStore ks = new P8KeyStore();

     ks.load(null, null);

     ks.setKeyEntry(p8Reader.getSubjectDN().toString(),
                    p8Reader.getPrivateKey(),
                    null,
                    p8Reader.getX509CertificateChain());

     String casFileName = System.getProperty("CertificateAuthorityPath");

     if (casFileName == null) {
       casFileName = System.getProperty("CertificateAuthorityFile", CertificateAuthorityFile);
     } else {
       casFileName += "/";
       casFileName += System.getProperty("CertificateAuthorityFile", CertificateAuthorityFile);
     }

     X509Certificate[] caCerts = new X509CertReader().readX509Certs(casFileName);

     if (caCerts.length == 0) {
       throw new GeneralSecurityException("NO CAS FOUND in " + certFileName);
     }
 
     for (int i = 0; i < caCerts.length; ++i) {
       String alias = caCerts[i].getSubjectDN().toString();
 
       ks.setCertificateEntry(alias, caCerts[i]);
     }

     KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");
 
     kfac.init(ks, null);
 
     TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");
 
     tfac.init(ks);

     File f = new File(".rand");
     byte seed[] = null;

     try {
       if (f.exists()) {
         FileInputStream fis = new FileInputStream(f);
         byte temp[] = new byte[20];
 
         fis.read(temp);
         seed = temp;
       }
     } catch(IOException ex) {
       // DO NOTHING
     }

     SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
 
     if (seed == null) {
       seed = random.generateSeed(20);
 
       try {
         FileOutputStream fos = new FileOutputStream(f);
         fos.write(seed);
         fos.flush();
         fos.close();
       } catch(IOException ex) {
       }
     } else {
       random.setSeed(seed);
     }

     SSLContext ctx = SSLContext.getInstance("SSLv3");

     ctx.init(
       kfac.getKeyManagers(),
       tfac.getTrustManagers(),
       random
     );

     SSLSocketFactory sslFactory = ctx.getSocketFactory();
     HttpsURLConnection.setDefaultSSLSocketFactory(sslFactory);
 
     HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
       public boolean verify (String host, SSLSession s) {
         return true;
       }
     });
  }
}
