/*
 * ArgumentParser.java
 *
 * Created on July 1, 2004, 11:23 AM
 */

package lbl.akenti.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * This is a small program to parse arguments used in command line programs.
 *
 * @author Tommaso Sciortino
 */
public class ArgumentParser {
  private String[] argv;
  private Argument[] args;
  
  /**
   * Sets the arguments to be parsed.
   * @param argv the command line arguments
   */
  public ArgumentParser(String[] argv) {
    this.argv = argv;
  }
    
  /**
   * Parses the arguments and associates to each flag,
   * it's value if any.
   *
   * @param flag A char array consisting of legal flags.
   * @throws IOException throws this if there are any flags that are not recognized.
   */
  public void parseArgumentTokens(char[] flag) throws IOException {
    List temp = new ArrayList();
    
    for (int j=0; j < argv.length; j++) {
      if (!argv[j].startsWith("-")) {
        throw new IOException("'" + argv[j] + "' flag must start with a dash ('-')");
      } else if(argv[j].length() != 2) {
        throw new IOException("'" + argv[j] + "' flag not recognized");
      }
      
      boolean found = false;
      for (int i=0; i < flag.length; i++) {
        if (argv[j].charAt(1) == flag[i]) {
          if(j + 1 < argv.length && !argv[j+1].startsWith("-")) {
            j++;
            temp.add(new Argument(flag[i], argv[j]));
          } else {
            temp.add(new Argument(flag[i], ""));
          }
          found = true;
          break;
        }
      }
      
      if (!found) {
        throw new IOException("'" + argv[j] + "' flag not recognized");
      }
    }
    
    args = (Argument[]) temp.toArray(new Argument[temp.size()]);
  }
  
  /**
   * Gets the value of this flag. Returns null if flag is
   * not present. Returns emptystring if flag is present
   * but without an argument.
   * @param flag the flag
   * @return the value of the flag; null if there is none
   */
  public String getOptionArg(char flag) {
    for (int i=0; i < args.length; i++) {
      if (args[i].getFlag() == flag) {
        return args[i].getValue();
      }
    }
    return null;
  }
  
  /**
   * Gets the values of this flag. Returns multiple entries
   * if the user has made multiple invocation. Returns null
   * if flag is not present. Returns emptystring if flag is
   * present but without an argument.
   * @param flag the flag
   * @return the value of the flag; empty array if there is none
   */  
  public String[] getOptionArgs(char flag) {
    List temp = new ArrayList();
    for (int i=0; i < args.length; i++) {
      if (args[i].getFlag() == flag) {
        temp.add(args[i].getValue());
      }
    }
    return (String[]) temp.toArray(new String[temp.size()]);
  }
  
  /**
   * Returns true if the user invoked the flag.
   * @return true if the flag is present in the command line arguments
   * @param flag the flag
   */  
  public boolean  getOption(char flag) {
    return getOptionArg(flag) != null;
  }
  
  class Argument {
    private char flag;
    private String value;
    
    public Argument(char flag, String value){
      this.flag = flag;
      this.value = value;
    }
    
    public char getFlag(){
      return flag;
    }
    
    public String getValue(){
      return value;
    }
    
  }
  
}