package lbl.akenti.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This object reads a configuration file into a
 * CaseInsesitiveProperties object and sets the System
 * Properties  to it.
 */
public class ConfFileParser {
  static Map resourceMap = new HashMap();
  static String ROOT_RESOURCE_NAME = "RootResourceName";
  static FileInputStream fis;
  static File confFile;
  static Properties originalProps;
  static Properties props;
  
  
  /**
   * Loads a configuration file in from a file.
   * @param file the configuration file
   * @throws IOException if there are problems reading the file
   * @throws FileNotFoundException if the file is not found
   */  
  static public void loadFromFile(File file) throws IOException, FileNotFoundException {
    props = System.getProperties();
    
    if (originalProps == null) {
      originalProps = cloneSysProps();
    }
    
    props.clear();
    props.putAll(originalProps);
    props.setProperty("MD5withRSA", "RSA-MD5");
    props.setProperty("RSA-MD5", "MD5withRSA");
    confFile = file;
    
    parseRootResource();
    
    System.setProperties(new CaseInsensitiveProperties(props));
  }
  
  /**
   * Loads a configuration file in from a file.
   * @param filePath the configuration file path
   * @throws IOException if there are problems reading the file
   * @throws FileNotFoundException if the file is not found
   */  
  static public void loadFromFile(String filePath) throws IOException, FileNotFoundException {
    loadFromFile(new File(filePath));
  }
  
  static private Properties cloneSysProps() {
    Properties temp = new Properties();
    Iterator i = props.keySet().iterator();
    while (i.hasNext()) {
      Object key = i.next();
      Object value = props.get(key);
      temp.put(key, value);
    }
    
    return temp;
  }
  
  static private void parseRootResource() throws IOException {
    FileInputStream fis = new FileInputStream(confFile);
    BufferedReader br = new BufferedReader(new InputStreamReader(fis));
    
    resourceMap.clear();
    
    while (true) {
      String line = br.readLine();
      
      if (line == null) {
        break;
      }
      
      if (line.startsWith("#")) {
        continue;
      } else if (line.toLowerCase().startsWith(ROOT_RESOURCE_NAME.toLowerCase())) {
        StringTokenizer st = new StringTokenizer(line, " ", false);
        
        if (st.countTokens() >= 3) {
          st.nextToken();
          String resource = st.nextToken();
          StringBuffer path = new StringBuffer(st.nextToken());
          
          while (st.hasMoreTokens()) {
            path.append(' ');
            path.append(st.nextToken());
          }
          resourceMap.put(resource, new File(path.toString()));
        }
      } else {
        
        StringTokenizer st = new StringTokenizer(line, " ", false);
        if (st.countTokens() < 2) {
          continue;
        }
        
        String name = st.nextToken().trim();
        
        String value = line.substring(name.length()+1).trim();
        props.setProperty(name, value);
      }
    }
  }
  
  /**
   * Returns the local file path of the root resource.
   * @param res the root resource
   * @return the path
   */  
  static public String getResourcePath(String res) {
    File resource = (File) resourceMap.get(res);
    if (resource == null) {
      return "";
    } else {
      return resource.getAbsolutePath();
    }
  }
  
  /**
   * Returns all the root resources located at this file path.
   * @param path the path
   * @return the root resources
   */  
  static public String[] getPathResources(File path) {
    ArrayList temp = new ArrayList();
    Iterator i = resourceMap.keySet().iterator();
    while (i.hasNext()) {
      String key = (String) i.next();
      File element = (File) resourceMap.get(key);
      if (element.equals(path)) {
        temp.add(key);
      }
    }
    
    return (String[]) temp.toArray(new String[temp.size()]);
  }
  
  /**
   * Returns all root resources
   * @return all root resources
   */  
  static public String[] getResources() {
    ArrayList temp = new ArrayList();
    Iterator i = resourceMap.keySet().iterator();
    while (i.hasNext()) {
      String key = (String) i.next();
      temp.add(key);
    }
    
    return (String[]) temp.toArray(new String[temp.size()]);
  }
  
}