package lbl.akenti.policyManager.base;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;
import java.util.Collection;

public class ResFileInfo { 

  /** 
    * Used to hold the contents of the <code>.reattribute</code> file.
    *
    * @see ResFileParser
    * @see GenAttribute
    * @see GenCa
    * @see GenActions
    */
  private HashMap attributes;

  /** 
    * CaDns and their list of identity certificate publishing dirs.
    *
    * @see GenCa
    */
  private HashSet cas;     

  /**
    * A list of actions.
    *
    * @see GenActions
    */
  private GenActions actions;

  public ResFileInfo() {
    attributes = new HashMap();
    actions = new GenActions();
    cas = new HashSet();
  }
    
  public GenAttribute[] getX509GenAttributes() {
    Vector x509Attrs = new Vector();
    GenAttribute[] attrs = getGenAttributes();

    for (int i = 0; i < attrs.length; ++i) {
      if (attrs[i].isX509Attribute()) {
        x509Attrs.addElement(attrs[i]);
      }
    }

    attrs = new GenAttribute[x509Attrs.size()];
    x509Attrs.toArray(attrs);
    return attrs;
  }

  public boolean containsX509GenAttributes() {
    return getX509GenAttributes().length != 0; 
  }

  public GenAttribute getGenAttribute(String attrName) {
    return (GenAttribute) attributes.get(attrName.toLowerCase());
  }

  public GenAttribute[] getGenAttributes() {
    Collection col = attributes.values();
    GenAttribute [] attributes = new GenAttribute[col.size()];

    col.toArray(attributes);
    return attributes;
  }
  
  public void addGenAttribute(GenAttribute attribute) {
    String attrName = attribute.getName();
    GenAttribute attr = getGenAttribute(attrName);
    
    if (attr == null) {
      attributes.put(attrName.toLowerCase(), attribute);
      return;
    } 

    attr.addValues(attribute.getValues());
  }

  public void addGenAttributes(GenAttribute []attributes) {
    for (int i = 0; i < attributes.length; i++) {
      addGenAttribute(attributes[i]);
    }
  }
    
  public GenCa[] getGenCas() {
    GenCa [] temp = new GenCa[cas.size()];
    
    cas.toArray(temp);
    return temp;
  }

  public void addGenCas(GenCa[] cas) {
    for (int i = 0; i < cas.length; i++) {
      addGenCa(cas[i]);
    }
  }

  public void addGenCa(GenCa ca) {
    cas.add(ca);
  }

  public String[] getActions() {
    return actions.getActions();
  }

  public void addAction(String action) {
    actions.addAction(action);
  }

  public void addActions(String[] actions) {
    for (int i = 0; i < actions.length; i++) {
      addAction(actions[i]);
    }
  }
    
  public String toString() {
    return paramString();
  }

  public String paramString() {
    StringBuffer sb = new StringBuffer();

    sb.append('[');
    GenAttribute[] attrs = getGenAttributes();

    sb.append('[');

    if (attrs.length > 0) {
      sb.append(attrs[0].paramString());

      for (int i = 1; i < attrs.length; ++i) {
        sb.append(',');
        sb.append(attrs[i].paramString());
      }
    }

    sb.append(']');
    sb.append(cas.toString());
    sb.append(',');
    sb.append(actions.paramString());
    sb.append(']');
    return sb.toString();
  }
}
